;Supervco_slave.asm

; ****************************************************************************
; * Device type and options.                                                 *
; ****************************************************************************
;
	LIST P=16F628;, R=DEC 
	INCLUDE "p16f628.inc"
	 ERRORLEVEL -302	
 __FUSES _PWRTE_ON & _CP_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _LVP_OFF & _MCLRE_OFF & _BODEN_OFF

PortA   equ     0x05
PortB   equ     0x06

WREN    equ     0x02
WR      equ     0x01
RD      equ     0x00




; idlocat - specifies a default series of 4 ID locations, lower 4 bits used        
idlocat org     0x2000          ;starting address for ID locations
	__IDLOCS 0x003B         ;revision = 003B

	

; preload the 16F628 internal eeprom
	
		org	0x2100
;
	;DE	35		; Variable 00 EEPROM	; Adresse du Module
	;DE	'L'		; Variable 01 EEPROM	; Identifiant 1 du Module;
	;DE	'C'		; 	 ariable 02 EEPROM	; Identifiant 2 du Module
	;DE	'D'		; Variable 03 EEPROM	; Identifiant 3 du Module
        

;
; ****************************************************************************
; * RAM page independent file registers:                                     *
; ****************************************************************************
;
INDF    EQU     0x00
PCL     EQU     0x02
STATUS  EQU     0x03
FSR     EQU     0x04
PCLATH  EQU     0x0A
INTCON  EQU     0x0B
;
; *****************************************************************************
; * Bit numbers for the STATUS file register:                                 *
; *****************************************************************************
;
B_RP0   EQU     5
B_NTO   EQU     4
B_NPD   EQU     3
B_Z     EQU     2
B_DC    EQU     1
B_C     EQU     0
;
; ****************************************************************************
; * Assign names to IO pins.                                                 *
; ****************************************************************************


#DEFINE pll_data		PORTA,0         	; data sur ra0
#DEFINE pll_clk 		PORTA,7    		; clk sur ra7
#DEFINE pll_e 	  		PORTA,1    		; enable sur ra1

#DEFINE max_din 		PORTA,2    		; data sur ra2
#DEFINE max_clk 		PORTA,3    		; clk sur ra3
#DEFINE max_cs 	  		PORTA,4    		; enable sur ra4

#DEFINE sortie_plus		PORTB,4    		; + DE HF sur rb4
#DEFINE sortie_moins	        PORTB,6      	        ; - DE HF sur rb3


#DEFINE _scl 			PORTB,1     		; _m_scl sur RA2
#DEFINE _statut_scl 		TRISB,1     		; configuration de _scl en entre ou sortie

#DEFINE _sda			PORTB,0    		; _m_sda sur RA4
#DEFINE	_statut_sda		TRISB,0			; configuration de _sda en entre ou sortie

;--------------

; ****************************************************************************
; *           Allocate variables in general purpose register space           *
; ****************************************************************************
;
	CBLOCK  0x20              ; Start Data Block
	
	count
	timer1
	timer2
	m_o_byte
	m_n_bit
	m_cmpt1
	m_cmpt2
	temp
	adresse
	m_i_byte
	donne
	s_i_byte
	s_n_bit
	s_o_byte
	w_temp
	status_temp
	mod_adr
	
	
;        
	ENDC                      ; End of Data Block
;******************************************************************************        
; to avoid a lot of bank switching, put the osc stuff in the 70-7F area of
;       ram, which is accessable from any of the 4 banks
	CBLOCK  0x70              ; Start Data Block
	
	valeur_cna
	commande_hf

	ENDC                      ; End of Data Block
; 
; ****************************************************************************
; * The 16F84 resets to 0x00.                                                * 
; * The Interrupt vector is at 0x04. (Unused)                                *
; ****************************************************************************             
;
	ORG     0x0000                
reset_entry
	goto    start             ; Jump around the band table to main program
; 
;*****************************************************************************

;________________________________________________________________________________
;
; Gestionnaire d'interruptions
;________________________________________________________________________________
;
;============================
; Routine de traitement des interruptions
; Interrupt:

	org 4  				; dbut de l'adresse d'interruptions

;--------------               
; Sauvegarde des paramtres
; Interrupt_0:
	;--------------
	movwf 	w_temp 			; on sauve le registre W dans w_temp
	swapf 	STATUS,W       		; et le registre STATUS 
	movwf 	status_temp   		; dans status_temp

;--------------
; Test RB0
Interrupt_1:
	;--------------
	btfsc	INTCON,INTE		; Test si interruptions RB0 autorises
	btfss	INTCON,INTF		; on teste si il s'agit d'une interruption RB0
	goto	Interrupt_2

	goto	Interrupt_RB0		; oui, on traite

;--------------
; Test RB4..RB7
Interrupt_2:
	;--------------
	;btfsc	INTCON,RBIE		; Test si interruptions RB4..RB7 autorises
	;btfss	INTCON,RBIF		; on teste si il s'agit d'une interruption RB4..RB7
	goto	Interrupt_3

	;goto	Interrupt_RB4		; oui, on traite

;--------------
; Test TIMER
Interrupt_3:
	;--------------
	;btfsc	INTCON,T0IE		; Test si interruptions TIMER autorises
	;btfss	INTCON,T0IF		; on teste si il s'agit d'une interruption TIMER
	goto	Interrupt_Reset_All	; Il existe donc des forces obscures!

	;goto	Interrupt_Timer		; oui, on traite


;--------------
; Reset le flag d'interruption RB0
Interrupt_Reset_RB0:

	bcf 	INTCON,INTF    		; efface le FLAG d'interruption externe RB0
	bsf 	INTCON,INTE     	; autorise  nouveau l'interruption externe RB0

	goto	Interrupt_Restore

;--------------
; Reset le flag d'interruption RB4..RB7
Interrupt_Reset_RB4:

	;bcf 	INTCON,RBIF 		; efface le FLAG d'interruption externe RB4..RB7
	;bsf 	INTCON,RBIE     	; autorise  nouveau l'interruption externe RB4..RB7

	;goto	Interrupt_Restore

;--------------
; Reset le flag d'interruption TIMER
;Interrupt_Reset_TIMER:

	;bcf 	INTCON,T0IF		; efface le FLAG d'interruption TIMER
	;bsf 	INTCON,T0IE		; autorise  nouveau l'interruption TIMER

	;goto	Interrupt_Restore

;--------------
; Reset les flags d'interruption
Interrupt_Reset_All:

	bcf 	INTCON,INTF    		; efface le FLAG d'interruption externe RB0
	bsf 	INTCON,INTE     	; autorise  nouveau l'interruption externe RB0

	;bcf 	INTCON,RBIF 		; efface le FLAG d'interruption externe RB4..RB7
	;bsf 	INTCON,RBIE     	; autorise  nouveau l'interruption externe RB4..RB7

	;bcf 	INTCON,T0IF		; efface le FLAG d'interruption TIMER
	;bsf 	INTCON,T0IE		; autorise  nouveau l'interruption TIMER

;--------------
; Restaure les registres
Interrupt_Restore:

	;--------------
	swapf 	status_temp,W  		; restaure les valeurs originales des registres
	movwf 	STATUS         		; W et STATUS
	swapf 	w_temp,f
	swapf 	w_temp,W       	

	retfie                 		; fin de la routine d'interruption
;________________________________________________
;
; Routines d'interruptions
;________________________________________________
;
;============================
Interrupt_RB0:

	; Bus I2c

	;--------------
	; test _scl

	bcf 	STATUS,RP0     		; bank 0
	btfsc 	_scl      		; on test si _scl est haut

	goto 	I2c_Slave		; OUI, donc reconnaissance d'une condition de start
					; I2c_Slave se trouve dans l'include I2c_S.asm
	;--------------
	goto	Interrupt_Reset_RB0

;============================
Interrupt_RB4:
	;--------------
	; Capteurs

	;--------------
	goto	Interrupt_Reset_RB4



; *****************************************************************************
;
start
 	clrf    INTCON            ; No interrupts for now
	bsf     STATUS,B_RP0      ; Switch to bank 1
	;
	; Registre OPTION
	;--------------
	;b7 	Pull Up sur PORTB 			0 = ON				1 = OFF
	;b6	Sens Interrup sur RB0			0 = de 1 vers 0			1 = de 0 vers 1
	;b5	Fonctionnement TIMER 			0 = sur horloge int		1 =  partir de RA4
	;b4	Sens comptage sur RA4 (si b5=1)		0 = de 0 vers 1 (default)	1 = de 1 vers 0

	movlw	B'00000000'
	movwf	OPTION_REG

	;________________________________________________________________________
	;
	; Registre INTCON
	;--------------
	; interruptions
	; b7	GIE	Global Interrupt		0 = OFF		1 = ON
	; b6	EEIE	EEPROM (fin d'criture)		0 =		1 =
	; b5	TOIE	TIMER				0 = OFF 	1 = ON	
	; b4	INTE	RB0 (sens dans OPTION)		0 = OFF		1 = ON
	; b3	RBIE	RB4..RB7			0 = OFF		1 = ON
	;--------------
	; flags
	; b2	TOIF	TIMER 		Flag Bit	0 = NON		1 = OUI (dpassement du TMR0)
	; b1	INTF	RB0 		Flag Bit	0 = NON		1 = OUI (modif de RB0 dans le sens dfinit dans OPTION)
	; b0	RBIF	RB4..RB7 	Flag Bit	0 = NON		1 = OUI (modif d'une des broches RB4..RB7)

	movlw	B'10010000'   ; org B'10111000'
	movwf	INTCON

	;________________________________________________________________________
	;

	bsf     STATUS,B_RP0      ; Switch to bank 1
	movlw   0x20      ; Tristate port  en out sau A5 EN IN
	movwf   TRISA
	movlw	0x73 ;
	movwf   TRISB            ; Set port B 

	bcf     STATUS,B_RP0      ; Switch back to bank 0

;        
	;added the next 2 statements for the 16f628
	movlw   b'00000111'     ;turn off the
	movwf   CMCON           ;    analog comparator pins
	movlw	0x60
	movwf	mod_adr		;adresse pu pic esclave

	bsf	max_cs		;mettre cs du max522 a 1
	call	init_slave



;*********************charger le programme pour le mb1504************************
	; Le pll est command a la place de son quartz par un dds le diviseur du pll  R = 250 la ref est donc 
;de  5 kcs 
; le nombre de pas des diviseurs  A et N est fix a 2500 pas
; si bien en somme la frequence envoy par le dds est mulitipli par 10 
; a la sortie pll/vco par ex dds =1.25 mgh sortie pll/vco 12.5 mgh
	
;************************programmation du mb1501 (referece) ******************************
;Les valeur donnnes des diviseurs sont valable pour le mb1501 et le  mb1504
; la seul chose a changer est le prediviseur pour 1501 on mettera un 1 pour le 1504 il faudra mettre un 0

;*************************envoie prediviseur sw = 1 	********************************
;prediviseur pour le 1501 1 =/ 64 0 = / 128
;  " "         "  le 1504 0 =/ 64  1 = / 32
	bsf	pll_data 		; prediviseur = 1 le pll est un mb1501	
	NOP
	bsf	pll_clk
	nop
	bcf	pll_clk
	nop
	bcf	pll_data
;**********************envoie des 6 bits apartir du msb*****************
;compteur R de 14 bits j'envoie a partir du msb 6 bits puis 8 bits soit 000000 11111010

	movlw	0x06
	movwf	timer2
	movlw	0x00
	movwf	temp
	call	envoie_octet
;*********************envoie des 8 autre bits***************************

	movlw	0x08
	movwf	timer2
	movlw	0xfa
	movwf	temp
	call	envoie_octet
;*******************envoie de bit "c" = 1 ******************************
	nop
	bsf	pll_data
	nop
	bsf	pll_clk
	nop
	bcf	pll_clk
;*****************envoie de enable *************************************

	nop
	bsf	pll_e
	nop
	nop
	bcf	pll_e
	nop
	nop
	bcf	pll_data
	nop
	nop
	
;************************programmation du mb1504 (diviseur) ******************************	
;compteur A ET N soit 18 bits que j'ai divis en trois partie j'envoie  du pic vers le pll en partant
 ;de msb 3 bits puis 8bits et enfin 7bits ce qui donne 000 00100111 0001000

	movlw	0x03
	movwf	timer2
	movlw	0X00
	movwf	temp
	call	envoie_octet

	movlw	0x08	;
	movwf	timer2
	movlw	0X27
	movwf	temp
	call	envoie_octet
	
;******compteur a

	movlw	0x07	;
	movwf	timer2
	movlw	0X08	; normalement c'est 0x04 qui est donn par le calcul ,mais avec 0x08 ca marche ???
	movwf	temp
	call	envoie_octet
	
;envoie du bit de control =0

	bcf	pll_data
	nop
	bsf	pll_clk
	nop
	bcf	pll_clk
	nop
	
;envoie de enable

	bsf	pll_e
	nop
	nop
	bcf	pll_e
	nop
	nop
	goto 	main

;*************************envoie octet**********************************

envoie_octet 
	 
		
	       BCF     pll_clk
	       nop
decal_gauche
	
		btfsc   temp,7   
		GOTO    _HIGH
		BCF     pll_data
		GOTO    _LOW
       
_HIGH           
		BSF     pll_data
_LOW         	nop
		BSF     pll_clk
		nop
		BCF     pll_clk
		nop
		rlf      temp,f
		DECFSZ  timer2,f
		GOTO    decal_gauche  ;LE CONTENU DE TIMER2 EST
				      ;DECREMENTE DE 1 SI RESULTAT DIFFERENT
				      ;DE ZERO SAUTE A DECAL_GAUCHE

				    
		BCF     pll_clk
	
		return

;*****************************envoie_max******************************************


envoie_max
	 
		
	       BCF     max_clk
	       nop
decal_max
	
		btfsc   temp,7   
		GOTO    _Haut
		BCF     max_din
		GOTO    _Las
       
_Haut           
		BSF     max_din
_Las         	nop
		BSF     max_clk
		nop
		BCF     max_clk
		nop
		rlf      temp,f
		DECFSZ  timer2,f
		GOTO    decal_max  ;LE CONTENU DE TIMER2 EST
				      ;DECREMENTE DE 1 SI RESULTAT DIFFERENT
				      ;DE ZERO SAUTE A DECAL_GAUCHE

				    
		BCF     max_clk
	
		return

; 
; Dominique M. - 9/2003
;________________________________________________________________________________
;
main


	btfss	sortie_plus	        ;plus hf rb4
	goto	plus_cna
	
	btfss	sortie_moins		;moins hf rb3	
	goto	moins_cna

	btfss	PortB,5
	call	write_cna	; mettre en memoire la valeur regl par plus_cna ou moins_cna 


	goto 	main

;********************************************************************************
plus_cna
	CALL	I2c_Tempo_Long
	movlw 	0XFF			; on teste 
	subwf	commande_hf	,W	; commande_hf - ff -> W
	btfsc	STATUS,Z		; test
	goto 	main  			; LE NCA EST A 5 volt

	incf	commande_hf,f		
	call	max522	
	goto 	main
;***********************************************************************
moins_cna
	CALL	I2c_Tempo_Long
	movlw 	0X00			; on teste si 
	subwf	commande_hf	,W	; commande_hf - 00 -> W
	btfsc	STATUS,Z		; test
	goto 	main  			; LE NCA EST A 0 volt
	
	decf	commande_hf,f			
	call	max522		
	goto 	main

;********************************MAX 522**************************************
max522
		

	movlw	0x08
	movwf	timer2

	movlw	0x21 		;config du max522 " sortie A active"
	movwf	temp
	bcf	max_cs
	call	envoie_max

	movlw	0x08
	movwf	timer2
	
	;exemple ;calcul 5v x 128/256 =2v050 en out
	movfw	commande_hf		
 	movwf	temp
	call	envoie_max
	bsf	max_cs
	return
	
;________________________________________________________________________________
;
; Routines gnrale I2c (mode Slave)     	    
;________________________________________________________________________________
;
;============================
; Lecture d'un octect sur le Bus.
; La donne lue sera dans s_i_byte.

I2c_Slave_In_Byte:
	;--------------
	clrf 	s_i_byte         	; on vide le tampon d'entre s_i_byte

	movlw 	8	       		; on initialise l'index  8 bits
	movwf 	s_n_bit			; on charge dans s_n_bit

;--------------	
I2c_Slave_In_Bit:  
	;--------------
	btfsc 	_scl      		; on attend que _scl soit BAS
	goto 	I2c_Slave_In_Bit	; on boucle

;--------------
I2c_Slave_In_Bit_Wait:
	;--------------
	btfss 	_scl      		; on attend que _scl soit HAUT
	goto 	I2c_Slave_In_Bit_Wait   ; on boucle
 
	;--------------
	btfss 	_sda      		; on lit la donne prsente sur _sda en testant son tat
	goto	I2c_Slave_In_Bit_0	
	goto	I2c_Slave_In_Bit_1

I2c_Slave_In_Bit_0:
	;--------------
	bcf 	STATUS,C      		; on met la retenue  zro
	rlf 	s_i_byte,f       	; on dcale  gauche s_i_byte (le bit est  zro) 
 
	goto	I2c_Slave_In_Bit_Next

I2c_Slave_In_Bit_1:
	;--------------
	bcf 	STATUS,C      		; on met la retenue  zro
	rlf 	s_i_byte,f       	; on dcale  gauche s_i_byte (le bit est  zro) 
	bsf 	s_i_byte,0       	; _sda est  un, on met le bit de poid faible  un

I2c_Slave_In_Bit_Next:
	;--------------
	decfsz 	s_n_bit,f          	; on dcremente l'index
	goto 	I2c_Slave_In_Bit        ; on lit le bit suivant tant que l'index n'est pas 0

	;--------------
	return
;***********************************************************************************************
init_slave
	; On configure _sda en entre

	bsf 	STATUS,RP0      	; bank 1
	bsf 	_statut_sda    ; _sda est une entre valant 5V (Pull Up sur _sda)
	bcf 	STATUS,RP0      	; bank 0
;______________________________________

	; On configure _scl en entre

	bsf 	STATUS,RP0      	; bank 1
	bsf 	_statut_scl    ; _scl est une entre valant 5V (Pull Up sur _scl)
	bcf 	STATUS,RP0      	; bank 0
	return
;****************************************************************************************
;
; Envoi d'un octect sur le Bus.

I2c_Slave_Out_Byte:
	;--------------
	movwf	s_o_byte		; on charge W dans s_o_byte

	movlw 	8	        	; on initialise l'index  8 bits
	movwf 	s_n_bit			; on charge dans s_n_bit

;--------------
I2c_Slave_Out_Bit:  
	;--------------
	btfsc 	_scl 			; on attend que _scl soit BAS
	goto 	I2c_Slave_Out_Bit	; on boucle

	nop
	nop

	;--------------
	; On configure _sda en entre

	bsf 	STATUS,RP0      	; bank 1
	bsf 	_statut_sda      	; _sda est une entre valant 5V (Pull Up sur _sda)
	bcf 	STATUS,RP0      	; bank 0

;--------------
I2c_Slave_Out_Bit_Wait:

   	;--------------
	; On lit le bit suivant de s_o_byte et on le teste

	bcf 	STATUS,C   
	rlf 	s_o_byte,f	       	; on dcale o_byte  gauche via la retenue

	btfsc 	STATUS,C        	; on teste la retenue
	goto	I2c_Slave_Out_Bit_Next	; c'est un 1 donc on ne touche pas  la ligne

	;--------------
	; C'est un zro, on configure _sda en sortie

	bcf 	_sda       		; et on force _sda  zro

	bsf 	STATUS,RP0      	; bank 1
	bcf 	_statut_sda      	; _sda est une sortie valant 0V (Pull Up sur _sda)
	bcf 	STATUS,RP0      	; bank 0

;--------------
I2c_Slave_Out_Bit_Next:
	;--------------
	btfss 	_scl       		; on attend que _scl soit HAUT
	goto 	I2c_Slave_Out_Bit_Next  ; on boucle

	decfsz 	s_n_bit,f            	; on dcremente l'index
	goto 	I2c_Slave_Out_Bit     	; on envoie le bit suivant tant que l'index n'est pas  0

;--------------
I2c_Slave_Out_Byte_End:
	;--------------
	btfsc 	_scl 			; on attend que _scl soit BAS
	goto 	I2c_Slave_Out_Byte_End	; on boucle

	nop
	nop

	;--------------
	; On configure _sda en entre

	bsf 	STATUS,RP0      	; bank 1
	bsf 	_statut_sda      	; SDA est une entre valant 5V (Pull Up sur _sda)
	bcf 	STATUS,RP0      	; bank 0

	;--------------
	return

;============================
; Envoi d'un ACK sur le Bus (sur le neuvime coup d'horloge)

I2c_Slave_Ack:
	;--------------
	btfsc 	_scl			; on attend que _scl soit BAS
	goto 	I2c_Slave_Ack   	; on boucle          	

;--------------	
I2c_Slave_Ack_Wait1:  
	;--------------
	; Le matre a laisser _sda  l'tat HAUT, on va donc le forcer  l'tat BAS.

	bcf 	_sda       		; on met _sda  zro

	;--------------
	bsf 	STATUS,RP0      	; bank 1
	bcf 	_statut_sda      	; _sda est une sortie valant 0V (Pull Up sur _sda)
	bcf 	STATUS,RP0      	; bank 0

;--------------	
I2c_Slave_Ack_Wait2:
	btfss 	_scl       		; on attend que _scl soit HAUT (neuvime coup d'horloge)
	goto 	I2c_Slave_Ack_Wait2	; on boucle

;--------------	
I2c_Slave_Ack_Wait3:  
	;--------------
	btfsc 	_scl			; on attend que _scl soit BAS	
	goto 	I2c_Slave_Ack_Wait3 	; on boucle
	
	nop
	nop

	;--------------		
	bsf 	STATUS,RP0      	; bank 1
	bsf 	_statut_sda      	; _sda est une entre valant 5V (Pull Up sur _sda)
	bcf 	STATUS,RP0      	; bank 0

	;--------------
	return        

;============================
; N'envoie PAS de ACK sur le Bus (sur le neuvime coup d'horloge)


I2c_Slave_No_Ack:
	;--------------
	btfsc 	_scl			; on attend que _scl soit BAS
	goto 	I2c_Slave_No_Ack 	; on boucle

;--------------	
I2c_Slave_No_Ack_Wait1:  
	;--------------
	btfss 	_scl      		; on attend que _scl soit HAUT
	goto 	I2c_Slave_No_Ack_Wait1  ; on boucle

	; on laisse _sda au niveau HAUT

;--------------		
I2c_Slave_No_Ack_Wait2: 
	;-------------- 
	btfsc 	_scl			; on attent que _scl soit BAS
	goto 	I2c_Slave_No_Ack_Wait2	; on boucle

	;--------------
	return               

;============================
I2c_Slave_Fin_No_Ack:
	;--------------
	bsf 	STATUS,RP0      	; bank 1
	bsf 	_statut_sda      	; _sda est une entre valant 5V (Pull Up sur _sda)
	bcf 	STATUS,RP0      	; bank 0

	call	I2c_Slave_No_Ack

	goto 	Interrupt_Reset_All    ; on termine en retournant au gestionnaire d'interruptions

;============================
I2c_Slave_Fin:
	;--------------
	bsf     STATUS,B_RP0      ;valeur_cna = s_i_byte,W = numero de la bande (envoy par le pic 
	movf	valeur_cna,W      ;maitre) lire la valeur en eeprom et la mettre dans le max522   
	movwf   EEADR             ;lors d'un changement de bande 
	call	read_EEPROM
	bcf     STATUS,B_RP0 
	movwf	commande_hf
	CALL	max522

	bsf 	STATUS,RP0        ; bank 1
	bsf 	_statut_sda      ; _sda est une entre valant 5V (Pull Up sur _sda)
	bcf 	STATUS,RP0     	 ; bank 0

	nop

	goto 	Interrupt_Reset_All    ; on termine en retournant au gestionnaire d'interruptions

;________________________________________________________________________________
;
; Routines de dbut de trame
;________________________________________________________________________________

;============================
I2c_Slave:   
 	;--------------
	bcf 	STATUS,RP0     		; bank 0

	;--------------
	; lecture adresse

	call 	I2c_Slave_In_Byte    	; lecture de l'adresse

	movf	s_i_byte,W		; on charge s_i_byte dans W	
	subwf	mod_adr,W		; mod_add - s_i_byte -> W
	btfss	STATUS,Z		; test
	goto 	I2c_Slave_Fin_No_Ack	; la transmition actuelle ne nous concerne pas
	
	call 	I2c_Slave_Ack           ; Ok l'adresse est la bonne!

	; Insrer ici ventuellement une sous adresse

	;--------------
	
	;lecture du numero de la bande (0 a 7)et la mettre dans valeur_cna
	;
	call 	I2c_Slave_In_Byte       ; lecture du numro du numero de la bande (0a7)
	movf	s_i_byte,W		; contenue dans s_i_byte
	movwf	valeur_cna		; et la mettre dans valeur_cna
	call 	I2c_Slave_Ack		; et on acquitte

	; lecture du numro de fonction demande et branchement
	;
	movlw 	0			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function1	; c'est la fonction X

	;--------------
	movlw 	1			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function2	; c'est la fonction X

	;--------------
	movlw 	2			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function3	; c'est la fonction X

	;--------------
	movlw 	3			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function4	; c'est la fonction X

	;--------------
	movlw 	4			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function5	; c'est la fonction 
	
		;--------------
	movlw 	5			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function6	; c'est la fonction X
	
	;--------------
	movlw 	6			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function7	; c'est la fonction X
	
		;--------------
	movlw 	7			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function8	; c'est la fonction X
	

	
		;--------------
	movlw 	8			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function9	; c'est la fonction X
	
	
		;--------------
	movlw 	9			; on teste si function X
	subwf	s_i_byte,W		; s_i_byte - Num function -> W
	btfsc	STATUS,Z		; test
	goto 	I2c_Slave_Function10	; c'est la fonction X
	
	
	; On peut rajouter jusqu' 255 fonctions, aprs ce n'est qu'un problme de place en mmoire programme!



	;--------------
	goto 	I2c_Slave_Fin 
;****************************************************************************************

;les diodes pins sont conductrices par le portb3et7 et le porta6 elles ajoutent ou enlevent des capa sur le vco
;
;******************************************************************************
I2c_Slave_Function1 ;BANDE 3.5
	;-------------

	bsf	PORTB,3
	bcf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin
	
;*******************************************************************************

	;
I2c_Slave_Function2  ;.BANDE 7
	;-------------
	

	bcf	PORTB,7
	bcf	PORTB,3
	bcf	PORTA,6
	goto 	I2c_Slave_Fin
	
;********************************************************************************
;
I2c_Slave_Function3   ;BANDE 10.1
	;-------------


	bCf	PORTB,3
	bcf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin
	
;********************************************************************************
;
I2c_Slave_Function4	;BANDE 14
	;-------------



	bcf	PORTB,3
	bcf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin
	
	
;********************************************************************************
;
I2c_Slave_Function5	 ;BANDE 18.068
	;-------------


	bCf	PORTB,3
	bcf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin	
;********************************************************************************
;
I2c_Slave_Function6	;BANDE 21
	;-------------

	

	bcf	PORTB,3
	bcf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin	
	

;********************************************************************************
;
I2c_Slave_Function7	;BANDE 24
	;-------------

	

	bCf	PORTB,3
	bcf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin	

;********************************************************************************
;
I2c_Slave_Function8	;BANDE 28
	;-------------


	bCf	PORTB,3
	bCf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin
	
;********************************************************************************
;
I2c_Slave_Function9	;BANDE 50
	;-------------


	bCf	PORTB,3
	bCf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin
	
;******************************************************************************
I2c_Slave_Function10 ;BANDE 1.8
	;-------------

	bsf	PORTB,3
	bcf	PORTB,7
	bcf	PORTA,6
	goto 	I2c_Slave_Fin
	
			
	
;******************************************************************************
; Routines Tempo
;________________________________________________________________________________


; 
I2c_Tempo_Long:
	;--------------
	movlw	0xff
	movwf	m_cmpt1
	movlw	0x40
	movwf	m_cmpt2


I2c_Tempo_Long_WAIT1:
	;--------------
	nop

	decfsz	m_cmpt1,f
	goto 	I2c_Tempo_Long_WAIT1
	decfsz	m_cmpt2,f
	goto 	I2c_Tempo_Long_WAIT1
	nop

	;--------------
	return

;============================
; Une pause de 20s

I2c_Tempo_Short:
	;--------------
	movlw	3		
	movwf	m_cmpt1

I2c_Tempo_Short_WAIT:
	;--------------
	nop

	decfsz	m_cmpt1,f
	goto 	I2c_Tempo_Short_WAIT

	nop
	nop
	nop

	;--------------
	return

; *****************************************************************************
; *                                                                           *
; * Purpose:  Write the byte of data at EEDATA to the EEPROM at address       *
; *           EEADR.                                                          *
; *                                                                           *
; *   Input:  The values at EEDATA and EEADR. doivent etre en w                                 *
; *                                                                           *
; *  Output:  The EEPROM value is updated.                                    *
; *                                                                           *
; *****************************************************************************
;
write_EEPROM
				  ;assume we are already in bank 1
	bcf	INTCON,GIE	  ; interruptions interdites
       bsf     STATUS,B_RP0       ; Switch to bank 1
	bsf     EECON1,WREN       ; Set the EEPROM write enable bit
	movlw   0x55              ; Write 0x55 and 0xAA to EEPROM
	movwf   EECON2            ;   control register, as required
	movlw   0xAA              ;   for the write
	movwf   EECON2            ; 
	bsf     EECON1,WR         ; Set WR to initiate write
bit_checko
	btfsc   EECON1,WR         ; Has the write completed?
	goto    bit_checko        ; No, keep checking
	bcf     EECON1,WREN       ; Clear the EEPROM write enable bit
	;incf    EEADR,f          ; Increment the EE write address
				  ; let the caller switch back to bank 0
	bsf	INTCON,GIE	  ; interruptions autorises
       bcf     STATUS,B_RP0       ; Switch to bank 0
       
	return                    ; Return to the caller
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Read a byte of EEPROM data at address EEADR into EEDATA.        *
; *                                                                           *
; *   Input:  The address EEADR.                                              *
; *                                                                           *
; *  Output:  The value in EEDATA.                                            *
; *                                                                           *
; *****************************************************************************
;
read_EEPROM
;assume we are in bank 1 already
;        bsf     STATUS,B_RP0      ; Switch to bank 1
	bsf     EECON1,RD         ; Request the read
; EEADR is in bank 1 in the 16F628
	incf    EEADR,f           ; Increment the read address
;let the calling routine switch back to bank 0
;        bcf     STATUS,B_RP0      ; Switch to bank 0
	movf    EEDATA,w          ; get the eeprom data byte, place in w
	return                    ; Return to the caller

;****************************************************************************
;
;ecrire la valeur dans l'eeprom
;valeur_cna  = numero de la bande
;commande_hf  = valeur pour le max522
;
;****************************************************************************
write_cna
	bsf		PORTB,2	 	;led a 1 la prise en compte du reglage du cna
 	call	I2c_Tempo_Long
	bcf		PORTB,2		;led a zero
	; 
	bsf     STATUS,B_RP0      ;  
	movf	valeur_cna,w      ;adresse ou ecrire
	movwf   EEADR             ; Write final value to EEPROM 	
 	movf	commande_hf,w	  ;VALEUR A ECRIRE	
	movwf   EEDATA  
    	
	call	write_EEPROM
	bcf     STATUS,B_RP0 	
	
	return

;*****************************************************************************
	end
