; ****************************************************************************
; * Direct Digital Synthesiser Controller		                     *
; *  Version 877_4_1a	                                                     *
; *  December    2002 
				                           	          
; ****************************************************************************
; Description:
; This is the control program for a DDS VFO controller to drive an AD9850 
; DDS chip, a 512 step/rev shaft encoder, hardware is minimal.
; It is adapted from software written by NJ-QRP 
; Club members. See author details below.
; 
;
; BAND MEMORIES an external push button switch allows the frequency to be 
; cycled around the HF ham bands.
;
; CALIBRATE MODE is entered if the external push button is pressed during
; power on. The display is set to 10,000.000 CAL and remains fixed, even as
; adjustments are being made. If the push button is held pressed, then turning
; the shaft encoder will increase or decrease the value "osc" used to 
; calculate the DDS control word. The basic calibrate adjustment rate is very
; low (on the order of a few cycles per turn of the encoder).  A somewhat
; faster adjustment speed is available by pressing the encoder shaft down
; while turning.  An external frequency counter on the DDS output is required
; to observe this adjustment. To exit calibrate mode, release the external
; push button and turn the shaft encoder one more time. The calibrated value
; of "osc" will then be stored in EEPROM memory.
;
; IF OFFSET is added or subtracted depending upon state of RA4. This operates 
; on the DDS data to shift the frequency leaving the display frequency constant.
; Two offset values, IF_Add and IF_Sub allow a BFO offset to be programmed.
; This feature was originally intended for use with ladder filters which
; favour LSB operation. By switching the LO injection USB and LSB operation
; is possible. Ron Taylor G4GXO October 1999
;
; An S-Meter feature provides a linear bar graph on line 2 representing an
; analogue voltage input (0-5v max) to the A to D.
;
; RIT function
;
; 2b changes;	Band control for filter selection
;	 	Store last used band and use on start up
;		DDS Data and Clock commoned to save pins
;		Port changes for more efficient use of pins *Note Pin Connections!!*
;
; 2c Changes;   Correct T/R comment on pin-out and port bit assignmentto show 
;		correct logic.
;		Added start up delay to allow control line states to settle before
;		main progrsm entry.
;		Added version number, displayed during start up delay. 
;		Inserted 64mSec 'debounce' delay infront of CALLXs to control functions
;		in Main. This allows any transients to settle before switch states
;		are read preventing unreliable operation.		 		
;
;******************************************************************************
;
; Original Author details
; Author - Curtis W. Preuss - WB2V
;		Bruce Stough, AA0ED
;		Craig Johnson, AA0ZZ 
;                            
;*****************************************************************************
;
; Target Controller -      PIC16F877
;                          
;Note. 
;
; 

;
; Processor Port Connections
;
; 
;
;	Port A
;		0	S-Meter Analogue Input (0 - 5V)
;		1	AGC Threshold Reference (0 - 5V)
;		2	
;		3	Spare Analogue Input for later Tx Power feature
;		4	
;		5	
;
;	Port B
;		0		 Shaft Encoder A
;		1		Shaft Encoder B
;		2		memoire a l'arret du trx
;		3		ptt
;		4		clavier col
;		5			"
;		6			"
;		7			"
;
;	Port C
;		0	clavier lignes	
;		1		"
;		2		"
;		3	i2c (scl)	
;		4	i2c	(sda)	
;		5	clavier lignes
;		6		"	
;		7		;		"	
;
;	Port D
;		0	 Bootloader Tx (actif a 1)
;		1	LCD_e           ; LCD 0=disable, 1=enable
;		2	LCD_rw			; LCD 0=write, 1=read
;		3	LCD_rs			; LCD 0=instruction, 1=data
;		4   LCD	D4         
;		5	LCD	D5               
;		6	LCD	D6 
;		7	LCD	D7 
;
;	Port E
;		0	LCD_rs/DDS_clk ; AD9850 write cloc
;		1	LCD_rw/DDS_dat ; AD9850 serial data input
;		2	DDS_load equ    ; Update pin on AD9850
;

; ****************************************************************************
; * General equates.  These may be changed to accommodate the reference clock* 
; * frequency, the desired upper frequency limit, and the default startup    *
; * frequency.                                                               *
; ****************************************************************************
;
; ref_osc represents the change in the frequency control word which results 
; in a 1 Hz change in output frequency.  It is interpreted as a fixed point
; integer in the format <ref_osc_3>.<ref_osc_2><ref_osc_1><ref_osc_0>
;
; The values for common oscillator frequencies are as follows:
;
; Frequency    ref_osc_3    ref_osc_2    ref_osc_1    ref_osc_0
; 125.00 MHz     0x22         0x5C         0x17         0xD0
; 120.00 MHz     0x23         0xCA         0x98         0xCE                  
; 100.00 MHz     0x2A         0xF3         0x1D         0xC4                  
;  90.70 MHz     0x2F         0x5A         0x82         0x7A                  
;  66.66 MHz     0x40         0x6E         0x52         0xE7                  
;  66.00 MHz     0x41         0x13         0x44         0x5F                  
;  50.00 MHz     0x55         0xE6         0x3B         0x88                  
;
; To calculate other values: 
;    ref_osc_3 = (2^32 / oscillator_freq_in_Hertz).                           
;    ref_osc_2, ref_osc_1, and ref_osc_0 are the fractional part of           
;     (2^32 / oscillator_freq_in_Hertz) times 2^24.                           
;    Note:   2^32 = 4294967296 and 2^24 = 16777216
;
; For example, for a 120 MHz clock:
;    ref_osc_3 is (2^32 / 120 x 10^6) = 35.791394133 truncated to 35 (0x23)  
;    ref_osc_2 is the high byte of (.791394133 x 2^24) = 13277390.32         
;      13277390.32 = 0xCA98CE, so high byte is CA.                           
;    ref_osc_1 is the next byte of 0xCA98CE, or 98
;    ref_osc_0 is the last byte of 0xCA98CE, or CE
;
;==== Currently set for 125 MHz Oscillator =======
;ref_osc_3   equ 0x22              ; Most significant osc byte
;ref_osc_2   equ 0x5C              ; Next byte
;ref_osc_1   equ 0x17              ; Next byte
;ref_osc_0   equ 0xD0              ; Least significant byte
;;==== Currently set for 100MHz Oscillator =======
ref_osc_3   equ 0x2A              ; Most significant osc byte
ref_osc_2   equ 0xF3             ; Next byte
ref_osc_1   equ 0x1D              ; Next byte
ref_osc_0   equ 0xC4              ; Least significant byte
;
;;==== Currently set for 120MHz Oscillator =======
;ref_osc_3   equ 0x23              ; Most significant osc byte
;ref_osc_2   equ 0xCA              ; Next byte
;ref_osc_1   equ 0x98              ; Next byte
;ref_osc_0   equ 0xCE              ; Least significant byte
;==== Currently set for 50 MHz Oscillator =======
;ref_osc_3   equ 0x55              ; Most significant osc byte
;ref_osc_2   equ 0xE6              ; Next byte
;ref_osc_1   equ 0x3B              ; Next byte
;ref_osc_0   equ 0x88              ; Least significant byte

;******************POUR UN OSC DE 80 Mgh*************************** 
;ref_osc_3   equ 0x35              ; Most significant osc byte
;ref_osc_2   equ 0xAF              ; Next byte
;ref_osc_1   equ 0xE5              ; Next byte
;ref_osc_0   equ 0x35              ; Least significant byte
;***************************************************************************
; ces data seront a changer si on prend une autre valeur de FI

;IF_Sub_3 equ 0x00	;MSb  1.105.951
;IF_Sub_2 equ 0x10
;IF_Sub_1 equ 0xe0
;IF_Sub_0 equ 0x1f	;LSb
;
;       
;IF_Add_3 equ 0x00	;MSb =1.105.651 mgh
;IF_Add_2 equ 0x10
;IF_Add_1 equ 0xde
;IF_Add_0 equ 0xf3	;LSb
; ********************************************************************

; 

band_end equ    d'36';      ; The offset to the last band table entry 
;;

; ****************************************************************************
; * Device type and options.                                                 *
; ****************************************************************************
;
;        processor       PIC16F877
        LIST            P=16F877
        	;#include <p16F877.inc>        ; fichier include

        radix           dec
        errorlevel-302
;
; ****************************************************************************
; * Configuration fuse information:

_CP_ALL                      EQU     H'0FCF'
_CP_HALF                     EQU     H'1FDF'
_CP_UPPER_256                EQU     H'2FEF'
_CP_OFF                      EQU     H'3FFF'
_DEBUG_ON                    EQU     H'37FF'
_DEBUG_OFF                   EQU     H'3FFF'
_WRT_ENABLE_ON               EQU     H'3FFF'
_WRT_ENABLE_OFF              EQU     H'3DFF'
_CPD_ON                      EQU     H'3EFF'
_CPD_OFF                     EQU     H'3FFF'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LP_OSC                      EQU     H'3FFC'
_XT_OSC                      EQU     H'3FFD'
_HS_OSC                      EQU     H'3FFE'
_RC_OSC                      EQU     H'3FFF'
                                          
; ****************************************************************************



       

		__CONFIG   _CP_OFF & _DEBUG_OFF & _WRT_ENABLE_ON & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC 

	
;
; ****************************************************************************
; * General equates.  These may be changed to accommodate the reference clock* 
; * frequency, the desired upper frequency limit, and the default startup    *
; * frequency.                                                               *
; ****************************************************************************
;


PortA   equ     0x05
PortB   equ     0x06
PortC	equ	0x07
PortD	equ	0x08
PortE	equ	0x09
TRISA   equ     0x05
TRISB   equ     0x06
TRISC	equ	0x07
TRISD	equ	0x08
TRISE	equ	0x09
EEdata  equ     0x10C
EEadr   equ     0x10D
EEdath  equ	0x10E
EEadrh	equ	0x10F
EEcon1  equ	0x18C
EEcon2	equ	0x18D
RCSTA	equ	0x18

; EEcon bits

EEPGD	equ	0x07
WREN    equ     0x02
WR      equ     0x01
RD      equ     0x00
;
; 
;
; ****************************************************************************
; * RAM page independent file registers:                                     *
; ****************************************************************************
;
INDF    	EQU     0x00
OPTION_REG	EQU	0x01
PCL     	EQU     0x02
STATUS  	EQU     0x03
FSR     	EQU     0x04
PCLATH  	EQU     0x0A
INTCON  	EQU     0x0B
PIR1		EQU	0x0C
PIE1		EQU	0x8C
ADRESH		EQU	0x1E
ADRESL		EQU	0X9E
ADCON0		EQU	0x1F
ADCON1		EQU	0x9F
;
; *****************************************************************************
; * Bit numbers for the STATUS file register:                                 *
; *****************************************************************************
;
B_RP1	EQU 	6
B_RP0   EQU     5
B_NTO   EQU     4
B_NPD   EQU     3
B_Z     EQU     2
B_DC    EQU     1
B_C     EQU     0
;
; ****************************************************************************
; * Bit numbers for other file registers:                                    *
; ****************************************************************************
GIE		EQU	7	; INTCON
PIE		EQU	6	; INTCON
RBIE	EQU 3	;	"
T0IF		EQU	2	; INTCON
RBIF	EQU 0	;	"
GO		EQU	2	; ADCON
ADIF		EQU	5	; PIR1
ADIE		EQU	6	; PIE1
;tune_flag	EQU	0 	; Flag	Used to indicate if tuning step needed	
;RIT_flag  	EQU	1	; Flag	Used to mark RIT operation state
;RIT_tx_flag	EQU	2  	; Flag	Used to indicate if Tx freq enabled under RIT
;rs_flag		EQU  	3	; Flag 	Used in LCD comms routine
bar_flag	EQU	4	; Flag	Used in bar graph routine
;band_flag	EQU	5	; Flag used to mark state of band change routine
;_rit		EQU 1
;tester_rit	equ 2

; ****************************************************************************
; * Assign names to IO pins.                                                 *
; ****************************************************************************
;
; Port A bits:
;
;data_a         equ   0x04     ;sortie datas,  broche 
;clock_c        equ   0x02     ;sortie clock,  broche 



; Port B bits:



; Port C	Controls

data_a         equ   0x04     ;sortie datas,  broche 
clock_c        equ   0x03     ;sortie clock,  broche

;RIT_State	equ	0x00		  ; RIT High Off Low on
;TR_Line		equ	0x01		  ; Tx (Low) Rx (High) Line
;Tune_Rate	equ	0x02		  ; High 10Hz, Low 1Hz
;band_ch 	equ 	0x03              ; Band/Power on calibrate, (Active Low)
;IF_Offset_Use	equ	0x04		  ; Use IF Offset High Yes, Low No
;IF_add_sub	equ	0x05		  ; IF Add/Sub High Add, Low Sub


; Port D	LCD Controls and 4 bit Data

LCD_rs  	equ     0x03             ; 0=instruction, 1=data
LCD_rw  	equ     0x02              ; 0=write, 1=read
LCD_e   	equ     0x01              ; 0=disable, 1=enable


; Port E Bits	Processor driven outputs

DDS_load 	equ     0x03              ; Update pin on AD9850
DDS_clk 	equ     0x00              ; AD9850 write clock
DDS_data 	equ     0x01              ; AD9850 serial data input
;
; ****************************************************************************
; * ID location information:                                                 *
; * (MPASM warns about DW here, don't worry)                                 *
; ****************************************************************************
;idlocat org     0x2000          ;starting address for ID locations
;	__IDLOCS 0x003B         ;revision = 003B

	 ;ORG     0x2000
        ;DATA    0x007F
        ;DATA    0x007F
       ; DATA    0x007F
        ;DATA    0x007F

; preload the 16F628 internal eeprom
	org     0x2100          ;starting address for 16F628 internal eeprom
;        ;             
; initialize tunable memory locations (9 bands x 4 bytes = 36 bytes total)
;**************************************************************************
;****************debut de bande*****************************************
	       ;VALEUR POUR LE LCD
	     
	      
	; la freq du dds est le dixime de la freq donne par le pll (le pll x par 10 la freq du dds)
	; le freq affiche sur le lcd est = a la freq du ddds car je n'affiche pas le hertz
		
	de      0x00              ; 80 meters
	de      0x05              ; ADRESSE = 0 
	de      0x57              ; 350.000
	de      0x30              ; 
	
	de      0x00              ; 40 meters
	de      0x0a              ; ADRESSE = 4 
	de      0xae              ;700.000= 00 0a ae 60
	de      0x60              ; 
		

	
	de      0x00              ; 10 MHz100 ;
	de      0x0f              ; 30 meters
	de      0x69              ; ADRESSE = 8
	de      0x50            ; 1.010.000
	
	de      0x00              ; 20 meters
	de      0x15              ; 
	de      0x5c              ;ADRESSE = 0c 
	de      0xc0              ;1.400.000
	
	de      0x00              ; 18 MHz068 
	de      0x1b              ; 17 metres
	de      0x91              ;ADRESSE = 10 
	de      0xd0             ; 1.806.800
	
	de      0x00              ; 21Mgh
	de      0x20              ; 15 metres
	de      0x0b              ;ADRESSE = 14
	de      0x20              ;2.100.000
	
	de      0x00              ; 24 MHz890 ;
	de      0x25              ; 12 metres
	de      0xfa              ;ADRESSE = 18  
	de      0xa8            ; 
			
	de      0x00 			 ;adresse 0x1c           
	de      0x2a             ; 28 MHz
	de      0xb9             ;10 metres
	de      0x80           ;
							;
							
	de      0x00              ; adresse 20
	de      0x4c              ; 50mgh
	de      0x4b              ;5.000.000
	de      0x40
	
	de      0x00              ; adresse 24 ; 
	de      0x02              ; 1.8 Mgh
	de      0xbf              ;180.000
	de      0x20	
;**********************************************************************************************
;**************frequence fin de bande********************
;	valeur pour le dds	
	
 	de      0x00   ;adresse 0x28 (freq de retour RIT vres simplex)
	de      0x00              ; 
	de      0x00              ;
	de      0x01
	
	de      0x00   ;adresse 0x2c (frequence RIT)
	de      0x16              ; 
	de      0x36              ;
	de      0x23
	
	

	de      0x00              ;adresse 30 
	de      0x29              ; 18.168 mgh
	de      0x74              ;2.716.800
	de      0x80
	
	de      0x00              ; adresse 34
	de      0x12              ; bande 21.450mgh
	de      0xff              ; dds = 1.245.000 mgh
	de      0x48
	
	de      0x00              ; adresse 38
	de      0x18              ; bande 24.990 mgh
	de      0x66              ; dds = 1.599.900 mgh
	de      0x18
	
	de      0x00              ; adresse 3c
	de      0x20              ; bande 30 mgh
	de      0x0b              ; dds = 2.100.000 mgh
	de      0x20
	
	de      0x00              ; adresse 40
	de      0x00              ; libre
	de      0x00              ;
	de      0x01
	
	de      0x00              ; adresse 44
	de      0x00              ; libre
	de      0x00              ;
	de      0x01
	
; ***************************************************************************
;     Clear 52 unused EEPROM bytes. (could be used for 52/4=13 more memories)
;
	de      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	de      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	de      0,0,0,0,0,0,0,0
;        
; initialize band to 0
;  starts at location 120 of the internal eeprom
	de      0x00              ; default - baseband
;        
; clear 3 unused locations
	de      0,0,0
; ****************************************************************************
; * Setup the initial constant, based on the frequency of the reference      *
; * oscillator.  This can be tweaked with the calibrate function.            *
; ****************************************************************************
; 
;  starts at location 124 of the internal eeprom
	de      ref_osc_0
	de      ref_osc_1
	de      ref_osc_2
	de      ref_osc_3 

; ****************************************************************************
; *           Allocate variables in general purpose register space           *
; ****************************************************************************

;*****************************************************************************
;                        VARIABLES BANQUE 0                                  *
;*****************************************************************************

; Zone de 80 bytes
; ----------------

	CBLOCK	0x20		; Dbut de la zone (0x20  0x6F)
	
;
	BCD_0                     ; Display frequency (BCD) 
	  BCD_1                   ;  (5 bytes)
	  BCD_2
	  BCD_3
	  BCD_4     
	AD9850_0                  ; AD9850 control word 
	 AD9850_1                ;  (5 bytes)
	  AD9850_2
	  AD9850_3
	  AD9850_4
	fstep_0                   ; Frequency inc/dec 
	  fstep_1                 ;  (4 bytes)
	  fstep_2
	  fstep_3
	BCD_count                 ; Used in bin2BCD routine
	BCD_temp                  ;   "
	mult_count                ; Used in calc_dds_word 
	bit_count                 ;   "
	byte2send                 ;
	LCD_char                  ; Character being sent to the LCD
	LCD_read                  ; Character read from the LCD
	LCDtmp						;utilis tempo efface ligne lcd
	timer1                    ; Used in delay routines
	timer2                    ;   "
	ren_timer_0               ; For variable rate tuning 
	ren_timer_1             ;  (2 bytes)
	ren_new                   ; New value of encoder pins A and B
	ren_old                   ; Old value of encoder pins A and B
	ren_read                  ; Encoder pins A and B and switch pin
	last_dir                  ; Indicates last direction of encoder
	next_dir                  ; Indicates expected direction
	count                     ; loop counter  (gets reused)
	rs_value                  ; The LCD rs line flag value
	osc_temp_0                ; Oscillator frequency 
	  osc_temp_1              ;  (4 bytes)
	  osc_temp_2        
	  osc_temp_3
	  temp
	  temp1
	  count1
	  count2
	
	  point_s
	  pas
	adresse
	add_sous_affi

	bli_bls

	  DDS_freq_0
	  DDS_freq_1
	  DDS_freq_2
	  DDS_freq_3				
	tester
	tester_rit
	tester_a_ou_b
	tester_entre
	memoire	 
 		R_memoire
 		band_cinquante
 		dixmega	    	; 1 re frappe sur les touches en mode NUM_riques
 		unmega			; 2
 		centkilo		; 3
 		dixkilo			; 4
 		unkilo			; 5
 		numero_touche   ; numero de la touche en mode NUM_erique
 		nombre_appui	;sur le clavier en mod NUM_erique on commence par 10mgh
 		 				;puis 1 mgh puis 100 kgh puis 10 khg puis 1 kgh
 						;soit 5 appuis sue les touches numeriques 
 	freq_00		
 	freq_01				;valeurs retournes par les touches lors de l'entree 
 	freq_02             ; de la frequence par le clavier en mode "NUM"emerique
 	freq_03
 	attenuateur	  		; si = 1 = preampli de reception en service
 	preampli_rec		; si = 1 = attenuateur de reception en service
	temp2  				;utilis pour l'add ou la soustrac des poids binaires de u17
 	tone
 	chut 			; silence dans les commutations
	S_point
	Flag
	AGC_level
	AGC_BIAS
	tester_LouU	
	;cag 
	pwr_pot 		  
 	entre_fi_qtz	;index si= 1 les touches du clavier sont utilise pour rentrer la frequence fi
	ENDC                      ; End of Data Block

    
;*****************************************************************************
;                      VARIABLES banque 1                                *
;*****************************************************************************

; Zone de 80 bytes
; ----------------

	CBLOCK 0xa0		; Dbut de la zone (0xa0  0xef)

	 ;tester_rit 
	IF_Sub_3 ;equ 0x00	;MSb  1.105.951
	IF_Sub_2 ;equ 0x10
	IF_Sub_1 ;equ 0xe0
	IF_Sub_0 ;equ 0x1f	;LSb
	
	IF_Add_3 ;equ 0x00	;MSb =1.105.651 mgh
    IF_Add_2 ;equ 0x10
    IF_Add_1 ;equ 0xde
    IF_Add_0 ;equ 0xf3	;LSb
	  
	ENDC
;*****************************************************************************
;                      VARIABLES ZONE COMMUNE                                *
;*****************************************************************************

; Zone de 16 bytes
; ----------------

	CBLOCK 0x70		; Dbut de la zone (0x70  0x7F)
	w_temp : 1		; Sauvegarde registre W
	status_temp : 1		; sauvegarde registre STATUS
	FSR_temp : 1		; sauvegarde FSR (si indirect en interrupt)
	PCLATH_temp : 1		; sauvegarde PCLATH (si prog>2K)
	

	
		band                      ; Used to index a table of frequencies
		osc_0                     ; Current oscillator 
	  osc_1                   ;  (4 bytes)
	  osc_2
	  osc_3
	freq_0                    ; Display frequency (hex) 
	  freq_1                  ;  (4 bytes) 
	  freq_2
	  freq_3
	  cag
     ;EEADRH
     ;EEADR

	  
	ENDC



; Bank Select Macros to simplify bank changes

Bank0	MACRO
	bcf	STATUS,B_RP0	; Select Bank 0
	bcf	STATUS,B_RP1	;
	ENDM

Bank1	MACRO
	bsf	STATUS,B_RP0	; Select Bank 1
	bcf	STATUS,B_RP1	;
	ENDM

Bank2	MACRO
	bcf	STATUS,B_RP0	; Select Bank 2
	bsf	STATUS,B_RP1	;
	ENDM

Bank3	MACRO
	bsf	STATUS,B_RP0	; Select Bank 3
	bsf	STATUS,B_RP1	;
	ENDM

; ****************************************************************************
; ****************************************************************************
			; Sauts inter-banques
			; -------------------

GOTOX	macro	ADRESSE			; saut inter-page
	local BIT4 = (ADRESSE & 0x1000)	; voir bit 12
	local BIT3 = (ADRESSE & 0x0800) ; voir bit 11
	local	ICI			; adresse courante
ICI
	local PICI = (ICI+2 & 0x1800)	; page du saut
	IF BIT3				; si page 1 ou 3
	bsf	PCLATH , 3		; b3 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 3		; b3 de PCLATH = 0
	ENDIF
	IF BIT4				; si page 2 ou 3
	bsf	PCLATH , 4		; b4 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 4		; b4 de PCLATH = 0
	ENDIF
	goto (ADRESSE & 0x7FF | PICI)	; adresse simule
	endm


PCLAX	macro	ADRESSE			; positionne PCLATH pour
					; les sauts sans le saut
	local BIT4 = (ADRESSE & 0x1000)	; voir bit 12
	local BIT3 = (ADRESSE & 0x0800) ; voir bit 11

	IF BIT3				; si page 1 ou 3
	bsf	PCLATH , 3		; b3 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 3		; b3 de PCLATH = 0
	ENDIF
	IF BIT4				; si page 2 ou 3
	bsf	PCLATH , 4		; b4 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 4		; b4 de PCLATH = 0
	ENDIF
	endm

GOTSX	macro	ADRESSE			; saut inter-page sans
					; slection de PCLATH
	local	ICI			; adresse courante
	local PICI = (ICI & 0x1800)	; page du saut
ICI
	goto (ADRESSE & 0x7FF | PICI)	; adresse simule
	endm


			; Sous-routines inter-banques
			; ----------------------------

CALLX	macro	ADRESSE			; CALLX inter-page
	local BIT4 = (ADRESSE & 0x1000)	; voir bit 12
	local BIT3 = (ADRESSE & 0x0800) ; voir bit 11
	local	ICI			; adresse courante
ICI
	local PICI = (ICI+2 & 0x1800)	; page du saut
	IF BIT3				; si page 1 ou 3
	bsf	PCLATH , 3		; b3 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 3		; b3 de PCLATH = 0
	ENDIF
	IF BIT4				; si page 2 ou 3
	bsf	PCLATH , 4		; b4 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 4		; b4 de PCLATH = 0
	ENDIF
	call (ADRESSE & 0x7FF | PICI)	; adresse simule
	local BIT4 = ((ICI+5) & 0x1000)	; voir bit 12
	local BIT3 = ((ICI+5) & 0x0800)	; voir bit 11
	IF BIT3				; si page 1 ou 3
	bsf	PCLATH , 3		; b3 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 3		; b3 de PCLATH = 0
	ENDIF
	IF BIT4				; si page 2 ou 3
	bsf	PCLATH , 4		; b4 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 4		; b4 de PCLATH = 0
	ENDIF
	endm


CALLSX	macro	ADRESSE			; sous-routine inter-page sans
					; slection de PCLATH
	local	ICI			; adresse courante
	local PICI = (ICI & 0x1800)	; page du saut
ICI
	CALL (ADRESSE & 0x7FF | PICI)	; adresse simule
	endm
;************************************************************************

	ORG	0x0000		  	  
reset_entry
        goto    Start             ; Jump around the band table to main program

    nop
	nop
	nop

;*****************************************************************************
;                     ROUTINE INTERRUPTION                                   *
;*****************************************************************************
;-----------------------------------------------------------------------------
; Si on n'utilise pas l'adressage indirect dans les interrupts, on se passera
; de sauvegarder FSR
; Si le programme ne fait pas plus de 2K, on se passera de la gestion de 
; PCLATH
;-----------------------------------------------------------------------------
			;sauvegarder registres	
			;---------------------
	org 0x004		; adresse d'interruption
	Bank0
	movwf   w_temp  	; sauver registre W
	swapf	STATUS,w	; swap status avec rsultat dans w
	movwf	status_temp	; sauver status swapp
	movf	FSR , w		; charger FSR
	movwf	FSR_temp	; sauvegarder FSR
	movf	PCLATH , w	; charger PCLATH
	movwf	PCLATH_temp	; le sauver
	clrf	PCLATH		; on est en page 0
	Bank0			; passer en banque0

			; interruption RB4/RB7
			; --------------------
intsw2
	btfsc	INTCON,RBIE	; tester si interrupt RB4/7 autorise
	btfss	INTCON,RBIF	; oui, tester si interrupt RB4/7 en cours
	goto	restorereg;	goto	 main		; non sauter
	
	GOTOX	intrb4  ; oui, traiter interrupt RB4/7
int_retour	
	Bank0
	
		
	bcf		PortC,0
	bcf		PortC,1
	bcf		PortC,2
	bcf		PortC,5
	bcf		PortC,6
	bcf		PortC,7
	
	
PAS_a_1
	movf    PortB,w   ;ne voir que les bits 4,5,6,7 du porb b 
	movwf	count
	ANDWF	count,w

	movlw	B'11110000'	
	subwf	count,w
	PCLAX	egal_a_1 
	btfsC	STATUS,B_C ;si c = 1 saut a egal_a_1
	GOTSX 	egal_a_1
	MOVLW   0X04	
	CALLX	temporis  ;tempo anti rebond
	GOTOX	PAS_a_1	; C est EGAL A  0
egal_a_1
	
	
	bcf	INTCON,RBIF	; effacer flag interupt RB4/7
	goto	restorereg	; et fin d'interrupt

			; dtection interruptions priphriques
			; le test peut tre supprim si une seule
			; interrupt est traite  la fois
			; --------------------------------------


			;restaurer registres
			;-------------------
restorereg
	movf	PCLATH_temp , w	; recharger ancien PCLATH
	movwf	PCLATH		; le restaurer
	movf	FSR_temp , w	; charger FSR sauv
	movwf	FSR		; restaurer FSR
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
				; W restaur sans modifier status
	retfie  		; return from interrupt

;*****************************************************************************


;*****************************************************************************
;                         INTERRUPTION RB0/RB4                               *
;*****************************************************************************
;
;intrb4
;	return			; fin d'interruption RB0/RB4
				; peut tre remplac par 
;*********************************************************************

; ****************************************************************************  
; S Meter increments
;
; The 6dB S Meter steps are mapped as a linear range of values from 0 to 90 
; (0 to 5Eh) over a 1 volt range. Each increment is 1dB, 90dB = S9+40. 
; Set AGC input to the A/D with a pot to suit.
; The steps are used to compare the analogue input samples and compute magnitude.
; Note that the linear law assumes that the IF AGC characteristics are also 
; linear. If they are not an alternative set of values can be programmed here
; by measuring AGC voltage against a calibrated input signal and normalising
; 6dB steps against 0 - 90 (00 - 5Eh).
;
; ****************************************************************************  

;==============================================================================
;
;	SECTION 1
;	
;	MAIN PROGRAM INITIALISATION AND CONTROL LOOP
;
;==============================================================================

; *****************************************************************************
; *                                                                           *
; * Purpose:  This is the start of the program.  It initializes the LCD and   *
; *           detects whether to enter calibrate mode.  If so, it CALLXs the   *
; *           Calibrate routine.  Otherwise, it sets the power-on frequency   *
; *           and enters the loop to poll the encoder.                        *
; *                                                                           *
; *   Input:  The start up frequency is defined in the default_3 ...          *
; *           definitions above, and relies on the reference oscillator       *
; *           constant defined in ref_osc_3 ... ref_osc_0.                    *
; *                                                                           *
; *  Output:  Normal VFO operation.                                           *
; *                                                                           *
; *****************************************************************************
;

Start
	Bank0
; Configure Ports
       ;
   
	clrf	PortA		  ; Initialise Port A
	clrf	PortB		  ; Initialise Port B
	clrf	PortC		  ; Initialise Port C
	clrf	PortD		  ; Initialise Port D
	clrf 	PortE		  ; Initialise Port E

	Bank1			  ; Switch to bank 1
    bcf     0x01,7            ; Disable weak pullups

	movlw	b'11110111'  ; Set Port b 
	movwf	TRISB		  ;	  
    movlw 	b'00000000'	  ; Set Port C 0, as OUTput(
	movwf   TRISC
	movlw	0x00		  ; Set Port D as outputs(lcd)
	movwf	TRISD		  ;

	
		;Bank1			  ; Switch to bank 1
	movlw	b'00000100' ;  
	movwf	ADCON1		;Left justify, 3 analog channel, no Vref
	movlw	b'00111111';
	movwf	TRISA		  ; PortA  Inputs 
	movlw	0x00		  ; Set Port E as ouputs (dds)
	movwf	TRISE			  ;
	Bank0
	
	movlw	b'01000001'	  ; Fosc/8, A/D enabled
	movwf	ADCON0
	
	 movlw	b'00001000'	    ;rb4/7 autoris
    movwf	INTCON 
      
	;bsf		INTCON,GIE      ;int general autoris
	
	bcf		PortC,0
	bcf		PortC,1
	bcf		PortC,2
	bcf		PortC,5
	bcf		PortC,6
	bcf		PortC,7
	
				  ; Switch back to bank 0
	;movlw	b'01000001'	  ; Fosc/8, A/D enabled
	;movwf	ADCON0
;ttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt
	CALLX	S_Meter

;*******************************************************************************************
	bsf		 PortB,2	; ptt a 1
	
	bsf		PortB,3     ;relais emission decoller
	
	movlw	0X01
	MOVWF	band_cinquante   ; osc en infradyne (en supradyne pour les autres bandes
	
	movlw 	0x01 		
	movwf	chut			;si = a 0 = chut dans les commutations
	
	clrf	tone			;si = 0 = arret ,si = 1 = marche
	
	clrf	cag		  	;si = 0 = arret ,si = 1 = marche
	
	clrf	temp2			;add ou soustrac sur les sorties de u17
		
	clrf	attenuateur		;si = 0 = arret ,si = 1 = marche
				
	clrf	preampli_rec	;si = 0 = arret ,si = 1 = marche
			
	clrf	memoire			;1 = touche mise en memoire active
			
	clrf	R_memoire		;1 = touche mise en R_memoire active

 	movlw	0x01
	movwf	pas;			:pas = 100 hertz
	
	;Bank1
	movlw	0x01
	movwf	 tester_rit 	 ; 0 = rit 1 = normal
	;Bank0
	
	movlw	0x01	
	movwf	tester_a_ou_b	; 1= vfo a ; 0= vfo b
	
	movlw	0x01
	movwf	tester_entre	;si = 0 la touche "NUM"erique a ete active
	
	movlw	0x01			;si = 1 la touche Numerique a ete activ et on est dans le s/p
	movwf	entre_fi_qtz	; entr de frequence du fitre a quartz 
	
	
	clrf	tester_LouU		;si = 0 = lsb si = 1 =usb
	
	clrf	nombre_appui	; nombre d'appui sur les touches numeriques 'entre de la freq par le clavier)
	
	movlw	0X72 ;
	MOVWF	adresse			;adresse du pcf8574a (commutation l/usb,tone,att,preamp,ect
	
	clrf	temp1			; au depart tout le port du pcf8574a a zero
	
	;GOTOX   u_lsb	
	CALLX   envoie_i2c		
		
	CALLX	i2c_continue    ;il n'y a pas d'autre data
	
;********************************
	
	movlw	0X76
	MOVWF	adresse		;adresse du 2eme pcf8574a  (commutation des relais de la tete hf)
	
	clrf	temp1	; 
	CALLX    envoie_i2c

	CALLX	i2c_continue    ;il n'y a pas d'autres data
	
	
	CALLX    init_LCD ;nitialize the LCD

	;GOTOX	S_Meter

;***************************************************************************
;              ECRIRE SUR LE LCD F1ARO 2005
;***************************************************************************

	movlw   0X81              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 

	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'F'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   '1'               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'A'               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'R'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw	'O'               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
         ; 
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	movlw   '*'               ; 
	movwf   LCD_char	          
	CALLX    data2LCD
	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	 
	movlw   '2'               ;ECRIRE 2 
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   '0'               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   '0'               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   '7'
	movwf   LCD_char          ; 
       CALLX    data2LCD          ;
       
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD	

	Bank0
		
	MOVLW   0X015	 ;TEMPS POUR VOIR F1ARO 2005 SUR LE LCD
	CALLX	temporis
         	
	CALLX	ATTENU_m
	CALLX	PRE_m
	CALLX	TONE_m 
	CALLX	CAG_m 	
 	CALLX	LCDcls1			;effacer la ligne 1
;*********************************************************************** 

	movlw   0xc4             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'V'
	movwf   LCD_char          ; 
	CALLX    data2LCD           ; 
	MOVLW	'F'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'O'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'A'
	movwf   LCD_char          ; 
	CALLX    data2LCD           ; 

	


;************************************************************************

	movlw 0x01

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;PCLAX	calibrate
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;BTFSs	PortB,7	;SI = O ALORS 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	CALLSX	calibrate
;	CALLX	 putflash

;
read_EEocs

 ;CALLX    getee ;lire la valeur de osc_3,osc_2,osc_1,osc_0 (ref du dds)
	;movlw	0x2A 
   		;movwf	osc_3                ; Most significant osc byte
   	;	movlw	0xf3
	;	movwf	osc_2                ; Next byte
		;movlw	0x1D
	;	movwf   osc_1                 ; Next byte
	;	movlw	0xC4
	;	movwf	osc_0


       CALLX		geteeflash;
   
		
	
;*******lire la valeur de la fi lsb*******************************
		     
	movlw	0xe8			;adresse ou est ranger la valeur de la fi
		 Bank2 
		movwf  EEadr            ; set the EEPROM read address
	CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Sub_3    
	Bank0
	
		CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Sub_2    
	Bank0
	
		CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Sub_1 
	Bank0
		CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Sub_0   
	Bank0
;*******lire la valeur de la fi usb*******************************
	
	movlw	0xf0			;adresse ou est ranger la valeur de la fi
		 Bank2 
		movwf  EEadr            ; set the EEPROM read address
	CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Add_3    
	Bank0
	
		CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Add_2    
	Bank0
	
		CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Add_1 
	Bank0
		CALLX    read_EEPROM       ; Read EEPROM
	Bank1
	movwf   IF_Add_0  
	Bank0

	movlw	0x04         ;get the band from eeprom
	movwf	band
	movlw	0X04
	CALLX   getfee          ;get the frequency from eeprom
	CALLX	COM_PASS_PAS
	CALLX	BUS_I2C		;donnees pour le pcf8574
	CALLX	val_pwr
	CALLX	i2c_pwr
	CALLX	PIC_esclave
	movlw	0x2c		;sauver la freq  qui est affich sur le LCD 
	CALLX 	putfee		;a l'adresse 0x2C (adresse de retour de RIT/normal) 

	CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la bande 
;       Display the power on frequency.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
	CALLX	 affi_100h

	
	CALLX    bin2BCD           ; Convert it to BCD
	CALLX    show_freq         ; Display it	
	CALLX    MEGAH 
;       Send power on frequency to the DDS chip.
 
	CALLX     newdan
	CALLX    calc_dds_word     ; Convert to delta value
	
	CALLX    send_dds_word     ; Send the power-on frequency to the
	CALLX    send_dds_word 	 ;   AD9850 in serial mode

;==========================================================================
;
;	SECTION 4

;       Get the power on encoder value.
;
	movf    PortB,w           ; Read port A	
	movwf   ren_read          ; Save it in ren_read
	movlw   0x03              ; Get encoder mask
	andwf   ren_read,w        ; Get encoder bits
	movwf   ren_old           ; Save in ren_old
;
;       Initialize variables.
                                       
	clrf    last_dir          ; Clear the knob direction indicator
	movlw   0X01
	movwf	count1
	movlw   0X01
	movwf	count2

;YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY

; Fall into the Main Program Loop
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  This is the Main Program Loop. The program's main loop          *
; *           CALLXs poll_encoder, which continuously polls the rotary shaft   *
; *           encoder.  When the shaft encoder has changed, the direction     *
; *           it moved is determined and stored in last_dir.  The subroutine  *
; *           then returns to main.                                           *                       
; *                                                                           *
; *           If the push button switch was not pressed, then the variable    *
; *           fstep is calculated based on the delay between shaft encoder    *
; *           changes.  ren_timer contains the delay value determined by      *
; *           the poll_encoder subroutine.  The variable fstep is added or    *
; *           subtracted from the current VFO frequency stored in freq.       *
; *           The contents of freq are then converted to a BCD number in      *
; *           subroutine bin2BCD.  The subroutine show_freq is then CALLXed    *
; *           to display the result on the Liquid Crystal Display.  Next, the *
; *           subroutine calc_dds_word is used to calculate the DDS           *
; *           frequency control word from the values in freq and osc.         *
; *           The result is stored in AD9850.  This data is transferred to    *
; *           the AD9850 DDS chip by CALLXing the subroutine send_dds_word.    *
; *                                                                           *
; *           If the push button is pressed while turning the encoder then    *
; *           freq is loaded with a constant stored in band_table.  The       *
; *           variable band is used as an index into the table.  Band         *
; *           is incremented or decremented based on the encoder direction.   *
; *                                                                           *
; *   Input:  None.                                                           *
; *                                                                           *
; *  Output:  None.                                                           *
; *                                                                           *
; *****************************************************************************
;

main


	GOTOX	freq_vfo
	
temporis	
	MOVWF   count	
delai 
  CALLX    wait_64ms          ;
 	PCLAX	delai
	decfsz  count,f
	GOTSX    delai
	return



	
valeur_pas

;       Determine step size to use (10 Hz or 100 Hz ou 1 Khz).

;	btfss	ren_read,2        ;Is the K/H = 1 sauter 

	
		                   
	CALLX    wait_16ms		 
	clrf    fstep_3           ; Guess that we want 1 Hz steps by 
	clrf    fstep_2           ;   setting fstep to one.
	clrf    fstep_1           ;
		
	movlw	0X00		;si pas = 0x00  alors Z=1
	XORWF	pas,w
	PCLAX	go_10h
	btfsc	STATUS,B_Z
	GOTSX 	go_10h          ;pas de 10hz

	movlw	0X01		
	XORWF	pas,w		;si pas = 0x01  alors Z=1
	PCLAX	go_100h
	btfsc	STATUS,B_Z
	GOTSX 	go_100h		;pas de 100hz

	movlw	0X02	
	XORWF	pas,w
	PCLAX	go_1000h
	btfsc	STATUS,B_Z	;si pas = 0x02 alors Z=1	
	GOTSX 	go_1000h	;pas de 1kz

	movlw	0X03	
	XORWF	pas,w		;si pas = 0x03  alors Z=1
	PCLAX	go_10kz
	btfsc	STATUS,B_Z	;pas = 10khz
	GOTSX	go_10kz

	;movlw	0X04	
;	XORWF	pas,w		;si pas = 0x04  alors Z=1
;	btfsc	STATUS,B_Z	;pas = 100Khz
;	GOTO	go_100kz

	;movlw	0X05	
	;XORWF	pas,w		;si pas = 0x05  alors Z=1
	;btfsc	STATUS,B_Z	;pas = 1Mhz
	;GOTO	go_1Mhz

	
	movlw	0x04;6
	xorwf	pas,w
	PCLAX	pas
	btfsc	STATUS,B_Z
	GOTSX	PASP
PASP	
	clrf	pas
	
go_10h	
	movlw	0X00
	movwf	pas
             ; pas de 10 Hz
	movlw	0x01
	movwf   fstep_0
	
	GOTSX    go_step           ; 
	
go_100h	           
	movlw   0x0a              ; Yes, set the step value to 100 Hz
	movwf   fstep_0           ;   by setting fstep_0 to 0x64 and
	movlw   0x00              ;   fstep_1 to 0x00
	movwf   fstep_1           ;
		
	GOTSX    go_step           ; Use the 1 kHz step	

go_1000h	           
      	movlw   0x64              ; Yes, set the step value to 1 kHz
	movwf   fstep_0           ;   by setting fstep_0 to 0xE8 and
	GOTSX    go_step


go_10kz
	movlw 	0xe8
	movwf	fstep_0
	movlw	0x03
	MOVWF	fstep_1
	GOTSX	go_step

;go_100kz
;	movlw 	0x10
;	movwf	fstep_0
;	movlw	0x27
;	MOVWF	fstep_1
;	goto	go_step
	
;go_1Mhz
;	movlw 	0xa0
;	movwf	fstep_0
;	movlw	0x86
;	MOVWF	fstep_1
;	movlw	0x01
;	MOVWF	fstep_2
	
	
go_step 
	;PCLAX	main	
;	btfss	PortA,2 	;si = 1 saut
;	GOTSX	main
ensuite
	PCLAX	up
	btfsc   last_dir,1        ; Is the knob going up?
	GOTSX    up                ; Yes, then add the increment
	
down

	CALLX    sub_step          ; Subtract fstep from freq		
	GOTOX    write1             ; Update LCD and DDS
up
		
	CALLX    add_step          ; Add fstep to freq	
	GOTOX    write1  	
	
write1
	CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la bande 
		
	
	CALLX    bin2BCD           ; Convert the frequency to BCD	
	CALLX    show_freq         ; Display the frequency on the LCD
	CALLX	newdan
	CALLX    calc_dds_word     ; Find the control word for the DDS chip
	CALLX    send_dds_word     ; Send the control word to the DDS chip	
	
	GOTSX   main      	
	

;************************************************************************************
;***************afficher LSB ou USB sur le lcd ********************************
lsb_usb	
		      

	movlw	0X0c		
	XORWF	band,w		;si band = 0x0c alors Z=1,14
	PCLAX	usb
	btfsc	STATUS,B_Z
	GOTSX 	usb

	
	movlw	0X10		;si band = 0x10  alors Z=1,18
	XORWF	band,w
	PCLAX	usb
	btfsc	STATUS,B_Z
	GOTSX 	usb          ;

	movlw	0X14		
	XORWF	band,w		;si band = 0x14  alors Z=1,21
	PCLAX	usb
	btfsc	STATUS,B_Z
	GOTSX 	usb

	
	movlw	0X18		;si band = 0x18  alors Z=1,24
	XORWF	band,w
	PCLAX	usb
	btfsc	STATUS,B_Z
	GOTSX 	usb         ;

	movlw	0X1c		
	XORWF	band,w		;si band = 0x1c  alors Z=1;28
	PCLAX	usb
	btfsc	STATUS,B_Z
	GOTSX 	usb

	movlw	0X20		;50 mgh
	XORWF	band,w		;si band = 0x1c  alors Z=1;50
	PCLAX	uusb
	btfsc	STATUS,B_Z	; comme l'osc local est en infradyne le bfo ce trouve en lsb 
	GOTSX	uusb	

	nop
lsb
	movlw	0x00
	movwf	bli_bls         ;si = zero = en lsb
	
	movlw	0x00			;si =0 alors supradyne si = 1 alors infradyne
	movwf	add_sous_affi	;variable pour OSC local
lsb1		
	movlw   0xc0             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'L'
	movwf   LCD_char          ; 
	CALLX    data2LCD           ; 
	MOVLW	'S'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'B'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	
	movlw	0X01		;
	XORWF	tester_LouU,w		;si tester_LouU -0x01 = 1 alors
	PCLAX	deja_en_L			;on saute a deja_en_L
	btfsc	STATUS,B_Z			;si tester_LouU -0x00 = 1 alors
	GOTSX	deja_en_L			;on continue le programme
	
	MOVLW	0X01
	MOVWF	tester_LouU
	
	MOVLW	0X40;80
	CALLX	ADD_U17		;METTRE LA PINS 11 de u17 A 0 (P6)
		
deja_en_L
		GOTOX	ret
		
		
uusb
         
bfo_pour_le_50

	movlw	0x01			;si =0 alors supradyne si = 1 alors infradyne
	movwf	add_sous_affi	;variable pour OSC local
	GOTOX	USSB				
usb
	MOVLW	0x00	;si =0 alors supradyne si = 1 alors infradyne
	movwf	add_sous_affi	;variable pour OSC local	

USSB	
	movlw	0x01            ;si = un = usb
	movwf	bli_bls
usb1	
	movlw   0xc0             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'U'
	movwf   LCD_char          ; 
	CALLX    data2LCD           ; 
	MOVLW	'S'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'B'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	
	
  	movlw	0X00		;
	XORWF	tester_LouU,w		;si tester_LouU - 0x00 = 0 alors
	PCLAX	deja_en_U				;on saute a deja_en_U
		btfsc	STATUS,B_Z			;si tester_LouU -0x00 = 1 alors
	GOTSX	deja_en_U			;on continue le programme
	
	MOVLW	0X00
	MOVWF	tester_LouU

	MOVLW	0X40
	CALLX	SOUS_U17			;METTRE LA PINS 11 de u17 A 1 (P6)
			
deja_en_U	
ret	
	RETURN
;*****************************************************************************************
              
;*******************************************************************************
limit_band
	return
;******************************************************************************

;
get_band
	CALLX    putband         ;save the new band setting in eeprom
	movf    band,w            ; Get the index of the high byte 
	CALLX    getfee          ;get the data from eeprom into frequency
				;       pointed to by band
	return                    ; Return to the CALLXer
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Power on initialization of Liquid Crystal Display.  The LCD     *
; *           controller chip must be equivalent to an Hitachi 44780.  The    *
; *           LCD is assumed to be a 16 X 1 display.                          *
; *                                                                           *
; *   Input:  None                                                            *
; *                                                                           *
; *  Output:  None                                                            *
; *                                                                           *
; *****************************************************************************
;



init_LCD
	CALLX    wait_64ms         ; Wait for LCD to power up
	movlw   0x30              ; LCD init instruction (First)
	movwf   PortD             ; Send to LCD via RB7..RB0
	bsf     PortD,LCD_e       ; Set the LCD E line high,
	CALLX    wait_64ms         ;   wait a "long" time,
	bcf     PortD,LCD_e       ;   and then Clear E 
	movlw   0x30              ; LCD init instruction (Second)
	movwf   PortD             ; Send to LCD via RB7..RB0
	bsf     PortD,LCD_e       ; Set E high,
	CALLX    wait_32ms         ;   wait a while,
	bcf     PortD,LCD_e       ;   and then Clear E 
	movlw   0x30              ; LCD init instruction (Third)
	movwf   PortD             ; Send to LCD via RB7..RB0
	bsf     PortD,LCD_e       ; Set E high,
	CALLX    wait_32ms         ;   wait a while,
	bcf     PortD,LCD_e       ;   and then Clear E
	movlw   0x20              ; 4-bit mode instruction
	movwf   PortD             ; Send to LCD via RB7..RB0
	bsf     PortD,LCD_e       ; Set E high,
	CALLX    wait_16ms         ;   wait a while,
	bcf     PortD,LCD_e       ;   and then Clear E
	movlw   0x28              ; 1/16 duty cycle, 5x8 matrix
	CALLX    cmnd2LCD          ; Send command in w to LCD
	movlw   0x08              ; Display off, cursor and blink off 
	CALLX    cmnd2LCD          ; Send command to LCD
	movlw   0x01              ; Clear and reset cursor
	CALLX    cmnd2LCD          ; Send command in w to LCD
	movlw   0x06              ; Set cursor to move right, no shift
	CALLX    cmnd2LCD          ; Send command in w to LCD
	movlw   0x0C              ; Display on, cursor and blink off
	CALLX    cmnd2LCD          ; Send command in w to LCD
	return                    ; 
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  This routine adds the 32 bit value of fstep to the 32 bit       *
; *           value in freq.  When incrementing, the fstep value is a         *
; *           positive integer.  When decrementing, fstep is the complement   *
; *           of the value being subtracted.                                  *
; *                                                                           *
; *   Input:  The 32 bit values in fstep and freq                             *
; *                                                                           *
; *  Output:  The sum of fstep and freq is stored in freq.  When incrementing *
; *           this value may exceed the maximum.  When decrementing, it may   *
; *           go negative.                                                    *
; *                                                                           *
; *****************************************************************************
add_step
	movf    fstep_0,w         ; Get low byte of the increment
	addwf   freq_0,f          ; Add it to the low byte of freq
	PCLAX	add1   	
	btfss   STATUS,B_C        ; Any carry?
	GOTSX    add1              ; No, add next byte
		PCLAX	add1   	
	incfsz  freq_1,f          ; Ripple carry up to the next byte
	GOTSX    add1              ; No new carry, add next byte
	PCLAX	add1
	incfsz  freq_2,f          ; Ripple carry up to the next byte
	GOTSX   add1              ; No new carry, add next byte
	incf    freq_3,f          ; Ripple carry up to the highest byte
add1
	movf    fstep_1,w         ; Get the next increment byte
	addwf   freq_1,f          ; Add it to the next higher byte
	PCLAX	add2  
	btfss   STATUS,B_C        ; Any carry?
	GOTSX   add2              ; No, add next byte
	PCLAX	add2  
	incfsz  freq_2,f          ; Ripple carry up to the next byte
	GOTSX    add2              ; No new carry, add next byte
	incf    freq_3,f          ; Ripple carry up to the highest byte
add2
	movf    fstep_2,w         ; Get the next to most significant increment	
	addwf   freq_2,f          ; Add it to the freq byte  
	PCLAX	add3
	btfss   STATUS,B_C        ; Any carry?
	GOTSX   add3              ; No, add last byte
	incf    freq_3,f          ; Ripple carry up to the highest byte
add3
	movf    fstep_3,w         ; Get the most significant increment byte
	addwf   freq_3,f          ; Add it to the most significant freq
	return                    ; Return to the CALLXer
;
; *****************************************************************************


; *****************************************************************************
; *                                                                           *
; * Purpose:  Subtract the increment step from freq, checking that it does    *
; *           not go below zero.                                              *
; *                                                                           *
; *   Input:  The values in fstep and freq.                                   *
; *                                                                           *
; *  Output:  The updated value in freq.                                      *
; *                                                                           *
; *****************************************************************************
;
sub_step
	comf    fstep_0,f         ; Subtraction of fstep from
	comf    fstep_1,f         ;   freq is done by adding the
	comf    fstep_2,f         ;   twos compliment of fstep to
	comf    fstep_3,f         ;   freq.
	PCLAX	comp_done 
	incfsz  fstep_0,f         ; Increment last byte
	GOTSX    comp_done         ; Non-zero, continue
	PCLAX	comp_done 
	incfsz  fstep_1,f         ; Increment next byte
	GOTSX    comp_done         ; Non-zero, continue
	PCLAX	comp_done 
	incfsz  fstep_2,f         ; Increment next byte
	GOTSX    comp_done         ; Non-zero, continue
	incf    fstep_3,f         ; Increment the high byte
comp_done
	CALLX    add_step          ; Add the compliment to do the subtraction
;
;       If the frequency has gone negative, clear it to zero.
	PCLAX	exit2  	
	btfss   freq_3,7          ; Is high order frequency byte "negative"?   
	GOTSX    exit2             ; No, keep going
set_min
	clrf    freq_0            ; Yes, set the frequency to zero
	clrf    freq_1            ; 
	clrf    freq_2            ; 
	clrf    freq_3            ; 
exit2
	return                    ; Return to the CALLXer
;
; *****************************************************************************

; *   Input:  Knob input read from port A                                     *
; *           ren_old -> the last encoder bits read
;	                 LIRE LE DERNIER BIT DE L'ENCODEUR
; *           last_dir -> the last direction moved                            *
; *                                                                           *
; *  Output:  ren_timer -> an indication the speed of the knob.               *
; *           ren_new -> the current encoder bits                             *
; *           last_dir -> the last direction (0 = down, 2 = up)               *
; *                                                                           *
; *****************************************************************************
;
freq_vfo

		
	PCLAX	emission
	BTFSS   PortB,2          ;si = 0 GOTO si = 1 SAUTE
	CALLSX   emission
	

	PCLAX	test_count1	    ; si count1 = 0 aller decfsz count2 
	decfsz	count1,f		; si = 1 aller a test_count1
	GOTSX	test_count1
	
	PCLAX	test_count2	    ; si test_count2 = 0 aller a callx S_METRE
	decfsz	count2,f		; si = 1 aller a test_count2
	GOTSX	test_count2		   ; le lcd est rafraichi toutes les 256x256 fois
							   ; cela evite un effet papillon
							
	bcf		INTCON,GIE 		;gie non autoris 
	CALLX	S_Meter
	bsf		INTCON,GIE 		;gie autoris 
test_count1
test_count2
	PCLAX	entre_00			;si = 1 indique que la touche "ENTRE_NUM" a ete activ
	BTFSS	tester_entre,0	;alors goto entre_00;'incrementation ne doit pas fonctionner dans cette config;
	GOTSX	entre_00			;sinon on saute et fait l'instruction suivante
	
	
	
	PCLAX	meme
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
					;alors gotox meme;l'incrementation ne doit pas fonctionner dans cette config
	GOTSX 	meme			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_meme
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
						;alors gotox RAP_mem;l'incrementation ne doit pas fonctionner dans cette config
	GOTSX 	RAP_meme		;sinon on saute a l'instruction suivante
		
	movf    PortB,w       ;Portb,w           ; Get the current encoder value
	movwf   ren_read          ; Save it
	movlw   0x03              ; Get encoder mask
	andwf   ren_read,w        ; Isolate encoder bits
	movwf   ren_new           ; Save new value
	xorwf   ren_old,w         ; Has it changed?		
	PCLAX	freq_vfo	;read_encoder	
	btfsc   STATUS,B_Z 
entre_00	
meme	
RAP_meme



	GOTSX   freq_vfo; read_encoder      ; No, keep looking until it changes
	
	

;       Determine which direction the encoder turned.
;
	bcf     STATUS,B_C        ; Clear the carry bit
	rlf     ren_old,f         ; 
	movf    ren_new,w         ; 
	xorwf   ren_old,f         ; 
	movf    ren_old,w         ; 
	andlw   0x02              ; 
	movwf   next_dir          ; 
	xorwf   last_dir,w        ; 
;
;       Prevent encoder slip from giving a false change in direction.
	PCLAX	peux_continue 
	btfsc   STATUS,B_Z        ; Zero?
	GOTSX    peux_continue       ; No slip; keep going
	movf    next_dir,w        ; Yes, update direction
	movwf   last_dir          ; 
	movf    ren_new,w         ; Save the current encoder bits for next time
	movwf   ren_old           ; 
	GOTSX   freq_vfo         ; read_encoder      ; Try again
peux_continue
	PCLAX	up22
	btfsc   ren_old,1         ; Are we going down?
	GOTSX    up22               ; No, indicate we are going up
	clrf    last_dir          ; Yes, clear last_dir
	GOTSX    exit33             ; Finish and return
up22 
	movlw   0x02              ; Set UP value in last_dir
	movwf   last_dir          ; 
exit33 
	movf    ren_new,w         ; Get the current encoder bits
	movwf   ren_old           ; Save them in ren_old for the next time
	
	
		
;PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP

	CALLX    wait_16ms		 
	clrf    fstep_3           ; Guess that we want 1 Hz steps by 
	clrf    fstep_2           ;   setting fstep to one.
	clrf    fstep_1           ;
		
	movlw	0X00		;si pas = 0x00  alors Z=1
	XORWF	pas,w
	PCLAX	go_10hh
	btfsc	STATUS,B_Z
	GOTSX 	go_10hh          ;pas de 10hz

	movlw	0X01		
	XORWF	pas,w		;si pas = 0x01  alors Z=1
	PCLAX	go_100hh
	btfsc	STATUS,B_Z
	GOTSX 	go_100hh		;pas de 100hz

	movlw	0X02	
	XORWF	pas,w
	PCLAX	go_1000hh
	btfsc	STATUS,B_Z	;si pas = 0x02 alors Z=1	
	GOTSX 	go_1000hh	;pas de 1kz

	movlw	0X03	
	XORWF	pas,w		;si pas = 0x03  alors Z=1
	PCLAX	go_10kzh
	btfsc	STATUS,B_Z	;pas = 10khz
	GOTSX	go_10kzh


	
	movlw	0x04;6
	xorwf	pas,w
	PCLAX	PASS
	btfsc	STATUS,B_Z
	GOTSX	PASS
PASS	
	clrf	pas
	
go_10hh	
	movlw	0X00 ; pas de 10 Hz
	movwf	pas         
	movlw	0x01
	movwf   fstep_0	
	GOTSX    go_stepp           ; 
	
go_100hh	           
	movlw   0x0a              ; Yes, set the step value to 100 Hz
	movwf   fstep_0           ;   by setting fstep_0 to 0x64 and
	movlw   0x00              ;   fstep_1 to 0x00
	movwf   fstep_1           ;		
	GOTSX    go_stepp           ; Use the 1 kHz step	

go_1000hh	           
      	movlw   0x64              ; Yes, set the step value to 1 kHz
	movwf   fstep_0           ;   by setting fstep_0 to 0xE8 and
	GOTSX    go_stepp

go_10kzh
	movlw 	0xe8
	movwf	fstep_0
	movlw	0x03
	MOVWF	fstep_1
	GOTSX	go_stepp
	
go_stepp	
		


	PCLAX	upp
	btfsc   last_dir,1        ; Is the knob going up?
	GOTSX    upp                ; Yes, then add the increment
	
downn
	CALLX    sub_step          ; Subtract fstep from freq		
	GOTOX    write11             ; Update LCD and DDS
	
upp		
	CALLX    add_step          ; Add fstep to freq
  
	
write11

	
	CALLX    bin2BCD           ; Convert the frequency to BCD
	CALLX    show_freq         ; Display the frequency on the LCD
	CALLX	newdan
	CALLX    calc_dds_word     ; Find the control word for the DDS chip
	CALLX    send_dds_word     ; Send the control word to the DDS chip
	CALLX    send_dds_word     ; Send the control word to the DDS chip

	GOTSX freq_vfo       		
	
	
;*******************************************************************************
affi_pas    ;s/p affichage de la valuer du pas sur le lcd
	
go_aff
init_pas
	movlw	0X00		;si le pas est= a 0 
	XORWF	pas,w		;alors goto affi_10h
	PCLAX	affi_10h
	btfsc	STATUS,B_Z
	GOTSX 	affi_10h

	movlw	0X01
	XORWF	pas,w
	PCLAX	affi_100h
	btfsc	STATUS,B_Z
	GOTSX	affi_100h

	movlw	0X02
	XORWF	pas,w
	PCLAX	affi_1000h
	btfsc	STATUS,B_Z
	GOTSX 	affi_1000h

	movlw	0X03
	xorwf	pas,w
	PCLAX	affi_10khz
	btfsc	STATUS,B_Z
	GOTSX	affi_10khz
	

	;movlw	0X04
	;xorwf	pas,w
	;btfsc	STATUS,B_Z
;	GOTO	affi_100khz

	movlw	0Xff		;si pas = 0, on est bute basse  et dela de 10 hertz
	xorwf	pas,w
	PCLAX	pppas
	btfsc	STATUS,B_Z
	GOTSX	pppas

	movlw	0x04		;si le pas est = 0 ,on est en bute haute et au dela des 10 khz 
	XORWF	pas,w		;alors goto ppas
	PCLAX	ppas		
	btfsc	STATUS,B_Z
	GOTSX	ppas
pppas
	clrf	pas
	RETURN
ppas	
	decf	pas,f
		return
	



	
affi_10h	

	
	movlw   0xca             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD           ; 
	MOVLW	'1'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'0'
	movwf   LCD_char          ; 
	CALLX    data2LCD         ;

	MOVLW	'H'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'z'
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	GOTOX    go_affi           ;  use the 10 Hz step

affi_100h	           
	       ;
	movlw   0xca             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'1'
	movwf   LCD_char          ; 
	CALLX    data2LCD           ; 
	MOVLW	'0'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'0'
	movwf   LCD_char          ;
	CALLX    data2LCD
	
	MOVLW	'H'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'z'
	movwf   LCD_char          ; 
	CALLX    data2LCD            ; Use the 100Khz step	
	GOTOX    go_affi           ;  use the 100 Hz step


affi_1000h	           
      	
	movlw   0xca             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	MOVLW	'1'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'K'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'h'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'z'
	movwf   LCD_char          ; 
	CALLX    data2LCD     	  ;Use the 1000Hz step
	GOTOX    go_affi
	
affi_10khz
	movlw	0XCA
	CALLX	cmnd2LCD
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'1'
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	MOVLW	'0'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'K'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'h'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'z'
	movwf   LCD_char          ; 
	CALLX    data2LCD  	  ;Use the 10KHz step
	GOTOX    go_affi	          		      
go_affi	
	return

;********************************************************************
;**************  La touche RIT a ete actionne ****************************
;                 PASSAGE DE DIRECT VERS RIT
;****************************************************************************
WRITsimplex 

	MOVLW   0X28		;mettre la freq DIRECT affiche du lcd en  memoire
	CALLX	putfee
RIT 
	movlw   0x8B              ; endroit ou ecrire sur le lcd
    CALLX    cmnd2LCD
     	movlw   ' '               ; Send a R
	movwf   LCD_char          ; 
	CALLX    data2LCD        
	
	movlw   'R'               ; Send a R
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'I'               ; Send an I
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'T'               ; Send an T
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	 	               
	movlw	0x00
	;Bank1	
	movwf	tester_rit	; si = 0 indique qu'ont est en mode rit
	;	Bank0

		
	GOTOX	int_retour   ;	GOTSX	mainrit      
;  
;**********************************************************************	
;**************  La touche RIT a ete de nouveau actionne ****************************
;                   PASSAGE DE RIT vers DIRECT 
;****************************************************************************
RETrit
	 movlw 	0X2c		;mettre la freq regle sur la position RIT en memoire
   	CALLX	putfee
	  
	MOVLW   0X28             
	CALLX    getfee		;lire en memoire la freq DIRECT  
	
	
	
	CALLX    bin2BCD           ; Convert the frequency to BCD
	CALLX    show_freq         ; Display the frequency on the LCD
	CALLX    MEGAH 
	CALLX	newdan
	CALLX    calc_dds_word     ; Find the control word for the DDS chip
	CALLX    send_dds_word     ; Send the control word to the DDS chip	

	movlw	 0x01
		;Bank1
	movwf	tester_rit
	
;	Bank0
	 	
	GOTSX	int_retour 
      
;*****************************************************************************
;       
;       		la touche  EMISSION a ete actionne 
;****************************************************************************

emission  
	;Bank1
	PCLAX 	emi_direct
	BTFSC	tester_rit,0 ;si = 0 = la touche a ete active et on est position rit
	GOTSX	emi_direct ;si = 1 = on est position direct 
	;Bank0
     movlw 	0X2c		;mettre la freq regle sur la position RIT en memoire
   	CALLX	putfee

   	 
	MOVLW   0X28		;RAPPELen memoire 0x28 de la freq DIRECT 
	CALLX   getfee    
emi_direct	


	CALLX    bin2BCD           ; Convert the frequency to BCD
	CALLX    show_freq         ; Display the frequency on the LCD
	  	
	CALLX    MEGAH 
	CALLX    newdan  
	  
	CALLX    calc_dds_word     ; Find the control word for the DDS chip
	CALLX    send_dds_word     ; S 

;**************FAIRE COLLER LE RELAIS EMISSION******************************
	bcf	PortB,3
trx_en_emis 

 
	
	PCLAX	 trx_en_emis      
	BTFSS   PortB,2	  		;si = 0 GOTO SI = 1 SAUTE     
	GOTSX    trx_en_emis 
	
;**************FAIRE DECOLLER LE RELAIS EMISSION*****************************
;	Bank1
	PCLAX 	emi_vers_rec
	BTFSS	tester_rit,0 ;si = 0 = la touche a ete active et on est position rit
	GOTSX	emi_vers_rec ;si = 1 = on est position direct ALORS RETURN
	; 	Bank0 
	CALLX    MEGAH   
	
	bsf	PortB,3          ;DECOLLER LE RELAIS EMISSION
	return
	
emi_vers_rec	
	 movlw 	0X2c		;RAPPEL de la freq en memoire (regle enposition RIT) 
   	CALLX	getfee
   	
   	movlw   0x8B              ; endroit ou ecrire sur le lcd
    CALLX    cmnd2LCD 
    
    	movlw   ' '               ; Send a R
	movwf   LCD_char          ; 
	CALLX    data2LCD        
	
	movlw   'R'               ; Send a R
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'I'               ; Send an I
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'T'               ; Send an T
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	

	CALLX    bin2BCD           ; Convert the frequency to BCD
	CALLX    show_freq         ; Display the frequency on the LCD
	   	CALLX    newdan 
   
	CALLX    calc_dds_word     ; Find the control word for the DDS chip
	CALLX    send_dds_word 
	bsf	PortB,3
   	return

;*****************************************************************************


;               AFFICHAGE DE Mhz ( sur le LCD )
;       
MEGAH   movlw   0x8b             ; Point ou ecrire sur le LCD
 	movwf   LCD_char          ; 2 eme partie de la ligne
	CALLX    cmnd2LCD          ; 
	
	movlw   ' '               ; Send a M
	movwf   LCD_char  
	CALLX    data2LCD   
	movlw   'M'               ; Send a M
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'h'               ; Send an h
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'z'               ; Send an z
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	return
		
	   
;***************************************************************************

;        
; putee - a routine to write the data byte in w into eeprom 
putee 
 	Bank2
 ;bsf     STATUS,B_RP0      ; Switch to bank 1
	movwf   EEdata         ;save in the eeprom data register
	CALLX    write_EEPROM    ;write the band
		Bank0
;	bcf     STATUS,B_RP0      ; Switch back to bank 0
	return                  ;end of putee routine




;
; *****************************************************************************
; *                                                                           *
; * Purpose:  This routine is entered at start up if the push button is       *
; *           pressed.  "14,000.00 CAL" is displayed on the LCD, and the      *
; *           the DDS chip is programmed to produce 10 MHz, based on the      *
; *           osc value stored in the EEPROM.  As long as the button is       *
; *           pressed, the osc value is slowly altered to allow the output    *
; *           to be trimmed to exactly 10 MHz.  Once the encoder is turned    *
; *           after the button is released, the new osc value is stored in    *
; *           the EEPROM and normal operation begins.                         *
; *                                                                           *
; *   Input:  The original osc constant in EEPROM                             *
; *                                                                           *
; *  Output:  The corrected osc constant in EEPROM                            *
; *                                                                           *
; *****************************************************************************
;
calibrate
	movlw   0xd0;c0             ; Set frequency dds = 1.410.000 mgh
	movwf   freq_0           ;le vco  donne une freq de 14.100.000mgh  
	movlw   0x83;5c            ;  
	movwf   freq_1            ;  setting freq to the binary equivalent
	movlw   0x15;15             ;   . 
	movwf   freq_2		
	movlw	0x00              ;   .
	movwf   freq_3            ;   .
;
; Read the starting reference oscillator value from EEPROM.
;
	CALLX    getee
	
	;pour calibrate 14m

	movlw	0x00		    ;si =0 alors addition  si =1 alors soustraction
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0x60	     	; adresse du pic esclave
	movwf	adresse
	movlw	0X03		;valeur correspondant a 14 mgh pour le pic esclave de facon
	movwf	temp1		;qu'il configure le D/A et met les bonnes valeurs pour l'attenuateur pins 
	CALLX	envoie_i2c
	;GOTOX	i2c_continue
	CALLX	i2c_continue    ;il n'y a pas d'autre data
	;CALLX	newdan           ;get the osc data from eeprom and place in ram
	CALLX    bin2BCD           ; Calculate BCD version of 14,000.00
	CALLX    show_freq         ; Display the frequency on the LCD
	movlw   0x8c              ; ou ecrire sur le LCD
	movwf   LCD_char          ; 
	CALLX    cmnd2LCD          ; 
	movlw   'C'               ; Send a C
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'A'               ; Send an A
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'L'               ; Send an L
	movwf   LCD_char          ; 
	CALLX    data2LCD  
	
	movlw   0xc1              ; ou ecrire sur le LCD
	movwf   LCD_char          ; 
	CALLX    cmnd2LCD          ; 
	movlw   'U'               ; Send a U
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'S'               ; Send an S
	movwf   LCD_char          ; 
	CALLX    data2LCD          ; 
	movlw   'B'               ; Send an B
	movwf   LCD_char          ; 
	CALLX    data2LCD         ; 
	
		movf 	freq_0,w		; Copy current "freq" into "DDS_freq" variables
	movwf	DDS_freq_0		; to allow offset to be added without
	movf 	freq_1,w		; affecting display value which is based
	movwf	DDS_freq_1		; upon content of "freq"
	movf 	freq_2,w		
	movwf	DDS_freq_2		
	movf 	freq_3,w
	movwf	DDS_freq_3
	
	
cal_loop
	CALLX    calc_dds_word     ; Calculate DDS value based on current osc
	CALLX    send_dds_word     ; Update the DDS chip
	CALLX    send_dds_word
	;***********************************************
;poll_encoder_cal

	clrf    ren_timer_1       ; Put starting values in ren_timer
	movlw   0x40              ; Start with the high bit set in ren_timer
	movwf   ren_timer_0       ; 
read_encoder_cal
	clrwdt                    ; Reset the watchdog timer
	PCLAX	no_inc_cal 
	btfsc   ren_timer_1,7     ; Has the bit floated to bottom of ren_timer?
	GOTSX   no_inc_cal            ; Yes, don't move it any further
	movlw   0x08              ;
	addwf   ren_timer_0,f     ; 
	PCLAX	renn 
	btfsc   STATUS,B_C	 
	GOTSX	renn     ; Did the add force a carry?
	
	incf    ren_timer_1,f     ; Yes, then add one to ren_timer_1
renn	
no_inc_cal                            ; 

       
	movf   PortB,w				; PortA,w           ; Get the current encoder value
	movwf   ren_read          ; Save it
	movlw   0x03              ; Get encoder mask
	andwf   ren_read,w        ; Isolate encoder bits
	movwf   ren_new           ; Save new value
	xorwf   ren_old,w         ; Has it changed?
	PCLAX	read_encoder_cal 	
	btfsc   STATUS,B_Z        ; 
	GOTSX   read_encoder_cal      ; No, keep looking until it changes
;
;       Determine which direction the encoder turned.
	bcf     STATUS,B_C        ; Clear the carry bit
	rlf     ren_old,f         ; 
	movf    ren_new,w         ; 
	xorwf   ren_old,f         ; 
	movf    ren_old,w         ; 
	andlw   0x02              ; 
	movwf   next_dir          ; 
	xorwf   last_dir,w        ; 
;
;       Prevent encoder slip from giving a false change in direction.
	PCLAX	pe_continue_cal 
	btfsc   STATUS,B_Z        ; Zero?
	GOTSX   pe_continue_cal       ; No slip; keep going
	movf    next_dir,w        ; Yes, update direction
	movwf   last_dir          ; 
	movf    ren_new,w         ; Save the current encoder bits for next time
	movwf   ren_old           ; 
	GOTSX    read_encoder_cal      ; Try again
pe_continue_cal
	PCLAX	up2_cal  
	btfsc   ren_old,1         ; Are we going down?
	GOTSX    up2_cal               ; No, indicate we are going up
	clrf    last_dir          ; Yes, clear last_dir
	GOTSX    exit3_cal             ; Finish and return
up2_cal 
	movlw   0x02              ; Set UP value in last_dir
	movwf   last_dir          ; 
exit3_cal 
	movf    ren_new,w         ; Get the current encoder bits
	movwf   ren_old           ; Save them in ren_old for the next time
	;return                    ; Return to the CALLXer


	      ; Wait until the encoder has moved.
	clrf    fstep_3           ; Clear the three most significant
	clrf    fstep_2           ;   bytes of fstep
	clrf    fstep_1           ; 
	movlw   0x10              ; Assume that we are adjusting slowly
	movwf   fstep_0           ; Use small increment
	PCLAX	update_osc 
	btfsc   ren_read,2        ; Was the encoder changing slowly?
	GOTSX   update_osc        ; Yes, then continue with small increment
	movlw   0x80              ; No, then use the large increment
	movwf   fstep_0           ;
update_osc
	nop                       ; Wait a cycle
	PCLAX	faster   
	btfsc   last_dir,1        ; Are we moving down?
	GOTSX   faster            ; No, increase the osc value
;
;       slower
;
	comf    fstep_0,f         ; Subtraction of fstep is done by
	comf    fstep_1,f         ;   adding the twos compliment of fsetp
	comf    fstep_2,f         ;   to osc
	comf    fstep_3,f         ; 
	PCLAX	faster  
	incfsz  fstep_0,f         ; Increment last byte
	GOTOX    faster            ; Non-zero, continue
	PCLAX	faster  
	incfsz  fstep_1,f         ; Increment next byte
	GOTSX    faster            ; Non-zero, continue
	PCLAX	faster  
	incfsz  fstep_2,f         ; Increment next byte
	GOTSX   faster            ; Non-zero, continue
	incf    fstep_3,f         ; Increment the high byte
faster
	movf    fstep_0,w         ; Get the low byte increment
	addwf   osc_0,f           ; Add it to the low osc byte
	PCLAX	add4  
	btfss   STATUS,B_C        ; Was there a carry?
	GOTSX   add4              ; No, add the next bytes
	PCLAX	 add4
	incfsz  osc_1,f           ; Ripple carry up to the next byte
	GOTSX    add4              ; No new carry, add the next bytes
	PCLAX	 add4
	incfsz  osc_2,f           ; Ripple carry up to the next byte
	GOTSX    add4              ; No new carry, add the next bytes
	incf    osc_3,f           ; Ripple carry up to the highest byte
add4
	movf    fstep_1,w         ; Get the second byte increment
	addwf   osc_1,f           ; Add it to the second osc byte
	PCLAX	 add5
	btfss   STATUS,B_C        ; Was there a carry?
	GOTSX   add5              ; No, add the third bytes
	PCLAX	 add5
	incfsz  osc_2,f           ; Ripple carry up to the next byte
	GOTSX    add5              ; No new carry, add the third bytes
	incf    osc_3,f           ; Ripple carry up to the highest byte
add5
	movf    fstep_2,w         ; Get the third byte increment
	addwf   osc_2,f           ; Add it to the third osc byte
	PCLAX 	add6	
	btfss   STATUS,B_C        ; Was there a carry?
	GOTSX    add6              ; No, add the fourth bytes
	incf    osc_3,f           ; Ripple carry up to the highest byte
add6
	movf    fstep_3,w         ; Get the fourth byte increment
	addwf   osc_3,f           ; Add it to the fourth byte
	

	PCLAX	cal_loop
	BTFSs	PortB,7	;SI = O ALORS
;	btfss   ren_read,7;3        ; Is the button still pressed?
	GOTSX    cal_loop        ; Yes, stay in calibrate mode
	Bank2	;bsf     STATUS,B_RP0      ; Switch to bank 1
	movlw   d'124'          ;load up the start of the osc data in eeprom
	movwf  EEadr            ; Write final value to EEPROM 
	movf    osc_0,w           ; Record the first
		Bank2
	movwf   EEdata           ;   osc
	CALLX    write_EEPROM      ;   byte
	movf    osc_1,w           ; Record the secondBank2
	Bank2		
	movwf   EEdata           ;   osc
	CALLX    write_EEPROM      ;   byte
	movf    osc_2,w           ; Record the third
	Bank2
	movwf   EEdata           ;   osc
	CALLX    write_EEPROM      ;   byte
	movf    osc_3,w           ; Record the fourth
	Bank2
	movwf   EEdata           ;   osc
	CALLX    write_EEPROM      ;   byte
		Bank0;bcf     STATUS,B_RP0      ; Switch back to bank 0
	return                    ; Return to the CALLXer
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Multiply the 32 bit number for oscillator frequency times the   *
; *           32 bit number for the displayed frequency.                      *
; *                                                                           *
; *                                                                           *
; *   Input:  The reference oscillator value in osc_3 ... osc_0 and the       *
; *           current frequency stored in freq_3 ... freq_0.  The reference   *
; *           oscillator value is treated as a fixed point real, with a 24    *
; *           bit mantissa.                                                   *
; *                                                                           *
; *  Output:  The result is stored in AD9850_3 ... AD9850_0.                  *
; *                                                                           *
; *****************************************************************************
;
calc_dds_word
	clrf    AD9850_0          ; Clear the AD9850 control word bytes
	clrf    AD9850_1          ; 
	clrf    AD9850_2          ; 
	clrf    AD9850_3          ; 
	clrf    AD9850_4          ; 
	movlw   0x20              ; Set count  to 32   (4 osc bytes of 8 bits)
	movwf   mult_count        ; Keep running count
	movf    osc_0,w           ; Move the four osc bytes
	movwf   osc_temp_0        ;   to temporary storage for this multiply
	movf    osc_1,w           ; (Don't disturb original osc bytes)
	movwf   osc_temp_1        ; 
	movf    osc_2,w           ; 
	movwf   osc_temp_2        ; 
	movf    osc_3,w           ; 
	movwf   osc_temp_3        ; 
mult_loop
	bcf     STATUS,B_C        ; Start with Carry clear
	PCLAX	noAdd  
	btfss   osc_temp_0,0      ; Is bit 0 (Least Significant bit) set?
	GOTSX    noAdd             ; No, don't need to add freq term to total
	movf    DDS_freq_0,w          ; Yes, get the freq_0 term
	addwf   AD9850_1,f        ;   and add it in to total
	PCLAX	add7   
	btfss   STATUS,B_C        ; Does this addition result in a carry?
	GOTSX    add7              ; No, continue with next freq term
	incfsz  AD9850_2,f        ; Yes, add one and check for another carry
	GOTSX   add7              ; No, continue with next freq term
	incfsz  AD9850_3,f        ; Yes, add one and check for another carry
	GOTSX   add7              ; No, continue with next freq term
	incf    AD9850_4,f        ; Yes, add one and continue
add7
	movf    DDS_freq_1,w          ; Use the freq_1 term
	addwf   AD9850_2,f        ; Add freq term to total in correct positio
	PCLAX	add8  
	btfss   STATUS,B_C        ; Does this addition result in a carry?
	GOTSX    add8              ; No, continue with next freq term
	incfsz  AD9850_3,f        ; Yes, add one and check for another carry
	GOTSX   add8              ; No, continue with next freq term
	incf    AD9850_4,f        ; Yes, add one and continue
add8
	movf    DDS_freq_2,w          ; Use the freq_2 term
	addwf   AD9850_3,f        ; Add freq term to total in correct position
	btfss   STATUS,B_C        ; Does this addition result in a carry?
	GOTSX    add9              ; No, continue with next freq term
	incf    AD9850_4,f        ; Yes, add one and continue
add9
	movf    DDS_freq_3,w          ; Use the freq_3 term
	addwf   AD9850_4,f        ; Add freq term to total in correct position
noAdd
	rrf     AD9850_4,f        ; Shift next multiplier bit into position
	rrf     AD9850_3,f        ; Rotate bits to right from byte to byte
	rrf     AD9850_2,f        ; 
	rrf     AD9850_1,f        ; 
	rrf     AD9850_0,f        ; 
	rrf     osc_temp_3,f      ; Shift next multiplicand bit into position
	rrf     osc_temp_2,f      ; Rotate bits to right from byte to byte
	rrf     osc_temp_1,f      ; 
	rrf     osc_temp_0,f      ; 
	PCLAX	 mult_loop  
	decfsz  mult_count,f      ; One more bit has been done.  Are we done?
	GOTSX    mult_loop         ; No, go back to use this bit
	clrf    AD9850_4          ; Yes, clear _4.  Answer is in bytes _3 .. _0
	return                    ; Done.
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  This routine sends the AD9850 control word to the DDS chip      *
; *           using a serial data transfer.                                   *
; *                                                                           *
; *   Input:  AD9850_4 ... AD9850_0                                           *
; *                                                                           *
; *  Output:  The DDS chip register is updated.                               *
; *                                                                           *
; *****************************************************************************
;
send_dds_word
	movlw   AD9850_0          ; Point FSR at AD9850
	movwf   FSR               ; 
next_byte
	movf    INDF,w            ; 
	movwf   byte2send         ; 
	movlw   0x08              ; Set counter to 8
	movwf   bit_count         ; 
next_bit
	rrf     byte2send,f       ; Test if next bit is 1 or 0
	PCLAX	send0
	btfss   STATUS,B_C        ; Was it zero?
	GOTSX    send0             ; Yes, send zero
	bsf     PortE,2           ; No, send one
	bsf     PortE,1           ; Toggle write clock 
	bcf     PortE,1           ; 
	GOTOX    break             ; 
send0
	bcf     PortE,2           ; Send zero
	bsf     PortE,1          ; Toggle write clock
	bcf     PortE,1           ;
break
	PCLAX next_bit
	decfsz  bit_count,f       ; Has the whole byte been sent?
	GOTSX   next_bit          ; No, keep going.
	incf    FSR,f             ; Start the next byte unless finished
	movlw   AD9850_4+1        ; Next byte (past the end)
	subwf   FSR,w             ;
	PCLAX	next_byte 
	btfss   STATUS,B_C        ;
	GOTSX   next_byte         ;
	bsf     PortE,0           ; Send load signal to the AD9850
	bcf     PortE,0           ; 
	return                    ;
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  This subroutine converts a 32 bit binary number to a 10 digit   *
; *           BCD number.  The input value taken from freq(0 to 3) is         *
; *           preserved.  The output is in BCD(0 to 4), each byte holds =>    *
; *           (hi_digit,lo_digit), most significant digits are in BCD_4.      *
; *           This routine is a modified version of one described in          *
; *           MicroChip application note AN526.                               *
; *                                                                           *
; *   Input:  The value in freq_0 ... freq_3                                  *
; *                                                                           *
; *  Output:  The BCD number in BCD_0 ... BCD_4                               *
; *                                                                           *
; *****************************************************************************
;
bin2BCD
	movlw   0x20              ; Set loop counter
	movwf   BCD_count         ;   to 32
	clrf    BCD_0             ; Clear output
	clrf    BCD_1             ;   "     "
	clrf    BCD_2             ;   "     "
	clrf    BCD_3             ;   "     "
	clrf    BCD_4             ;   "     "
bin_loop
	bcf     STATUS,B_C        ; Clear carry bit in STATUS
;
; Rotate bits in freq bytes.  Move from LS byte (freq_0) to next byte (freq_1).
; Likewise, move from freq_1 to freq_2 and from freq_2 to freq_3.

	rlf     freq_0,f          ; Rotate left, 0 -> LS bit, MS bit -> Carry
	rlf     freq_1,f          ; Rotate left, Carry->LS bit, MS bit->Carry
	rlf     freq_2,f          ; Rotate left, Carry->LS bit, MS bit->Carry
	rlf     freq_3,f          ; Rotate left, Carry->LS bit, MS bit->Carry
	
	btfsc   STATUS,B_C        ; Is Carry clear? If so, skip next instruction
	bsf     freq_0,0          
;
	;rlf     AFFI_0,f          ; Rotate left, 0 -> LS bit, MS bit -> Carry
	;rlf     AFFI_1,f          ; Rotate left, Carry->LS bit, MS bit->Carry
	;rlf     AFFI_2,f          ; Rotate left, Carry->LS bit, MS bit->Carry
	;rlf     AFFI_3,f          ; Rotate left, Carry->LS bit, MS bit->Carry
	
	;btfsc   STATUS,B_C        ; Is Carry clear? If so, skip next instruction
	;bsf     AFFI_0,0          ; Carry is set so wrap and set bit 0 in freq_0
;
; Build BCD bytes. Move into LS bit of BCD bytes (LS of BCD_0) from MS bit of
; freq_3 via the Carry bit.  
;
	rlf     BCD_0,f           ; Rotate left, Carry->LS bit, MS bit->Carry
	rlf     BCD_1,f           ; Rotate left, Carry->LS bit, MS bit->Carry
	rlf     BCD_2,f           ; Rotate left, Carry->LS bit, MS bit->Carry
	rlf     BCD_3,f           ; Rotate left, Carry->LS bit, MS bit->Carry
	rlf     BCD_4,f           ; Rotate left, Carry->LS bit, MS bit->Carry
	decf    BCD_count,f       ; Decrement loop count
	PCLAX	adjust 
	btfss   STATUS,B_Z        ; Is loop count now zero?
	GOTSX    adjust            ; No, go to adjust
	return                    ; Yes, EXIT 
; ============================================================================
adjust  ; Internal subroutine, CALLXed by bin2BCD main loop only
; 
; As BCD bytes are being built, make sure the nibbles do not grow larger than 9. 
; If a nibble gets larger than 9, increment to next higher nibble.  
; (If the LS nibble of a byte overflows, increment the MS nibble of that byte.)
; (If the MS nibble of a byte overflows, increment the LS nibble of next byte.)
;
	movlw   BCD_0             ; Get pointer to BCD_0
	movwf   FSR               ; Put pointer in FSR for indirect addressing
	CALLX    adj_BCD           ; 
	incf    FSR,f             ; Move indirect addressing pointer to BCD_1
	CALLX    adj_BCD           ; 
	incf    FSR,f             ; Move indirect addressing pointer to BCD_2
	CALLX    adj_BCD           ; 
	incf    FSR,f             ; Move indirect addressing pointer to BCD_3
	CALLX    adj_BCD           ; 
	incf    FSR,f             ; Move indirect addressing pointer to BCD_4
	CALLX    adj_BCD           ; 
	GOTOX    bin_loop          ; Back to main loop of bin2BCD
; ============================================================================
adj_BCD  ; Internal subroutine, CALLXed by adjust only
	movlw   3                 ; Add 3
	addwf   INDF,w            ;   to LS digit
	movwf   BCD_temp          ; Save in temp
	btfsc   BCD_temp,3        ; Is LS digit + 3 > 7  (Bit 3 set)
	movwf   INDF              ; Yes, save incremented value as LS digit
	movlw   0x30              ; Add 3
	addwf   INDF,w            ;   to MS digit
	movwf   BCD_temp          ; Save as temp
	btfsc   BCD_temp,7        ; Is MS digit + 3 > 7  (Bit 7 set)
	movwf   INDF              ; Yes, save incremented value as MS digit
	return                    ; Return to adjust subroutine
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Display the frequency setting on the LCD.                       *
; *                                                                           *
; *   Input:  The values in BCD_4 ... BCD_0                                   *
; *                                                                           *
; *  Output:  The number displayed on the LCD                                 *
; *                                                                           *
; *****************************************************************************
;
show_freq
	movlw   0x80              ; Point the LCD to first LCD digit location
	CALLX    cmnd2LCD          ; Send starting digit location to LCD
;
; Running 4-bit mode, so need to send Most Significant Nibble first.
;
; Extract and send "XXXX" from byte containing "XXXXYYYY"
;  - Swap halves to get YYYYXXXX
;  - Mask with 0x0F to get 0000XXXX
;  - Add ASCII bias (0030XXXX)
;
	swapf   BCD_3,w           ; Swap 100MHz BCD digit into lower nibble of W
	andlw	0x0f			;voir si ce digit est un zero si oui mettre un
	PCLAX	pas_zero	
	btfss	STATUS,B_Z		;espace a la place
	GOTSX	pas_zero
	movlw	0x20
	CALLX	data2LCD
	GOTOX	next_digit
pas_zero
	;andlw   0x0F              ; Mask for lower nibble only       (0000XXXX)
	addlw   0x30              ; Add offset for ASCII char set    (0030XXXX)
	CALLX    data2LCD          ; Send byte in W to LCD
;
; Extract and send "YYYY" from byte containing "XXXXYYYY"
;   - Mask with 0x0F to get 0000YYYY
;   - Add offset for ASCII character set in LCD  (0030YYYY)
;
next_digit	
	movf    BCD_3,w           ; Put 1MHz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000YYYY)
								;voir si ce digit est un zero si oui mettre un
	PCLAX	no_zero								
	btfss	STATUS,B_Z		;espace a la place
	GOTSX	no_zero
	movlw	0x20
	CALLX	data2LCD
	GOTOX	autre_digit
no_zero
	addlw   0x30              ; Add offset for ASCII char set    (0030YYYY)
	CALLX    data2LCD          ; Send byte in W to LCD
; 
autre_digit
	swapf   BCD_2,w           ; Swap 100KHz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000XXXX)
	addlw   0x30              ; Add offset for ASCII char set    (0030XXXX)
	CALLX    data2LCD          ; Send byte in W to LCD
;
	movlw   ','               ; Get a comma
	CALLX    data2LCD          ; Send byte in W to LCD

	movf    BCD_2,w           ; Put 10KHz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000YYYY)
	addlw   0x30              ; Add offset for ASCII char set    (0030YYYY)
	CALLX    data2LCD          ; Send byte in W to LCD
;

	swapf   BCD_1,w           ; Swap 1KHz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000XXXX)
	addlw   0x30              ; Add offset for ASCII char set    (0030XXXX)
	CALLX    data2LCD          ; Send byte in W to LCD
;

	;movlw   0xC0              ; Point to LCD digit number nine
	;CALLX    cmnd2LCD          ; Send command byte in W to LCD
;
	movf    BCD_1,w           ; Put 100 Hz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000YYYY)
	addlw   0x30              ; Add offset for ASCII char set    (0030YYYY)
	CALLX    data2LCD          ; Send data byte in W to LCD
;
	movlw   '.'               ; Set up W with ASCII Period 
	CALLX    data2LCD          ; Send data byte in W to LCD
;

	swapf   BCD_0,w           ; Swap 10 Hz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000XXXX)
	addlw   0x30              ; Add offset for ASCII char set    (0030XXXX)
	CALLX    data2LCD          ; Send data byte in W to LCD
;
	movf    BCD_0,w           ; Put 1 Hz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000YYYY)
	addlw   0x30	
	CALLX    data2LCD          ; Send byte in W to LCD
	
	movlw    0x00           ; Put 1 Hz BCD digit into lower nibble of W
	andlw   0x0F              ; Mask for lower nibble only       (0000YYYY)
	addlw   0x30     
	CALLX    data2LCD          ;   to LCD
	
;


;	movlw   0xC0              ; Point to LCD digit number nine
	;CALLX    cmnd2LCD          ; Send command byte in W to LCD
	
	
	 
	return                    ; 
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Check if LCD is done with the last operation.                   *
; *           This subroutine polls the LCD busy flag to determine if         *
; *           previous operations are completed.                              *
; *                                                                           *
; *   Input:  None                                                            *
; *                                                                           *
; *  Output:  PortB set as:  RB7..RB4 inputs                                  *
; *                          RB3..RB0 outputs                                 *
; *****************************************************************************
; 
busy_check
	clrf    PortD             ; Clear all outputs on PortB
	bsf     STATUS,B_RP0      ; Switch to bank 1 for Tristate operation
	movlw   b'11110000'       ; Set RB7..RB4 as inputs, RB3..RB0 outputs
	movwf   TRISD             ;   via Tristate
	bcf     STATUS,B_RP0      ; Switch back to bank 0
	bcf     PortD,LCD_rs      ; Set up LCD for Read Busy Flag (RS = 0) 
	bsf     PortD,LCD_rw      ; Set up LCD for Read (RW = 1)  
	movlw   0xFF              ; Set up constant 255
	movwf   timer1            ;   for timer loop counter
LCD_is_busy
	bsf     PortD,LCD_e       ; Set E high
	movf    PortD,w           ; Read PortB into W
	movwf   LCD_read          ; Save W for later testing
	bcf     PortD,LCD_e       ; Drop E again
	nop                       ; Wait a
	nop                       ;   while
	bsf     PortD,LCD_e       ; Pulse E high (dummy read of lower nibble),
	nop                       ;   wait,
	bcf     PortD,LCD_e       ;   and drop E again
	decf    timer1,f          ; Decrement loop counter
	PCLAX	 not_busy
	btfsc   STATUS,B_Z        ; Is loop counter down to zero?
	GOTSX     not_busy     ; If yes, return regardless
	PCLAX	LCD_is_busy
	btfsc   LCD_read,7        ; Is Busy Flag (RB7) in save byte clear?
	GOTSX   LCD_is_busy       ; If not, it is busy so jump back
not_busy
	return                    ; 
;
; *****************************************************************************
; * Purpose:  Send Command or Data byte to the LCD                            *
; *           Entry point cmnd2LCD:  Send a Command to the LCD                *
; *           Entry Point data2LCD:  Send a Data byte to the LCD              *
; *                                                                           *
; *   Input:  W has the command or data byte to be sent to the LCD.           *
; *                                                                           *
; *  Output:  None                                                            *
; *****************************************************************************
;
cmnd2LCD   ; ****** Entry point ******
	movwf   LCD_char          ; Save byte to write to LCD
	clrf    rs_value          ; Remember to clear RS  (clear rs_value)   
	bcf     PortD,LCD_rs      ; Set RS for Command to LCD
	GOTOX    write2LCD         ; Go to common code
data2LCD   ; ****** Entry point ********
	movwf   LCD_char          ; Save byte to write to LCD
	bsf     rs_value,0        ; Remember to set RS (set bit 0 of rs_value)
	bsf     PortD,LCD_rs      ; Set RS for Data to LCD
write2LCD
	CALLX    busy_check        ; Check to see if LCD is ready for new data
	clrf    PortD             ; Clear all of Port B (inputs and outputs)
	bsf     STATUS,B_RP0      ; Switch to bank 1 for Tristate operation
	movlw   0x00              ; Set up to enable PortB data pins
	movwf   TRISD             ; All pins (RB7..RB0) are back to outputs
	bcf     STATUS,B_RP0      ; Switch to bank 0
	bcf     PortD,LCD_rw      ; Set LCD back to Write mode  (RW = 0)
	bcf     PortD,LCD_rs      ; Guess RS should be clear              
	btfsc   rs_value,0        ; Should RS be clear?  (is bit 0 == 0?) 
	bsf     PortD,LCD_rs      ; No, set RS                            
;
; Transfer Most Significant nibble  (XXXX portion of XXXXYYYY)
;
	movlw   0x0F              ; Set up mask                                 
	andwf   PortD,f           ; Clear old RB7..RB4 
	movf    LCD_char,w        ; Put byte of data into W
	andlw   0xF0              ; Mask to give XXXX0000 in W
	iorwf   PortD,f           ; Send to RB7..RB4 without changing RB3..RB0
	bsf     PortD,LCD_e       ; Pulse the E line high,
	nop                       ;   wait, 
	bcf     PortD,LCD_e       ;   and drop it again
;
; Transfer Least Significant nibble  (YYYY portion of XXXXYYYY)
;
	movlw   0x0F              ; Set up mask                                 
	andwf   PortD,f           ; Clear old RB7..RB4
	swapf   LCD_char,w        ; Move LS nibble of data to MS position in W
	andlw   0xF0              ; Mask to give YYYY0000 in W
	iorwf   PortD,f           ; Send to RB7..RB4 without changing RB3..RB0
	bsf     PortD,LCD_e       ; Pulse the E line high,
	nop                       ;   wait, 
	bcf     PortD,LCD_e       ;   and drop it again
	return
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Write the byte of data at EEDATA to the EEPROM at address       *
; *           EEADR.                                                          *
; *                                                                           *
; *   Input:  The values at EEDATA and EEADR.                                 *
; *                                                                           *
; *  Output:  The EEPROM value is updated.                                    *
; *                                                                           *
; *****************************************************************************
	
;******************lire l'eeprom et mettre data dans w********************
read_EEPROM
	Bank3
	bcf	EEcon1,EEPGD	  ; Point to Data Memory
        bsf     EEcon1,RD         ; Request the read
	Bank2
        incf    EEadr,f           ; Increment the read address
       	movf    EEdata,w          ; get the eeprom data byte, place in w
	Bank0
	return

; *****************************************************************************
; *                                                                           *
; * Purpose:  Write the byte of data at EEdata to the EEPROM at address       *
; *           EEadr. Based upon 16F877 Datasheet example.                     *
; *                                                                           *
; *   Input:  The values at EEdata and EEadr.                                 *
; *                                                                           *
; *  Output:  The EEPROM value is updated.                                    *
; *                                                                           *
; *****************************************************************************
;
write_EEPROM
	Bank3
	btfsc	EEcon1,WR	  ; Wait for write to finish
	goto	$-1		  ; Jump back one
	bcf	EEcon1,EEPGD	  ; Point to Data Memory
	bsf	EEcon1,WREN	  ; Enable Write
	
	;bcf	INTCON,GIE
        movlw   0x55              ; Write 0x55 and 0xAA to EEPROM
        movwf   EEcon2            ; control register2, as required
        movlw   0xAA              ; for the write
        movwf   EEcon2            ; 
        bsf     EEcon1,WR         ; Set WR to initiate write
   	;bsf	INTCON,GIE
bit_check
        btfsc   EEcon1,WR         ; Has the write completed?
        goto    bit_check         ; No, keep checking
        bcf     EEcon1,WREN       ; Clear the EEPROM write enable bit
	Bank2
        incf    EEadr,f           ; Increment the EE write address
	Bank0
        return                    ; Return to the CALLXer
;
; *****************************************************************************
;	
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Wait for a specified number of milliseconds.                    *
; *                                                                           *
; *           Entry point wait_128ms:  Wait for 128 msec                      *
; *           Entry point wait_64ms :  Wait for 64 msec                       *
; *           Entry point wait_32ms :  Wait for 32 msec                       *
; *           Entry point wait_16ms :  Wait for 16 msec                       *
; *           Entry point wait_8ms  :  Wait for 8 msec                        *
; *                                                                           *
; *   Input:  None                                                            *
; *                                                                           *
; *  Output:  None                                                            *
; *                                                                           *
; *****************************************************************************
;
wait_128ms  ; ****** Entry point ******    
	movlw   0xFF              ; Set up outer loop 
	movwf   timer1            ;   counter to 255
	GOTOX    outer_loop        ; Go to wait loops
wait_64ms  ; ****** Entry point ******     
	movlw   0x80              ; Set up outer loop
	movwf   timer1            ;   counter to 128
	GOTOX    outer_loop        ; Go to wait loops
wait_32ms   ; ****** Entry point ******    
	movlw   0x40              ; Set up outer loop
	movwf   timer1            ;   counter to 64
	GOTOX   outer_loop        ; Go to wait loops
wait_16ms   ; ****** Entry point ******    
	movlw   0x20              ; Set up outer loop
	movwf   timer1            ;   counter to 32  
	GOTOX    outer_loop        ; Go to wait loops
wait_8ms   ; ****** Entry point ******     
	movlw   0x10              ; Set up outer loop
	movwf   timer1            ;   counter to 16
				  ; Fall through into wait loops
wait_1ms   ; ****** Entry point ******     
        movlw   0x02              ; Set up outer loop
        movwf   timer1            ;   counter to 1				  
;
; Wait loops used by other wait routines
;  - 1 microsecond per instruction (with a 4 MHz microprocessor crystal)
;  - 510 instructions per inner loop
;  - (Timer1 * 514) instructions (.514 msec) per outer loop
;  - Round off to .5 ms per outer loop
;
outer_loop                        
	movlw   0xFF              ; Set up inner loop counter
	movwf   timer2            ;   to 255
inner_loop
	PCLAX	inner_loop  	
	decfsz  timer2,f          ; Decrement inner loop counter
	GOTSX    inner_loop        ; If inner loop counter not down to zero, 
				  ;   then go back to inner loop again
	PCLAX	outer_loop			  
	decfsz  timer1,f          ; Yes, Decrement outer loop counter
	GOTSX   outer_loop        ; If outer loop counter not down to zero,
				  ;   then go back to outer loop again
	return                    ; Yes, return to CALLXer
;       
; *****************************************************************************
;
tempo_i2c
		MOVLW   0X0F
		MOVWF   timer1
interne 
	PCLAX	interne		
		DECFSZ   timer1,f
		GOTSX    interne
		 return
;***************************************************************************

	 
;****************************************************************************
; S/P de commutation des pins du PCF8574A et des relais en fonction des bandes
;****************************************************************************
BUS_I2C	

	;return
;bande 3.5
	movlw	0x76
	movwf	adresse
	
	movlw	0X00		
	SUBWF	band,w
	PCLAX	pour_04	
	btfss	STATUS,B_Z	;si  0X00 - band = 0 alors
	GOTSX	pour_04		;sinon saut a pour_04
	movlw	0X81		;mettre le bit 1 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c

	CALLX	i2c_continue    ;il n'y a pas d'autres data
pour_04
;bande 7

	movlw	0X04
	SUBWF	band,w
	PCLAX	pour_08	
	btfss	STATUS,B_Z
	GOTSX	pour_08
	movlw	0x40
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	CALLX i2c_continue

pour_08
;bande 10.1
	movlw	0X08
	SUBWF	band,w
	PCLAX	pour_0c	
	btfss	STATUS,B_Z
	GOTSX	pour_0c
	movlw	0X03
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	CALLX i2c_continue
	
pour_0c
;bande 14
	movlw	0X0c
	SUBWF	band,w
	PCLAX	pour_10
	btfss	STATUS,B_Z
	GOTSX	pour_10
	movlw	0X02
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	CALLX i2c_continue
pour_10
;bande 18.068
	movlw	0X10
	SUBWF	band,w
	PCLAX	pour_14
	btfss	STATUS,B_Z
	GOTSX	pour_14
	movlw	0X28
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	CALLX i2c_continue
pour_14
;bande 21
	movlw	0X14
	SUBWF	band,w
	PCLAX	pour_18
	btfss	STATUS,B_Z
	GOTSX	pour_18
	movlw	0X08
	movwf	temp1
		;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	CALLX i2c_continue
pour_18
;bande 24.890
	movlw	0X18
	SUBWF	band,w
	PCLAX	pour_1c
	btfss	STATUS,B_Z
	GOTSX	pour_1c
	movlw	0X10
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	goto	i2c_continue
pour_1c
;bande 28
	movlw	0X1c
	SUBWF	band,w
	PCLAX	pour_20	
	btfss	STATUS,B_Z
	GOTSX	pour_20
	movlw	0X30
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	goto	i2c_continue
pour_20
;bande 50
	movlw	0X20
	SUBWF	band,w
	PCLAX	pour_24
	btfss	STATUS,B_Z
	GOTSX	pour_24
	movlw	0X04
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	goto	i2c_continue
pour_24
;bande 1.8
	movlw	0X24
	SUBWF	band,w
	PCLAX	pour_nop
	btfss	STATUS,B_Z
	GOTSX	pour_nop
		nop 
			
	movlw	0X80
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c
	goto	i2c_continue
pour_nop			
	return
;*****************************************************************************

ADD_U17		  ;sous programme de gestion du ci pcf8574a (u17) en mode add
	addwf	temp2,W ;de cette faon les pins s du ci peuvent etre commandes
	movwf	temp2   ; individuelement tout en laissant dans leurs etats les autres pins 
	movwf	temp1	;programmees prcdamment
	movlw	0x72
	movwf	adresse
	CALLX	envoie_i2c
	goto	i2c_continue
	return		

;***************************************************************************
SOUS_U17		     ;sous programme de gestion du ci pcf8574a (u17) en mode soustration
	SUBWF	temp2,W  ;de cette faon les pins s du ci peuvent etre commandes
	movwf	temp2    ; individuelement tout en laissant dans leurs etats les autres pins
	movwf	temp1 	 ;programmees prcdamment
	movlw	0x72
	movwf	adresse
	CALLX	envoie_i2c
	goto	i2c_continue
	return	
	
	

;****************************************************************************



					
envoie_i2c	;
	
				
		bsf     STATUS,B_RP0
		
		movlw 	b'00000000'	  ; Set Port C 0, as OUTput(
		movwf   TRISC
	
		bcf     STATUS,B_RP0

		BSF     PortC,data_a              
		CALLX    tempo_i2c
		BSF       PortC,clock_c               
		CALLX    tempo_i2c
		BCF     PortC,data_a         ;condition start i2c, data descend
		CALLX    tempo_i2c            ;quand clock_c_cest haut
		BCF     PortC,clock_c     

		CALLX    tempo_i2c
		
		movfw	adresse			
		MOVWF   temp 
		CALLX    envoie_octet ;ENVOI 8bits adresse byte 1 
			 		
		CALLX    ack
	
autre_bit				
		movf	temp1,w		  ;data
		MOVWF   temp
		CALLX     envoie_octet	
		CALLX    ack
			RETURN
i2c_continue		
		

		BCF      PortC,clock_c
		CALLX    tempo_i2c
		BCF  	 PortC,data_a
		CALLX    tempo_i2c
		BSF     PortC,clock_c       ;condition stop du protocole i2c
		CALLX    tempo_i2c  ;data remonte pendant que clock_cest haut.
		BSF  	PortC,data_a
		CALLX    tempo_i2c
		
      
		bsf     STATUS,B_RP0
		MOVLW   0X00
		MOVWF   TRISC ;ra0,1,2,3,4  =ENTREE 
		bcf     STATUS,B_RP0   ;TRISC en in
		
	
	
		;MOVLW   0XEF 
		;MOVWF   PortA           ;PROT_EMIS=0 le reste de RA4 a 1
		
			return
;__________________________________________________________________________
;

envoie_octet 
	 
		MOVLW   0X08
		MOVWF   timer2
	       BCF      PortC,clock_c   
	       	CALLX   tempo_i2c
	      
decal_gauche
		PCLAX	_HIGH	
		btfsc   temp,7   
		GOTSX    _HIGH
		BCF      PortC,data_a
		GOTOX   _LOW
	
       
_HIGH           
		BSF     PortC,data_a
_LOW            
		CALLX    tempo_i2c
		BSF      PortC,clock_c
		CALLX    tempo_i2c
 		BCF      PortC,clock_c
		CALLX    tempo_i2c
		rlf      temp,f
		PCLAX	decal_gauche
		DECFSZ  timer2,f
		GOTSX   decal_gauche  ;LE CONTENU DE TIMER2 EST
				      ;DECREMENTE DE 1 SI RESULTAT DIFFERENT
				      ;DE ZERO SAUTE A DECAL_GAUCHE

				    
		BCF      PortC,clock_c
    	CALLX     tempo_i2c
		return

ack            
        BCF      PortC,clock_c
		CALLX     tempo_i2c
		BSF       PortC,data_a
		CALLX     tempo_i2c
		BSF      PortC,clock_c
		CALLX     tempo_i2c
		BCF      PortC,clock_c
		CALLX     tempo_i2c
		return
;*********************************************************************
;***************communication des 2 pics  **************************
;****************************************************************************
; S/P de commutation des pins du PIC_ecclave en fonction des bandes
;****************************************************************************

PIC_esclave	
	movlw	0x60
	movwf	adresse
	
	movlw	0x00		;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X00		
	SUBWF	band,w
	PCLAX	pour_7m		
	btfss	STATUS,B_Z	; si  0X00 - band = 0 alors envoie_i2c
	GOTSX	pour_7m		; sinon saut a pour_07
	movlw	0X00		; valeur envoyer a l'esclave qui servira de ref
	movwf	temp1		; pour la commutation des capas sur la self du vco 
						; mais aussi comme adresse eeprom pour la mise en
						; memoire eeprom du CNA(attenuateur a diode pin)
	CALLX	envoie_i2c		
	goto	i2c_continue					 
pour_7m

	movlw	0x00		;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X04
	SUBWF	band,w
	PCLAX	pour_10m
	btfss	STATUS,B_Z
	GOTSX	pour_10m
	movlw	0x01
	movwf	temp1
   ;GOTSX	envoie_i2c
    CALLX	envoie_i2c		
	goto	i2c_continue

pour_10m

	movlw	0x00		;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X08
	SUBWF	band,w
	PCLAX	pour_14m
	btfss	STATUS,B_Z
	GOTSX	pour_14m
	movlw	0X02
	movwf	temp1
	;GOTSX	envoie_i2c
	CALLX	envoie_i2c		
	goto	i2c_continue
pour_14m

	movlw	0x00		;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X0c
	SUBWF	band,w
	PCLAX	pour_18m
	btfss	STATUS,B_Z
	GOTSX	pour_18m
	movlw	0X03
	movwf	temp1
	;GOTSX	envoie_i2c
	CALLX	envoie_i2c		
	goto	i2c_continue
pour_18m

	movlw	0x00		;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X10
	SUBWF	band,w
	PCLAX	pour_21m
	btfss	STATUS,B_Z
	GOTSX	pour_21m
	movlw	0X04
	movwf	temp1
	;GOTSX	envoie_i2c
	CALLX	envoie_i2c		
	goto	i2c_continue
pour_21m

	movlw	0x00;1		;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X14
	SUBWF	band,w
	PCLAX	pour_24m
	btfss	STATUS,B_Z
	GOTSX	pour_24m
	movlw	0X05
	movwf	temp1
	;GOTSX	envoie_i2c
	CALLX	envoie_i2c		
	goto	i2c_continue
pour_24m

	movlw	0x00;1		;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X18
	SUBWF	band,w
	PCLAX	pour_28m
	btfss	STATUS,B_Z
	GOTSX	pour_28m
	movlw	0X06
	movwf	temp1
;GOTSX	envoie_i2c;
	CALLX	envoie_i2c		
	goto	i2c_continue
pour_28m

	movlw	0x00	    ;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X1c
	SUBWF	band,w
	PCLAX	pour_50m
	btfss	STATUS,B_Z
	GOTSX	pour_50m
	movlw	0X07  
	movwf	temp1
	;GOTSX	envoie_i2c
	CALLX	envoie_i2c		
	goto	i2c_continue
pour_50m

	movlw	0x01	    ;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X20
	SUBWF	band,w
	PCLAX	pour_1m8
	btfss	STATUS,B_Z
	GOTSX	pour_1m8
	movlw	0X08  
	movwf	temp1
	;GOTOX	envoie_i2c
	CALLX	envoie_i2c		
	goto	i2c_continue
pour_1m8

	movlw	0x00	    ;si =0 alors add si =1 alors soustrac
	movwf	add_sous_affi	;variable pour OSC- FI 

	movlw	0X24
	SUBWF	band,w
	PCLAX	nopour
	btfss	STATUS,B_Z
	GOTSX	nopour
nopour
		
	nop   		
	movlw	0X09  
	movwf	temp1
	;GOTOX	envoie_i2c		
	CALLX	envoie_i2c		
	goto	i2c_continue
	return
;*******************************
; getee - a routine to get the 4 dds oscillator data bytes from eeprom
;               and place them into osc_0,1,2,3
getee
	 Bank2 ; bsf     STATUS,B_RP0      ; Switch to bank 1
	movlw   d'124'          ;load up the start of the osc data bytes
	movwf  EEadr            ; set the EEPROM read address
	CALLX    read_EEPROM       ; Read EEPROM
	movwf   osc_0             ; Save osc frequency
	CALLX    read_EEPROM       ; Get next byte
	movwf   osc_1             ; Save it
	CALLX    read_EEPROM       ; Get the third byte
	movwf   osc_2             ; Save it
	CALLX    read_EEPROM       ; Get the fourth byte
	movwf   osc_3             ; Save it
     Bank0 ; Switch back to bank 0
	return                  ;end of getee routine
;******************************************************************************
; - a routine to get the 4 dds oscillator data bytes from eeprom
;               and place them into osc_0,1,2,3
geteeflash
	 Bank2 ; bsf     STATUS,B_RP0      ; Switch to bank 1
	movlw   0x1f ;d'124'          ;load up the start of the osc data bytes
	movwf  EEadrh          ; set the FLASH read addres
	movlw 0x00
	movwf	EEadr
	CALLX    readflach       ; Read en flash
	movwf   osc_0             ; Save osc frequency
	CALLX    readflach; Get next byte
	movwf   osc_1             ; Save it
	CALLX    readflach; Get the third byte
	movwf   osc_2             ; Save it
	CALLX    readflach       ; Get the fourth byte
	movwf   osc_3             ; Save it
     Bank0 ; Switch back to bank 0
	return                  ;end of getee routine
;**********************************************************************
;s/p lecture en flash
readflach
    Bank3
	bsf		EEcon1,EEPGD
	bsf		EEcon1,RD
    nop
    nop
	 Bank2
	incf    EEadr,f	   
	 movf	EEdata,W	  
	 Bank0
	 RETURN	 
;**********************************************************************
;s/p ecriture en flash
writeflash
	Bank3
	bsf		EEcon1,EEPGD
	bsf		EEcon1,WREN
	bcf		INTCON,GIE
	movlw	 0x55
	movwf	EEcon2
	movlw	0xaa
	movwf	EEcon2
	bsf		EEcon1,WR
	nop
	nop
	bsf		INTCON,GIE
	bcf		EEcon1,WREN
	 Bank2
		incf    EEadr,f	
	 	Bank0
	 return	 	 	 
;************************************************************************
;ECRIRE DANS LA FLASH
putflash
    Bank2 ;             ; Switch to bank 1
	movlw   0x1f ;'          ;load pour write data bytes
	movwf  EEadrh          ; set the FLASH read addres
	movlw 0x00
	movwf	EEadr
	
	movf    freq_3,w     
		Bank2	
	movwf   EEdata           ; save the first osc byte
	CALLX   writeflash      ; write EEPROM

	movf    freq_2,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    writeflash      ; write next byte

	movf    freq_1,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    writeflash      ; write the third byte

	movf    freq_0,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    writeflash     ; write the fourth byte
		Bank0
	return   


;***********************************************************************
; getband - get the band stored in eeprom, return with band in w
getband	
	Bank2	;	 bsf     STATUS,B_RP0      ; Switch to bank 1
	movlw   d'120'            ;address of last band setting
	movwf  EEadr            ; set the EEPROM read address
	CALLX    read_EEPROM       ; Read EEPROM

	movwf   band            ;load up the last band
		Bank0
	;bcf     STATUS,B_RP0      ; Switch back to bank 0
	return                  ;end of getband routine
;**********************************************************************

	

;
; getfee - a routine to get the 4 frequency data bytes from eeprom
;               and place them into freq_0,1,2,3
getfee
;       CALLX with the first address of the data in w
   	 Bank2    ;bsf     STATUS,B_RP0      ; Switch to bank 1
	movwf  EEadr            ; set the EEPROM read address
	CALLX    read_EEPROM       ; Read EEPROM
	movwf   freq_3            ; Save osc frequency
  
	CALLX    read_EEPROM       ; Get next byte
	movwf   freq_2            ; Save it
 
	CALLX    read_EEPROM       ; Get the third byte
	movwf   freq_1
 		
	CALLX    read_EEPROM       ; Get the fourth byte
	movwf   freq_0            ; Save it

	Bank0		; Switch back to bank 0
	return                  ;end of getfee routine
;*******************************************************************************


;************************************************************************       
; putband - a routine to write the current band into eeprom location 120
putband	
	Bank2; bsf     STATUS,B_RP0      ; Switch to bank 1
	movf    band,w          ;load the current band
	;	Bank2
	movwf   EEdata         ;save in the eeprom data register
	movlw   d'120'          ;load the address of the band
		;Bank2
	movwf  EEadr          ;save in the eeprom address register
	CALLX    write_EEPROM    ;write the band
		Bank0
;	bcf     STATUS,B_RP0      ; Switch back to bank 0
	return                  ;end of putband routine



;
; putfee - a routine to write the 4 dds frequency data bytes from freq_0,1,2,3
;       to eeprom 
;       CALLX with the first address of the data in w
putfee
	Bank2 ;bsf     STATUS,B_RP0      ; Switch to bank 1
	movwf  EEadr            ; Reset the EEPROM read address
	movf    freq_3,w          ; load osc frequency
		Bank2	
	movwf   EEdata           ; save the first osc byte
	CALLX   write_EEPROM      ; write EEPROM

	movf    freq_2,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write next byte

	movf    freq_1,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write the third byte

	movf    freq_0,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write the fourth byte
		Bank0
;	bcf     STATUS,B_RP0      ; Switch back to bank 0
	return                  ;end of putfee routine
	;RETURN
;****************************************************************************



LCDcls1  movlw 0x10            ; effacage de la premire ligne
         movwf  LCDtmp
         movlw  0x80
       	CALLX    cmnd2LCD 
ENC       	 
         movlw  ' '
       	movwf   LCD_char          ; 
		CALLX    data2LCD 
		PCLAX	 ENC        
         decfsz LCDtmp,f
         GOTSX   ENC
	return
       
; *****************************************************************************
;LCDcls2  movlw  H'10'             ; effacage de la deuxieme ligne
        ; movwf  LCDtmp
        ; movlw  H'C0'
       	;CALLX    cmnd2LCD  
        ; movlw  '2'
       	;movwf   LCD_char          ; 
		;CALLX    data2LCD 
		;PCLAX    $-4      
        ; decfsz LCDtmp,f
        ; GOTSX   $-4
;	return
;****************************************************************************
;LCDcls3  movlw  H'10'             ; effacage de la troisime ligne
         ;movwf  LCDtmp
        ; movlw  H'90'
       	;CALLX    cmnd2LCD  
        ; movlw  '3'
       	;movwf   LCD_char          ; 
	;	CALLX    data2LCD 
	;PCLAX     $-4     
        ; decfsz LCDtmp,f
         ;GOTSX   $-4
	;return
;****************************************************************************
;LCDcls4  movlw  H'10'             ; effacage de la quatrime ligne
        ; movwf  LCDtmp
         ;movlw  H'D0'
       	;CALLX    cmnd2LCD  
        ; movlw  '4'
       	;movwf   LCD_char          ; 
	;	CALLX    data2LCD 
		;PCLAX	  $-4        
        ; decfsz LCDtmp,f
        ; GOTSX   $-4
	;return
;****************************************************************************
;explo_clavier
intrb4

 	Bank0	
	movlw	b'01111111'
	movwf     PortC
	
	movf	PortB,w	
	MOVWF	tester	

	CALLX	pause

	PCLAX	PPAS_PLUS	
	BTFSC	tester,7	;SI = O ALORS CALL UN
	GOTSX	PPAS_PLUS
	GOTOX	PAS_PLUS
	
PPAS_PLUS
	PCLAX	PPAS_MOINS
	BTFSC	tester,6	;SI = O ALORS CALL UN
	GOTSX	PPAS_MOINS
	GOTOX	PAS_MOINS
	
PPAS_MOINS
	PCLAX	BBAND_PLUS
	BTFSC	tester,5	;SI = O ALORS CALL UN
	GOTSX	BBAND_PLUS
	GOTOX	CAG
	
BBAND_PLUS
	PCLAX	BBAND_MOINS		
	BTFSC	tester,4	;SI = O ALORS CALL UN
	GOTSX	BBAND_MOINS
	GOTOX    TONE
BBAND_MOINS	



;****************************************************************************   	
    Bank0	
	movlw	b'11111110'		
	movwf     PortC
	
	movf	PortB,w	
	MOVWF	tester
	
	
	
	CALLX	pause


	PCLAX	one
	btfsc	 tester,7  ;PortB,7	;SI = O ALORS CALL UN		
	GOTSX   one
	GOTOX	un	
one

	PCLAX	too
	btfsc	tester,6  ;PortB,6	;SI = O ALORS CALL UN
	GOTSX   too
	GOTOX	deux
too	

	PCLAX	tree
	BTFSC	tester,5 ;PortB,5	;SI = O ALORS 
	GOTSX   tree
	GOTOX	trois
tree
	
	
	PCLAX	rrit
	BTFSC	tester,4 ;PortB,4	;SI = O ALORS CALL UN
	GOTSX   rrit
	GOTOX	rit
rrit	


;********************************************************************
	 Bank0	
	movlw	b'11111101'
	movwf     PortC
	
	movf	PortB,w	
	MOVWF	tester	
	;CALLX	pause

	PCLAX	for
	BTFSC	tester,7	;SI = O ALORS CALL UN
	GOTSX   for	
	GOTOX	quatre
for	

	PCLAX	five
	BTFSC	tester,6	;SI = O ALORS CALL UN
	GOTSX   five
	GOTOX	cinq
five
	
	PCLAX	sixx
	BTFSC	tester,5	;SI = O ALORS CALL UN
	GOTSX   sixx
	GOTOX	six
sixx
	
	PCLAX	uu_lsb
	BTFSC	tester,4	;SI = O ALORS CALL UN
	GOTSX   uu_lsb
	GOTOX	u_lsb
	
uu_lsb
		
;********************************************************************
 	Bank0	
	movlw	b'11111011'
	movwf     PortC
	
	movf	PortB,w	
	MOVWF	tester	

	;CALLX	pause

	PCLAX	seven
	BTFSC	tester,7	;SI = O ALORS CALL UN
	GOTSX	seven
	GOTOX	sept	
seven
	PCLAX	eight
	BTFSC	tester,6	;SI = O ALORS CALL UN
	GOTSX	eight
	GOTOX	huit
eight	
	PCLAX	nine
	BTFSC	tester,5	;SI = O ALORS CALL UN
	GOTSX	nine
	GOTOX	neuf
nine	
	PCLAX	aatt
	BTFSC	tester,4	;SI = O ALORS CALL UN
	GOTSX	aatt
	GOTOX	att
aatt	

	
;********************************************************************

	 Bank0	
	movlw	b'11011111'
	movwf     PortC
	
	movf	PortB,w	
	MOVWF	tester

	;CALLX	pause

	PCLAX	mmem
	BTFSC	tester,7	;SI = O ALORS CALL UN
	GOTSX	mmem
	GOTOX	mem
mmem
	PCLAX	zzero
	BTFSC	tester,6	;SI = O ALORS CALL UN
	GOTSX	zzero
	GOTOX	zero
zzero
	PCLAX	rr_mem
	BTFSC	tester,5	;SI = O ALORS CALL UN
	GOTSX	rr_mem
	GOTOX    r_mem 	
rr_mem
	PCLAX	ppreampli	
	BTFSC	tester,4	;SI = O ALORS CALL UN
	GOTSX	ppreampli
	GOTOX	preampli
ppreampli

	
;********************************************************************
 	Bank0	
	movlw	b'10111111'
	movwf     PortC
	
	movf	PortB,w	
	MOVWF	tester

	;CALLX	pause

	PCLAX	AA_ou_B	
	BTFSC	tester,7	;SI = O ALORS CALL UN
	GOTSX	AA_ou_B
	GOTOX	A_ou_B
AA_ou_B
	PCLAX	AA_egal_B	
	BTFSC	tester,6	;SI = O ALORS CALL UN
	GOTSX	AA_egal_B
	GOTOX	A_egal_B
AA_egal_B
	PCLAX	BB_egal_A	
	BTFSC	tester,5	;SI = O ALORS CALL UN
	GOTSX	BB_egal_A
	GOTOX    B_egal_A
BB_egal_A		
	PCLAX	EENTRE_NUM
	BTFSC	tester,4	;SI = O ALORS CALL ENTRE_NUM
	GOTSX	EENTRE_NUM
	GOTOX	ENTRE_NUM
EENTRE_NUM
	
	GOTOX	int_retour	

;*********************************************************************
COMMUT_HF	;s/p commutation des relais de la tete hf 
	movwf	band;
	CALLX   getfee
	CALLX	BUS_I2C	;commutation des filtres de la tete hf

	CALLX	COM_PASS_PAS; s/p de commutation des pins du pcf8474a et des relais du pass bas	

	CALLX	PIC_esclave;commuter le relais 1.8/3.5 mgh
	CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band 
	CALLX	write3
	CALLX	val_pwr		;read la valeur dans l'eeprom 
	CALLX	i2c_pwr		;et l'envter dans la ds18030 ( potar electro)
	RETURN
;*********************************************************************

un
	  
	
	PCLAX	 trx_un      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_un 		; dans ce mode
		;Bank1
	PCLAX 	unn
	BTFSS	tester_rit,0	;si = 1 indique que la touche "RIT" e =a t active
	GOTSX	unn				;cette touche ne doit pas etre acvite dans ce mode
	;	Bank0					;alors  GOTO unn sinon on saute a l'instruction suivante														

	PCLAX	mem1
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem1			;sinon on saute a l'instruction suivante
	
	
	PCLAX	RAP_mem1
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem1			;sinon on saute a l'instruction suivante

		
	PCLAX	num1
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num1			; sinon on saute a l'instruction suivante	
;*************************************************************************************		
	
	movlw	0X24		;bande 1m8
	CALLX	COMMUT_HF
	;movwf	band
	;CALLX   getfee
	;CALLX	BUS_I2C	;commutation des filtres de la tete hf
	
	;CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band 
	;CALLX	write3

	
trx_un 
unn	
	GOTOX	int_retour	
mem1
; sur un appui normal de la touche "UN"le programme saute a "cont_uno;NUM"
;sur un appui long "1 seconde" de la touche "UN"le
;programme saute a "MEMOIRE_lsb"

	movlw	0x7f
	MOVWF   count
		
plus_haut

    CALLX    wait_64ms          ;
 	PCLAX	cont_uno
 	BTFSc	PortB,7	;SI = 0 ALORS cont_uno
 	GOTSX	cont_uno
 PCLAX	plus_haut
	DECFSZ	count,f
	GOTSX 	plus_haut
	
;****************MEMOIRE_lsb   ******************* 

	 movlw  0xf0            ; mise en memoire sur la TOUCHE 1
	GOTOX	MISEmemoire		; de la frequence lsb du filtre a quartz	
cont_uno;NUM
	
	movlw	0x40	    ;mises en memoire a l'adresse 0x40 memoire n 1
	GOTOX	MISEmemoire	

RAP_mem1
	movlw	0x40
	GOTOX	RAPPELmemoire
num1 
;uno
;	PCLAX	onne        ;anti rebond
;	btfsc	 PortB,7  ;PortB,7	;SI = 0 ALORS uno		
;	GOTSX   onne
;	GOTOX	uno	
;onne
	movlw 0x01		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;
;********************************************************************************

deux
	PCLAX	 trx_deux      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_deux 		; dans ce mode

	;Bank1
	PCLAX 	deuxx
	BTFSS	tester_rit,0	 ;si = 1 = GOTO;voir a 'un'
	GOTSX	deuxx
	;Bank0
	PCLAX	mem2
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem2			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem2
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem2			;sinon on saute a l'instruction suivante

;*******************************************************************************************	
		
	PCLAX	num2
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num2			; sinon on saute a l'instruction suivante	
;*************************************************************************

	movlw	0X00
		CALLX	COMMUT_HF	
	;movwf	band
		;CALLX   getfee			;bande 3.5
		;CALLX	BUS_I2C	;commutation des filtres de la tete hf

	;CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band  
	;CALLX	write3
trx_deux	      
deuxx	
	GOTOX	int_retour	
mem2
; sur un appui normal de la touche "DEUX"le programme saute a "cont_deuse
;sur un appui long "1 seconde" de la touche "DEUX"le
;programme saute a "MEMOIRE_usb"

	movlw	0x7f
	MOVWF   count
		
plus_hautt

    CALLX    wait_64ms          ;
 	PCLAX	cont_deuse
 	BTFSc	PortB,6	    ;SI = 0 ALORS cont_deuse
 	GOTSX	cont_deuse
 PCLAX	plus_hautt
	DECFSZ	count,f
	GOTSX 	plus_hautt
	
;****************MEMOIRE_usb   ******************* 

	 movlw  0xe8            ; mise en memoire sur la TOUCHE 2
	GOTOX	MISEmemoire		; de la frequence usb du filtre a quartz
cont_deuse
	
	movlw	0x47	    ;mises en memoire a l'adresse 0x44 
	GOTOX	MISEmemoire	
	
RAP_mem2
	movlw	0x47
	GOTOX	RAPPELmemoire
num2 
deuxo
	PCLAX	deuxe       ;anti rebond
	btfss 	PortB,6  ;PortB,6	;SI = 0 ALORS deuxe		
	GOTSX   deuxe
	GOTOX	deuxo	
deuxe	

	movlw 0x02		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;	

	;********************************************************************************

trois 
	PCLAX	 trx_trois     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_trois		; dans ce mode

	;Bank1
	PCLAX 	troiss
	BTFSS	tester_rit,0	 ;si = 1 = GOTO;voir a 'un'
	GOTSX	troiss
	;Bank0
	PCLAX	mem3
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem3			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem3
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem3			;sinon on saute a l'instruction suivante
	
	;*******************************************************************************************	
		
	PCLAX	num3
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num3			; sinon on saute a l'instruction suivante	
;*************************************************************************

	movlw	0X04
	CALLX	COMMUT_HF		
	;movwf	band
	;CALLX   getfee			;bande 7
		;CALLX	BUS_I2C	;commutation des filtres de la tete hf
	
	;CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band  
	;CALLX	write3
trx_trois	      
troiss	
	GOTOX	int_retour	
mem3
	
	movlw	0x4e	    ;mises en memoire a l'adresse 0x44 
	GOTOX	MISEmemoire	
	
RAP_mem3
	movlw	0x4e
	GOTOX	RAPPELmemoire

		
num3 
troiso
	PCLAX	troise     ;anti rebond
	btfss	PortB,5  ;PortB,5	;SI = 0 ALORS troiso	
	GOTSX   troise
	GOTOX	troiso	
troise

	movlw 0x03		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;
	;********************************************************************************

rit
	PCLAX	 trx_rit     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_rit		; dans ce mode


	PCLAX	r_rit
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche RIT ne doit pas etre acvite dans ce mode
	GOTSX 	r_rit			;alors gotox r_rit sinon on saute a l'instruction suivante	
	
	
	PCLAX	r_ritt
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche RIT  ne doit pas etre acvite dans ce mode
	GOTSX 	r_ritt			;alors gotox r_ritt sinon on saute a l'instruction suivante
	
	PCLAX	num_rit
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_rit
	GOTSX 	num_rit			; sinon on saute a l'instruction suivante		
	
	
			
	 ;	Bank1	
	PCLAX 	RET
	BTFSC	tester_rit,0	 ;si = 0 = goto	RET = WRITsimplex
	GOTSX	RET			     ;si = 1 = goto RETrit
	;	Bank0	
	GOTOX	RETrit	
RET    	
	GOTOX	WRITsimplex     ;  faire WRITsimplex 
num_rit	
trx_rit	
r_ritt	
r_rit	
	GOTOX	int_retour	

	;*****************************************************************************
	
quatre
	PCLAX	 trx_quatre      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_quatre 		; dans ce mode

	;Bank1
	PCLAX 	quatree
	BTFSS	tester_rit,0	;si = 1 indique que la touche "RIT" e =a t active
	GOTSX	quatree			;cette touche ne doit pas etre acvite dans ce mode
		;Bank0						;alors  GOTO unn sinon on saute a l'instruction suivante
	PCLAX	mem4
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem4			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem4
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem4			;sinon on saute a l'instruction suivante	
		
	PCLAX	num4
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num4			; sinon on saute a l'instruction suivante	
;*************************************************************************	
	
	
	movlw	0X08		;bande 10
		CALLX	COMMUT_HF
;	movwf	band
		;CALLX   getfee
		;CALLX	BUS_I2C	;commutation des filtres de la tete hf
 
	;CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band
	;CALLX	write3
trx_quatre
quatree
	GOTOX	int_retour	
mem4	
	movlw	0x55	    ;mises en memoire a l'adresse 0x4C memoire n 4
	GOTOX	MISEmemoire	

RAP_mem4
	movlw	0x55
	GOTOX	RAPPELmemoire
num4 
	movlw 0x04		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;	


;********************************************************************************

cinq
	PCLAX	 trx_cinq      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_cinq		; dans ce mode
		;Bank1
	PCLAX 	cinqq
	BTFSS	tester_rit,0	 ;si = 1 = GOTO;voir a 'un'
	GOTSX	cinqq
;	Bank0
	PCLAX	cinq5
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	cinq5			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem5
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem5			;sinon on saute a l'instruction suivante
	
		
	PCLAX	num5
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num5			; sinon on saute a l'instruction suivante
		

	movlw	0X0C
		CALLX	COMMUT_HF	
;	movwf	band
		;CALLX   getfee			;bande 14
	;CALLX	BUS_I2C	;commutation des filtres de la tete hf	

	;CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band  
	;CALLX	write3
trx_cinq	      
cinqq	
	GOTOX	int_retour	

cinq5	
	movlw	0x5c	    ;mises en memoire a l'adresse 0x44 
	GOTOX	MISEmemoire	
	
RAP_mem5
	movlw	0x5c
	GOTOX	RAPPELmemoire
num5 
	movlw 0x05		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;	
;********************************************************************************

six
	PCLAX	 trx_six    
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_six 		; dans ce mode

		;Bank1
	PCLAX 	sixi
	BTFSS	tester_rit,0	 ;si = 1 = GOTO;voir a 'un'
	GOTSX	sixi
	;Bank0
	PCLAX	mem6
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem6			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem6
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem6			;sinon on saute a l'instruction suivante
	
		
	PCLAX	num6
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num6			; sinon on saute a l'instruction suivante	


	movlw	0X10
	CALLX	COMMUT_HF		
	;movwf	band
	;CALLX   getfee			;bande 18
		;CALLX	BUS_I2C	;commutation des filtres de la tete hf
	
;	CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band  
;	CALLX	write3
trx_six	      
sixi	
	GOTOX	int_retour	
mem6
	
	movlw	0x63	    ;mises en memoire a l'adresse 0x44 
	GOTOX	MISEmemoire	
	
RAP_mem6
	movlw	0x63
	GOTOX	RAPPELmemoire
num6 
	movlw 0x06		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;	

	;********************************************************************************

u_lsb 

	PCLAX	 trx_u_lsb     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_u_lsb 		; dans ce mode

	
	PCLAX	u_lsbb
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche i_lcb ne doit pas etre acvite dans ce mode
	GOTSX 	u_lsbb			;alors gotox uu_lsb sinon on saute a l'instruction suivante	
	
	
	PCLAX	u_llsb
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche u_lsb  ne doit pas etre acvite dans ce mode
	GOTSX 	u_llsb			;alors gotox u_llsb sinon on saute a l'instruction suivante	
	
	PCLAX	num_u_lsb
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_u_lsb
	GOTSX 	num_u_lsb			; sinon on saute a l'instruction suivante

	PCLAX 	lsbb
	BTFSC	bli_bls,0	;;si = a 1 indique que la touche lsb/usb a ete active 
	GOTSX	lsbb	;alors gotox lsbb
		
	PCLAX 	band_cinq_ente_usb
	BTFSS	band_cinquante,0	;;si = a 1 indique que la touche 50 a ete active 
	GOTSX	band_cinq_ente_usb		;alors gotox cinq_ente
							
pour_le_50_usb
	movlw 0X01
	MOVWF	bli_bls	
	
	;MOVLW	0X80
	;CALLX	SOUS_U17		;METTRE LA PINS 11 de u17 A 0 (P6)
	
	CALLX	USSB
	
	CALLX	write3
	
	;CALLX	DAN
	;CALLX    bin2BCD           ; Convert the frequency to BCD	
	;CALLX    show_freq         ; Display the frequency on the LCD
	;CALLX    calc_dds_word     ; Find the control word for the DDS chip
	;CALLX    send_dds_word     ; Send the control word to the DDS chip
				
	GOTOX	int_retour
	
band_cinq_ente_usb

	movlw	0x01			;si =0 alors supradyne si = 1 alors infradyne
	movwf	add_sous_affi	;variable pour OSC local
	GOTOX 	pour_le_50_usb
			
lsbb

	
	PCLAX 	band_cinq_ente_lsb
	BTFSS	band_cinquante,0	;;si = a 1 indique que la touche 50 a ete active 
	GOTSX	band_cinq_ente_lsb		;alors gotox cinq_ente
	
	
pour_le_50_lsb
	;MOVLW	0X80
	;CALLX	ADD_U17		;METTRE LA PINS 11 de u17 A 0 (P6)
	

	movlw 0X00
	MOVWF	bli_bls
	CALLX	lsb1  

	CALLX	write3		
	;CALLX	DAN
	;CALLX    bin2BCD           ; Convert the frequency to BCD	
	;CALLX    show_freq         ; Display the frequency on the LCD
	;CALLX    calc_dds_word     ; Find the control word for the DDS chip
	;CALLX    send_dds_word     ; Send the control word to the DDS chip
	
	GOTOX	int_retour	
		
band_cinq_ente_lsb

	movlw	0x01			;si =0 alors supradyne si = 1 alors infradyne
	movwf	add_sous_affi	;variable pour OSC local
	GOTOX 	pour_le_50_lsb
num_u_lsb	
trx_u_lsb	
u_lsbb
u_llsb
	GOTOX	int_retour		
	;*******************************************************************************
sept

	PCLAX	 trx_sept      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_sept 		; dans ce mode
		;Bank1
	PCLAX 	septt
	BTFSS	tester_rit,0	;si = 1 indique que la touche "RIT" e =a t active
	GOTSX	septt				;cette touche ne doit pas etre acvite dans ce mode
	;Bank0					;alors  GOTO unn sinon on saute a l'instruction suivante
	PCLAX	mem7
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem7			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem7
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem7			;sinon on saute a l'instruction suivante	
				
	PCLAX	num7
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num7			; sinon on saute a l'instruction suivante	

		
	movlw	0X14		;bande 21
		CALLX	COMMUT_HF
	;movwf	band
		;CALLX   getfee
		;CALLX	BUS_I2C	;commutation des filtres de la tete hf
 
;	CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band
	;CALLX	write3
trx_sept
septt	
	GOTOX	int_retour	
mem7	
	movlw	0x6a	    ;mises en memoire a l'adresse 0x40 memoire n 1
	GOTOX	MISEmemoire	

RAP_mem7
	movlw	0x6a
	GOTOX	RAPPELmemoire
num7 
	movlw 0x07		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;	
;********************************************************************************
	
huit
	PCLAX	 trx_huit     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_huit 		; dans ce mode

	;	Bank1
	PCLAX 	huitt
	BTFSS	tester_rit,0	;si = 1 indique que la touche "RIT" e =a t active
	GOTSX	huitt			;cette touche ne doit pas etre acvite dans ce mode
	;	Bank0						;alors  GOTO unn sinon on saute a l'instruction suivante
	PCLAX	mem8
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem8			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem8
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem8			;sinon on saute a l'instruction suivante	
		
		
	PCLAX	num8
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num8			; sinon on saute a l'instruction suivante	

	
	movlw	0X18		;bande 24
		CALLX	COMMUT_HF
;	movwf	band
		;CALLX   getfee
		;CALLX	BUS_I2C	;commutation des filtres de la tete hf
 
	;CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band	
	;CALLX	write3
trx_huit
huitt
	GOTOX	int_retour	
mem8	
	movlw	0x71	    ;mises en memoire a l'adresse 0x40 memoire n 1
	GOTOX	MISEmemoire	

RAP_mem8
	movlw	0x71
	GOTOX	RAPPELmemoire
num8 
	movlw 0x08		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;	
;********************************************************************************	
	
neuf
	PCLAX	 trx_neuf     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_neuf 		; dans ce mode

	;Bank1
	PCLAX 	neuff
	BTFSS	tester_rit,0	;si = 1 indique que la touche "RIT" e =a t active
	GOTSX	neuff			;cette touche ne doit pas etre acvite dans ce mode
	;	Bank0						;alors  GOTO unn sinon on saute a l'instruction suivante
	PCLAX	mem9
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem9			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem9
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem9			;sinon on saute a l'instruction suivante			
		
	PCLAX	num9
	BTFSS	tester_entre,0		;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;donc goto num1
	GOTSX 	num9			; sinon on saute a l'instruction suivante	

	
	movlw	0X1C		;bande 28
		CALLX	COMMUT_HF
		;movwf	band
	;CALLX   getfee
		;CALLX	BUS_I2C	;commutation des filtres de la tete hf

	;CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band 
	;CALLX	write3
trx_neuf
neuff	
	GOTOX	int_retour	
mem9	
	movlw	0x78	    ;mises en memoire a l'adresse 0x40 memoire n 1
	GOTOX	MISEmemoire	

RAP_mem9
	movlw	0x78
	GOTOX	RAPPELmemoire
num9
	movlw 0x09		;correspond au numero de la touche enfoncee
	GOTOX T_num	
	;	
;*****************************************************************************

	
att	
	PCLAX	 trx_att     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_att		; dans ce mode


	PCLAX	att1
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche ATTENUATEUR ne doit pas etre acvite dans ce mode
	GOTSX 	att1			;alors gotox entre1 sinon on saute a l'instruction suivante	
	
	
	PCLAX	att2
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche ATTENUATEUR ne doit pas etre acvite dans ce mode
	GOTSX 	att2		    ;alors gotox entree2 sinon on saute a l'instruction suivante
	
	PCLAX	num_att
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_att
	GOTSX 	num_att			; sinon on saute a l'instruction suivante


	movlw 	0x01 				;inverser le 1er bit 
	xorwf	attenuateur,f 	;si = 0 attenuateur hors service
	
	
	movlw	0X00		 
	SUBWF	attenuateur,w	
	PCLAX	aller_02
	btfss	STATUS,B_Z	;si  0X00 - attenuateur = 1 alors
	GOTSX	aller_02	
	movlw	0X02;1	;mettre le bit 0 du pcf8574 a 1 (p)
	CALLX	SOUS_U17					;saut a pour_01
	
	CALLX	ATTENU_m
	;movlw   0X95;D7            ;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	;CALLX    cmnd2LCD 	
	
;	movlw   '-'               ; 
  ;	movwf   LCD_char      
;	CALLX    data2LCD
	
	GOTOX	int_retour
	
aller_02
	movlw	0x02;1
	CALLX	ADD_U17
		;movlw   0X95;D7            ;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
;	CALLX    cmnd2LCD 	
	CALLX	ATTENU_p
	;movlw   '+'               ; 
  	;movwf   LCD_char      
	;CALLX    data2LCD
	
	;movlw   'A'               ;ECRIRE F
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
;	movlw   'T'               ;ECRIRE F
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
	;movlw   'T'               ;ECRIRE F
	;movwf   LCD_char          ; 
	;CALLX    data2LCD 
num_att	
trx_att	
att1			
att2
	GOTOX	int_retour	

;***********************************************************************************
mem
	PCLAX	 trx_mem     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_mem 		; dans ce mode
	;Bank1
	PCLAX 	memm
	BTFSS	tester_rit,0	;si = 1 indique que la touche "RIT" e =a t active
	GOTSX	memm				;cette touche ne doit pas etre acvite dans ce mode
		;Bank0						;alors  GOTO memm sinon on saute a l'instruction suivante							
 ;
	
	PCLAX	num_mem
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_mem
	GOTSX 	num_mem			; sinon on saute a l'instruction suivante
	
	CALLX	QUELmem      
	
	MOVLW	0X01
	MOVWF	memoire	;sert d'index pour que la touche numero de memoire
num_mem
trx_mem	
memm	
	GOTOX	int_retour		
	

;*********************************************************************	
zero
	PCLAX	 trx_zero    
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_zero		; dans ce mode

		;Bank1	
	PCLAX 	zeroo	
	BTFSS	tester_rit,0	;si = 0 indique que la touche "RIT" e =a t active
	GOTSX	zeroo			;cette touche ne doit pas etre acvite dans ce mode
		;Bank0						;alors  GOTO unn sinon on saute a l'instruction suivante
	PCLAX	mem0
	BTFSC	memoire,0		;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
							;alors gotox mem1
	GOTSX 	mem0			;sinon on saute a l'instruction suivante
	
	PCLAX	RAP_mem0
	BTFSC	R_memoire,0		;si = a 1 indique que la touche RAP_mem a ete active 
							;alors gotox RAP_mem
	GOTSX 	RAP_mem0			;sinon on saute a l'instruction suivante
	
	PCLAX	entre_0			;si = 1 indique que la touche "ENTRE_NUM" a ete activ
	BTFSS	tester_entre,0	;alors goto entre_0
	GOTSX	entre_0			;sinon on saute et fait l'instruction suivante
	
	movlw	0X20		;bande 50
	movwf	band
	
	CALLX   getfee
	CALLX	val_pwr		;read la valeur dans l'eeprom 
	CALLX	i2c_pwr		;et l'envter dans la ds18030 ( potar electro)
		
		CALLX	BUS_I2C	     ;commutation des filtres de la tete hf
		CALLX	COM_PASS_PAS; s/p de commutation des pins du pcf8474a et des relais du pass bas	

		CALLX	PIC_esclave  ;commuter le relais 1.8/3.5 mgh SUR LE 50MGH	
    							
	movlw	0X01         	;parce l'osc 50 est en infradyne alors que pour toutes
	MOVWF	band_cinquante  ;les autres bandes sont en supradyne donc il me faut un repere 
	                        ;pour le s/p dan add la valeur de l'oc + la valeur de la fi

	CALLX 	lsb_usb		;afficher sur le lcd (USB ou LSB) en fonction de la band 
	CALLX	write3
trx_zero
zeroo
	GOTOX	int_retour	
mem0	
	movlw	0x7f	    ;mises en memoire a l'adresse 0x76 memoire n 0
	GOTOX	MISEmemoire	

RAP_mem0
	movlw	0x7f
	GOTOX	RAPPELmemoire
	
entre_0	
	movlw	0X00
	GOTOX	T_num
	GOTOX	int_retour	
	
;******************************************************************************	

r_mem 

	PCLAX	 trx_r_mem     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_r_mem		; dans ce mode

	;Bank1
	PCLAX 	r_memm
	BTFSS	tester_rit,0	;si = 1 indique que la touche "RIT" a t active
	GOTSX	r_memm			;la touche  r_mem ne doit pas etre acvite dans ce cas la
		;Bank0						;alors  GOTO r_mem sinon on saute a l'instruction suivante
											
	PCLAX	r_mem1
	BTFSC	memoire,0	;si = a 1 indique que la touche "mise en MEMOIRE" a ete active 
	GOTSX 	r_mem1		;la touche r_mem ne doit pas etre acvite dans ce cas la
			        	;alors gotox mem1 sinon on saute a l'instruction suivante
			        	
	PCLAX	num_r_mem
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_r_mem
	GOTSX 	num_r_mem		; sinon on saute a l'instruction suivante
	
	movlw	0x01						
	movwf	R_memoire						
				
	CALLX	QUELmem
num_r_mem
trx_r_mem		
r_mem1	
r_memm	         	
	GOTOX	int_retour	
		;********************************************************************************

preampli
	PCLAX	 trx_pre      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_pre		; dans ce mode


	PCLAX	preampli1
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche PREAMPLI ne doit pas etre acvite dans ce mode
	GOTSX 	preampli1			;alors gotox entre1 sinon on saute a l'instruction suivante	
	
	
	PCLAX	preampli2
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche PREAMPLI ne doit pas etre acvite dans ce mode
	GOTSX 	preampli2		    ;alors gotox entree2 sinon on saute a l'instruction suivante
	
	PCLAX	num_pre
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_pre
	GOTSX 	num_pre			; sinon on saute a l'instruction suivante


	movlw 	0x01			; inverserle  1er bit = preampli_rec
	xorwf	preampli_rec,f 	 ;si = 0 preampli_rec hors service
	
	;movlw	0x70
	;movwf	adresse
	
	movlw	0X00
	SUBWF	preampli_rec,w	;si  0X00 - preampli = 1 alors
									;saut a aller_02
	
	PCLAX	aller_01			
	btfss	STATUS,B_Z	
	GOTSX	aller_01
	movlw	0X01;2   
	CALLX	SOUS_U17	;mettre le bit 1 de u17 a zero ()	
	
	CALLX	PRE_m
;	movlw   0X90;D7             ;POSITION OU COMMENCER A ECRIRE sur le lcd  
;	CALLX    cmnd2LCD 
		
;	movlw   '-'               ; 
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	movlw   'P'               ;ECRIRE F
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	movlw   'r'               ;ECRIRE F	movwf   LCD_char          ; 
;	movwf   LCD_char  
;	CALLX    data2LCD
;	movlw   'e'               ;ECRIRE F
;	movwf   LCD_char          ; 
;	CALLX    data2LCD 
;	movlw   'a'               ;ECRIRE F
;	movwf   LCD_char          ; 
;	CALLX    data2LCD 
;	movlw   'm'               ;ECRIRE F
;	movwf   LCD_char               ; 
;;	CALLX    data2LCD

;	movlw   'p'               ;ECRIRE F
;	movwf   LCD_char               ; 
;	CALLX    data2LCD
	
	
	GOTOX	int_retour
	
aller_01

	movlw	0X01;2		;mettre le bit 1 du pcf8574 a 1 
	CALLX	ADD_U17
	CALLX	PRE_p
	
num_pre
trx_pre	
preampli1	
preampli2
		
		GOTOX	int_retour	
	;************************************************************************
A_ou_B

	PCLAX	 trx_a_ou_b     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_a_ou_b 		; dans ce mode

	PCLAX	aoub
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche A_OU_B ne doit pas etre acvite dans ce mode
	GOTSX 	aoub			;alors gotox aoub sinon on saute a l'instruction suivante	
	
	
	PCLAX	aaoubb
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche A_OU_B  ne doit pas etre acvite dans ce mode
	GOTSX 	aaoubb			;alors gotox aaoubb sinon on saute a l'instruction suivante	
	
	PCLAX	num_a_ou_b
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_a_ou_b
	GOTSX 	num_a_ou_b			; sinon on saute a l'instruction suivante


	PCLAX 	tester_a
	BTFSS	tester_a_ou_b,0	;si = 0 = goto r_tester_a
	GOTSX	tester_a		;si = 1 on continue

							
	movlw   0xc8             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'B'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	movlw	0xd0;7C	    		;mise en mem du vfo "A"
		CALLX   put_memoire 		;freq 0,1,2,3 = memoire numero 1

	
	movlw	0xd8;82	         ;rappel memoire du vfo "B"
	CALLX	get_memoire	
	CALLX	write2
	
	movlw	0X00
	MOVWF	tester_a_ou_b
		
 	GOTOX	int_retour	
 	
tester_a
	movlw   0xc8             ; endroit ou ecrire sur le lcd
	CALLX    cmnd2LCD
	
	MOVLW	' '
	movwf   LCD_char          ; 
	CALLX    data2LCD
	MOVLW	'A'
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	movlw	0xd8;82	         ;mise en mem du vfo "B"
	CALLX   put_memoire 		;freq 0,1,2,3 = memoire numero 1
	
	movlw	0xd0;7C
	CALLX	get_memoire   ;rappel memoire du vfo "A";
	CALLX	write2
	
	movlw	0X01
	MOVWF	tester_a_ou_b
num_a_ou_b	
trx_a_ou_b	
aaoubb		
aoub
	GOTOX	int_retour	
			;********************************************************************************

A_egal_B
	PCLAX	 trx_a_egal_b      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_a_egal_b 		; dans ce mode

	
	PCLAX	aegalb
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche A_egal_B ne doit pas etre acvite dans ce mode
	GOTSX 	aegalb			;alors gotox aegalb sinon on saute a l'instruction suivante	
	
	
	PCLAX	aaegalbb
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche A_egal_B  ne doit pas etre acvite dans ce mode
	GOTSX 	aaegalbb			;alors gotox aaegalbb sinon on saute a l'instruction suivante
	
	PCLAX	num_a_egal_b
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_a_egal_b
	GOTSX 	num_a_egal_b		; sinon on saute a l'instruction suivante	

	movlw	0xd0;7C
	CALLX	put_memoire   ;mise en memoire du vfo "A";
	
	movlw	0xd8;82	         ;mise en mem du vfo "B"
	CALLX   put_memoire 		;freq 0,1,2,3 = memoire numero 1
	
	CALLX	write2
	
	movlw   0Xd2        ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 

	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'a'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
num_a_egal_b	
trx_a_egal_b	         ;
aegalb	
aaegalbb		
	GOTOX	int_retour	
			;********************************************************************************

B_egal_A
	PCLAX	 trx_b_egal_a     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_b_egal_a		; dans ce mode


	PCLAX	begala
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche B_egal_A ne doit pas etre acvite dans ce mode
	GOTSX 	begala			;alors gotox begala sinon on saute a l'instruction suivante	
	
	
	PCLAX	bbegalaa
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche B_egal_A  ne doit pas etre acvite dans ce mode
	GOTSX 	bbegalaa		;alors gotox bbegalaa sinon on saute a l'instruction suivante
	
	PCLAX	num_b_egal_a
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_b_egal_a
	GOTSX 	num_b_egal_a			; sinon on saute a l'instruction suivante


	movlw	0xd0;82	         ;mise en  memoire du vfo "B"
	CALLX	put_memoire	
	
	movlw	0xd8;7C	    		;mise en mem du vfo "A"
	CALLX   put_memoire 		;freq 0,1,2,3 = memoire numero	
	
	CALLX	write2
	
	movlw   0Xd4        ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 

	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'b'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
num_b_egal_a	
trx_b_egal_a	         ;
begala		
bbegalaa	
		
	GOTOX	int_retour	
	
			;********************************************************************************
		;cette touche sert pour indiquer qu'on va se servir des touches numerique du clavier
	;pour cela l'index tester_entre est mis a 0		;*******************************************************************************

ENTRE_NUM
		
	PCLAX	 trx_num      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_num 		; dans ce mode
		
	PCLAX 	entree	    	;si = a 0 indique que la touche "RIT" a ete active 
		;Bank1						;cette touche RIT ne doit pas etre acvite dans ce mode
	BTFSS	tester_rit,0	;alors gotox entre sinon on saute a l'instruction suivante
	GOTSX	entree
		;Bank0
	PCLAX	entree1
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche MEMOIRE ne doit pas etre acvite dans ce mode
	GOTSX 	entree1			;alors gotox entre1 sinon on saute a l'instruction suivante	
	
	
	PCLAX	entree2
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche RAPPEL MEMOIRE ne doit pas etre acvite dans ce mode
	GOTSX 	entree2		    ;alors gotox entree2 sinon on saute a l'instruction suivante
	
	CALLX	LCDcls1					
			
								
					;*********************************************
										
	movlw   0X80  ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 


	movlw   	'E'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
		movlw   'N'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'T'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
		movlw   'R'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'E'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'R'                   ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	 	movlw   ' '               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	   
		movlw   'F'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	   	movlw   'R'            ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	 	movlw   'E'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	 	movlw   'Q'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'U'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'E'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'N'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
    	movlw   'C'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD  
    	movlw   'E'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD   
	
	
	  
	      
	movlw	0x00			;indique que la touche "NUM"rique a ete enfonce
	movwf	tester_entre
	
	
	
; sur un appui normal de la touche "ENTRE_NUM"le programme saute a "cont_NUM"
;sur un appui long "1 seconde" de la touche "ENTRE_NUM"le
;programme saute a "GOTOX entre_fi"
	movlw	0x7f
	MOVWF   count
		
plus_haute

    CALLX    wait_64ms          ;
 	PCLAX	cont_NUM
 	BTFSc	PortB,4	;SI = 0 ALORS cont_NUM
 	GOTSX	cont_NUM
 	PCLAX	plus_haute
	DECFSZ	count,f
	GOTSX 	plus_haute
	
	GOTOX	entr_fi    ; s/p entr par les touches du clavier
	                	; de la frequence du filtre a quartz 
					
cont_NUM
		
	movlw	0x00
	movwf	freq_00
	movlw	0x00
	movwf	freq_01
	movlw	0x00
	movwf	freq_02
	movlw	0x00
	movwf	freq_03

			
	movlw	0x00
	movwf	freq_0
	movlw	0x00
	movwf	freq_1
	movlw	0x00
	movwf	freq_2
	movlw	0x00
	movwf	freq_3

trx_num	
entree
entree1	
entree2
	
	GOTOX	int_retour		

				
				;********************************************************************************

PAS_MOINS
	PCLAX	 trx_moins     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_moins 		; dans ce mode
	
	PCLAX	num_moins
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_moins
	GOTSX 	num_moins			; sinon on saute a l'instruction suivante

	
	incf	pas,f 
	CALLX	affi_pas  ;s/p affichage de la valuer du pas sur le lcd	
num_moins	
trx_moins         ;	
	GOTOX	int_retour	
	
;********************************************************************************


PAS_PLUS

	PCLAX	 trx_plus      
	BTFSS   PortB,2	  		; si le trx est en emission cette touche ne doit pas etre active  
	GOTSX    trx_plus		; dans ce mode
	
	PCLAX	num_plus
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_plus
	GOTSX 	num_plus			; sinon on saute a l'instruction suivante

	decf	pas,f
	CALLX	affi_pas  ;s/p affichage de la valuer du pas sur le lcd
num_plus		
trx_plus	
	GOTOX	int_retour	
;********************************************************************************


TONE
	PCLAX	tone1
	BTFSC	memoire,0		;si = a 1 indique que la touche "MEMOIRE" a ete active 
							;cette touche TONE ne doit pas etre acvite dans ce mode
	GOTSX 	tone1			;alors gotox entre1 sinon on saute a l'instruction suivante	
	
	
	PCLAX	tone2
	BTFSC	R_memoire,0		;si = a 1 indique que la touche "RAPPEL MEMOIRE" a ete active 
							;cette touche TONE ne doit pas etre acvite dans ce mode
	GOTSX 	tone2		    ;alors gotox entree2 sinon on saute a l'instruction suivante
	
	PCLAX	num_tone
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_att
	GOTSX 	num_tone			; sinon on saute a l'instruction suivante


	movlw 	0x01			; inverserle  1er bit = preampli_rec
	xorwf	tone,f 	 ;si = 0 preampli_rec hors service
	
	;movlw	0x70
	;movwf	adresse
	
	movlw	0X00
	SUBWF	tone,w		;inverser la valeur de tone
								
	
	PCLAX	aller_10			
	btfss	STATUS,B_Z	;si  0X00 - tone = 1 alors
	GOTSX	aller_10	;saut a aller_10
	movlw	0X10  
	CALLX	SOUS_U17	;mettre la pins 5 de u17 a zero	(p4)
	
	
	movlw   0X98	;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	movlw   't'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD	
	GOTOX	int_retour
	
aller_10

	movlw	0X10		;mettre la pins 5 du pcf8574 a 1 (p4)
	CALLX	ADD_U17
		movlw   0X98	;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	
		;movlw   '+'               ; 
  	;movwf   LCD_char      
	;CALLX    data2LCD
	
		movlw   'T'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
	;	movlw   'O'               ; 
  ;	movwf   LCD_char      
	;CALLX    data2LCD
	;	movlw   'N'               ; 
  	;movwf   LCD_char      
	;CALLX    data2LCD
	
	
		
num_tone
tone1
tone2	
	GOTOX	int_retour	
	
;********************************************************************************

CAG
	;PCLAX	 trx_band_moins     
	;BTFSS   PortB,2	  		; si le trx est en emission cette touche ne dois pas etre active  
	;GOTSX    trx_band_moins 
	
    PCLAX	 trx_pwr     
	BTFSS   PortB,2	  		; si le trx est en emission cette touche commande la fonction pwr du pa
	GOTSX    trx_pwr	
			
	
	PCLAX	num_band_moins
	BTFSS	tester_entre,0	;si = a 1 indique que la touche ""NUM"erique" a ete active 
							;cette touche ne doit pas etre active dans ce mode
							;donc goto num_att
	GOTSX 	num_band_moins	; sinon on saute a l'instruction suivante

	;PCLAX 	bandmoins
	;BTFSS	tester_rit,0	 ;si = 1 = GOTO
	;GOTSX	bandmoins
	

	movlw 	0x01			; inverserle  1er bit = preampli_rec
	xorwf	cag,f 	        ;si = 0 preampli_rec hors service
	

	movlw	0X00
	SUBWF	cag,w	    	;inverser la valeur de tone
								
	
	PCLAX	aller_20			
	btfss	STATUS,B_Z	;si  0X00 - tone = 1 alors
	GOTSX	aller_20	;saut a aller_10
	movlw	0X20  
	CALLX	SOUS_U17	;mettre la pins 5 de u17 a zero	(p4)	
	
	movlw   0X9c        ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 

	movlw   'c'               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	GOTOX	int_retour
	
aller_20	
	movlw	0X20		;mettre la pins 5 du pcf8574 a 1 (p4)
	CALLX	ADD_U17
	
	movlw   0X9c	;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 		
		movlw   'C'               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD

num_band_moins	
trx_band_moins
bandmoins	       ;	
	GOTOX	int_retour	

;****************************************************************************************
; objet commander par le bouton incre/dcre la puissance en emission
;****************************************************************************************
trx_pwr
    ;PCLAX 	trx_non              ;si = a 1 indique que la touche ""RIT" a ete active 
	;BTFSS	tester_rit,0	 	;cette touche ne doit pas etre active dans ce mode
	;GOTSX	trx_non	            ;donc goto trx_non
	
					
						
	CALLX	LCDcls1		;	effacer la premire ligne

	
	movlw   0X83  ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 

	movlw   	'R'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	
		movlw   'E'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
		movlw   'G'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	 
		movlw   'L'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
		movlw   'A'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
		movlw   'G'                   ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
		movlw   'E'                   ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD	
		 	movlw   ' '               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	   
		movlw   'P'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	
	  	movlw   'W'            ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	 	movlw   'R'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	clrf	pwr_pot ; le potar electroniqe = 0
	CALLX i2c_pwr
;trx_0
	;movlw	0x50
	;movwf	adresse
	;movlw   0xaf         ;envoie de la donnee selection du potar
	;MOVWF	temp1
;	CALLX	envoie_i2c
;	movf 	pwr_pot,w
;	movwf	temp1
;	CALLX	autre_bit	 ;envoie de la donnee variation potar
;	CALLX	i2c_continue
	;return	
	
trx_01	
	

	PCLAX	 trx_out    
	BTFSC   PortB,2	  		; le trx n'est plus emission alors saut a trx_out
	GOTSX    trx_out

	movf    PortB,w       ;Portb,w           ; Get the current encoder value
	movwf   ren_read          ; Save it	
	movlw   0x03              ; Get encoder mask
	andwf   ren_read,w        ; Isolate encoder bits
	movwf   ren_new           ; Save new value
	xorwf   ren_old,w         ; Has it changed?		
	PCLAX	trx_01	;read_encoder	
	btfsc   STATUS,B_Z 
	GOTSX   trx_01; read_encoder      ; No, keep looking until it changes
		
	
;	       Determine which direction the encoder turned.
;
	bcf     STATUS,B_C        ; Clear the carry bit
	rlf     ren_old,f         ; 
	movf    ren_new,w         ; 
	xorwf   ren_old,f         ; 
	movf    ren_old,w         ; 
	andlw   0x02              ; 
	movwf   next_dir          ; 
	xorwf   last_dir,w        ; 
;
;       Prevent encoder slip from giving a false change in direction.
	PCLAX	peux_continue_trx 
	btfsc   STATUS,B_Z        ; Zero?
	GOTSX    peux_continue_trx       ; No slip; keep going
	movf    next_dir,w        ; Yes, update direction
	movwf   last_dir          ; 
	movf    ren_new,w         ; Save the current encoder bits for next time
	movwf   ren_old           ; 
	GOTSX   trx_01         ; read_encoder      ; Try again
peux_continue_trx
	PCLAX	up222
	btfsc   ren_old,1         ; Are we going down?
	GOTSX    up222               ; No, indicate we are going up
	clrf    last_dir          ; Yes, clear last_dir
	GOTSX    exit333             ; Finish and return
up222 
	movlw   0x02              ; Set UP value in last_dir
	movwf   last_dir          ; 
exit333 
	movf    ren_new,w         ; Get the current encoder bits
	movwf   ren_old           ; Save them in ren_old for the next time	
		     			
	PCLAX	uppp
	btfsc   last_dir,1        ; Is the knob going up?	
	GOTSX    uppp           ; Yes, then add the increment
		
;DOWN	             		
	decf 	pwr_pot,f
	   
	movlw   0X80  ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 

	movf  	pwr_pot,w              ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	   CALLX i2c_pwr	;envoie des infos de pwr dans le ds18030
	   
	  	GOTOX trx_01  ;pour une nouvelle passe
uppp		
		incf	pwr_pot,f
		 
		 
	movlw   0X80          ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 

	movf	pwr_pot,w               
	movwf   LCD_char      ;ECRIRE
	CALLX    data2LCD 
	
	CALLX i2c_pwr	;envoie des infos de pwr dans le ds18030
	
		GOTOX trx_01  ;pour une nouvelle passe
			
trx_out				
	;le trx est repass en reception
		bCf		PortB,3
		CALLX	write3
;trx_non	
	CALLX 	write_pwr
	
	GOTOX	int_retour
	
;*****************************************************************************
;*******************  envoie des infos de pwr dans le ds18030*****************
i2c_pwr
	movlw	0x50
	movwf	adresse
	movlw   0xaf         ;envoie de la donnee selection du potar
	MOVWF	temp1
	CALLX	envoie_i2c
	movf 	pwr_pot,w
	movwf	temp1
	CALLX	autre_bit	 ;envoie de la donnee variation potar
	CALLX	i2c_continue
	return		
	
	
;******************************************************************************	
write_pwr	
;***************Mettre en memoire la valeur du reglage l'adresse = band  + 80****** 
	
	movlw   0x80     ;ecrire dans l'eeprom; l'adresse = "la valeur de band + 80 "
	addwf   band,w

	Bank2 				    
	movwf  EEadr           	 ; EEPROM  address
	Bank0	
	movf    pwr_pot,w        ; load la valeur du potar electronique	
	Bank2
	movwf   EEdata         	   
	CALLX   write_EEPROM      ; write EEPROM
  	Bank0 				      ; STATUS,B_RP0      
	return
;**********************************************************************		
val_pwr	
;********* read la valeur de pwr l'adresse = band + 80 et 
;*********via le i2c l'envoyer au ds18030
	
	movlw   0x80     ;ecrire dans l'eeprom; l'adresse = "la valeur de band + 80 "
	addwf   band,w

	Bank2 				    
	movwf  EEadr
	CALLX	read_EEPROM           	 ; EEPROM  address
	Bank0	
	movwf    pwr_pot      ; load la valeur du potar electronique	
	
	return


;**************************************************************************************

write2

	
	PCLAX 	bliI
	BTFSS	bli_bls,0	;si = 1 indique la variable "bli_bls" 				
	GOTSX	bliI
	
	CALLX	usb1
	GOTOX	write3
bliI	
	CALLX	lsb1
						
write3	
	PCLAX	write4
	BTFSS	entre_fi_qtz,0	;si = a 0 indique que la touche ""NUM"erique" a ete active 
							;et qu'on est dans le mode "entr la frequence du filtre a quartz"
							;et donc allez a write4
						
	GOTSX 	write4	; sinon on saute a l'instruction suivante

	CALLX    bin2BCD           ; Convert the frequency to BCD
	CALLX    show_freq         ; Display the frequency on the LCD					
	CALLX	newdan
	CALLX    calc_dds_word     ; Find the control word for the DDS chip
	CALLX    send_dds_word     ; Send the control word to the DDS chip
	return
	
write4	


	
	CALLX    bin2BCD           ; Convert the frequency to BCD
	CALLX    show_freq         ; Display the frequency on the LCD
	
		movlw	0x01	;le mode entr la freq du filtre a quartz est fini
	movwf	entre_fi_qtz
	return		
	
;******************;mettre en memoire le freq du filtre a quartz*****
	    Bank2    
	    movlw	0xf0
	movwf  EEadr              ; Reset the EEPROM read address
	movf    freq_3,w  ;AFFI_3,w          ; load AFFI_3 frequency
		Bank2	
	movwf   EEdata            ; save the first osc byte
	CALLX   write_EEPROM      ; write EEPROM
	
	movf    freq_2,w  ;AFFI_2,w          ; load osc frequency
		Bank2
	movwf   EEdata            ; save the first osc byte
	CALLX    write_EEPROM      ; write next byte
	
	movf    freq_1,w  ;AFFI_1,w          ; load osc frequency
		Bank2
	movwf   EEdata            ; save the first osc byte
	CALLX    write_EEPROM      ; write the third byte
	
	movf    freq_0,w  ;AFFI_0,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; writ
		Bank0
	
	movlw	0x01	;le mode entr la freq du filtre a quartz est fini
	movwf	entre_fi_qtz
	return							
;**************************************************************************************
pause		
  CALLX    wait_64ms          ;TEMP
 	return
	
;**********************************************************************************
QUELmem
	movlw   0X80              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'Q'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'U'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'E'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	movlw   'L'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
		movlw   'L'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'E'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
		movlw   ' '               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
  		movlw   'M'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'E'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'M'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD  
		movlw   'O'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 	 	
	 	movlw   'I'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'R'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'E'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
		movlw   '?'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	
	  
	RETURN
;**********************************************************************************
MISEmemoire	
	CALLX   put_memoire 		;freq 0,1,2,3 = memoire numero 1
	CALLX	LCDcls1		;effacer la ligne 1 du lcd   
	CALLX	write2		;et reafficher la freq precedamment effacee 
	CALLX    MEGAH		;par le message"QUELLE MEMOIRE?"
	MOVLW	0X00		;sert d'indicateur, la touche "MEMOIRE" est "desactive"
	MOVWF	memoire 
	
	GOTOX	int_retour
;**********************************************************************************
RAPPELmemoire
	
	CALLX	get_memoire
	
	CALLX	val_pwr		;read la valeur dans l'eeprom 
	CALLX	i2c_pwr		;et l'envter dans la ds18030 ( potar electro)
	
	CALLX	LCDcls1		;effacer la ligne 1 du lcd  
	 CALLX	write2
	 CALLX    MEGAH	
	 CALLX BUS_I2C		;commuter la tete hf
	 CALLX	PIC_esclave ;commuter le relais 1.8/3.5 mgh	
	movlw	0x00						
	movwf	R_memoire	;sert d'indicateur, la touche "Rmemoire" est "desactive"	       

	
		GOTOX	int_retour
;*************************************************************************************		

;
; put_memoire - a routine to write the 4 dds frequency data bytes from freq_0,1,2,3
;      le mode bli ou bls et l'osc local infra ou supraadyne etla band to eeprom 
; la variable bls_bli si = 1 ont est en lsb ; si = 0 ont est en usb
;        "    add_sous_affi si = 1 suppradyne ; si  = 0 infradyne
	
put_memoire 
	Bank2 
	movwf  EEadr            ; Reset the EEPROM read address
	movf    freq_3,w          ; load osc frequency
		Bank2	
	movwf   EEdata           ; save the first osc byte
	CALLX   write_EEPROM      ; write EEPROM

	movf    freq_2,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write next byte

	movf    freq_1,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write the third byte

	movf    freq_0,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write the fourth byte
	
		movf    bli_bls,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write the fourth byte
	
	movf    add_sous_affi,w          ; load osc frequency
		Bank2
	movwf   EEdata           ; save the first osc byte
	CALLX    write_EEPROM      ; write the fourth byte
		
	movf    band,w          ; load 
		Bank2
	movwf   EEdata           ; save the  byte
	CALLX    write_EEPROM      ; write the  byte	
	
	
		Bank0
		return	
;*******************************************************************************
; getfee - a routine to get the 4 frequency data bytes from eeprom
;               and place them into freq_0,1,2,3  ; bli_bls , add_sous_affi
get_memoire
;       CALLX with the first address of the data in w
   	 Bank2   
	movwf  EEadr            ; set the EEPROM read address
	CALLX    read_EEPROM       ; Read EEPROM
	movwf   freq_3            ; Save osc frequency
  
	CALLX    read_EEPROM       ; Get next byte
	movwf   freq_2            ; Save it
 
	CALLX    read_EEPROM       ; Get the third byte
	movwf   freq_1
	
	CALLX    read_EEPROM       ; Get the third byte
	movwf   freq_0
	
 		
	CALLX    read_EEPROM       ; Get the fourth byte
	movwf   bli_bls     ; Save it
	
	CALLX    read_EEPROM       ; Get the fourth byte
	movwf   add_sous_affi      ;Save it
	
	CALLX    read_EEPROM       ; Get the  byte
	movwf   band      				;Save it

	Bank0		;
	
	return                  ;end of getfee routine
;*******************************************************************************                
T_num		; detection de la touche enfonc dans le mode "NUM"rique
;********************************************************************************
  
     	
 	movwf	numero_touche 
 							
 	movlw	0X00			
	XORWF	nombre_appui,w	;si = 0 = premier appui = premier chiffre de
	PCLAX	dix_mega_0
	btfsc	STATUS,B_Z      ;la freq rentre par le clavier numerique
	GOTSX 	dix_mega_0      ;si z = a 1 sauter et voir pour 0x01

	movlw	0X01		    ;	
	XORWF	nombre_appui,w	;    
	PCLAX	un_mega_0
	btfsc	STATUS,B_Z
	GOTSX 	un_mega_0
	        
		
		
	movlw	0X02		
	XORWF	nombre_appui,w	
	PCLAX	cent_kilo_0
	btfsc	STATUS,B_Z
	GOTSX 	cent_kilo_0
		        
	movlw	0X03		
	XORWF	nombre_appui,w	;    
	PCLAX	dix_kilo_0
	btfsc	STATUS,B_Z
	GOTSX 	dix_kilo_0       
	
 	movlw	0X04			
	XORWF	nombre_appui,w	
	PCLAX	un_kilo_0
	btfsc	STATUS,B_Z
	GOTSX 	un_kilo_0 
	
;******************************************************************************	

dix_mega_0
	CALLX	LCDcls1
	movlw	0x08	;valeur utilise par le s/p lsb_usb pour affichier sur le
	movwf	band	;lcd LSB ou USB et aussi si l'osc est en supra ou en infradyne
 
	MOVLW 	0X00    	
	XORWF 	numero_touche,w		;touche 0
	PCLAX	dix_mega_1
	btfss	STATUS,B_Z		;;si z = 1 on saute
	GOTSX	dix_mega_1
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0X00
	movwf	freq_02
		movlw 	0X00
	movwf	freq_01
		movlw 	0X00
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
						
	movlw   0X80              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   '0'               ;ECRIRE 
	
	GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f	
	;	GOTOX	int_retour
	 	      
dix_mega_1
	movlw	0x1c	;valeur utilise par le s/p lsb_usb pour affichier sur le
	movwf	band	;lcd LSB ou USB et aussi si l'osc est en supra ou en infradyne
 


	MOVLW 	0X01    	
	XORWF 	numero_touche,w
	PCLAX	vingt_mega		;si z = 1 on saute
	btfss	STATUS,B_Z
	GOTSX	vingt_mega
	
		movlw 	0X00
	movwf	freq_03
		movlw 	0X0f
	movwf	freq_02
		movlw 	0X42
	movwf	freq_01
		movlw 	0X40
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
	
	movlw   0X80              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   '1'			 ;ECRIRE 
		GOTOX	affi_num_touche_lcd              
;	movwf   LCD_char          ; 
	;CALLX    data2LCD	
	;incf 	nombre_appui,f
		
;	GOTOX	int_retour
	
vingt_mega
	movlw	0x1c	;valeur utilise par le s/p lsb_usb pour affichier sur le
	movwf	band	;lcd LSB ou USB et aussi si l'osc est en supra ou en infradyne
 
		;movlw	0x00			;si =0 alors supradyne si = 1 alors infradyne
	;movwf	add_sous_affi	;variable pour OSC
	MOVLW 	0X02    
	XORWF 	numero_touche,w
	PCLAX	trente_mega
	btfss	STATUS,B_Z
	GOTSX	trente_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0X1e
	movwf	freq_02
		movlw 	0x84
	movwf	freq_01
		movlw 	0X80
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
	movlw   0X80              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   '2'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
	;incf 	nombre_appui,f
	;GOTOX	int_retour
	
trente_mega	

	movlw	0x1c	;valeur utilise par le s/p lsb_usb pour affichier sur le
	movwf	band	;lcd LSB ou USB et aussi si l'osc est en supra ou en infradyne
 
		;movlw	0x00			;si =0 alors supradyne si = 1 alors infradyne
	;movwf	add_sous_affi	;variable pour OSC 
	MOVLW 	0X03    
	XORWF 	numero_touche,w
	PCLAX	quarante_mega
	btfss	STATUS,B_Z		;;si z = 1 on saute
	GOTSX	quarante_mega	

			movlw 	0X00
	movwf	freq_03
		movlw 	0X2d
	movwf	freq_02
		movlw 	0Xc6
	movwf	freq_01
		movlw 	0Xc0
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
	movlw   0X80              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   '3'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;		incf 	nombre_appui,f
;	GOTOX	int_retour
	
quarante_mega

	movlw	0x1c	;valeur utilise par le s/p lsb_usb pour affichier sur le
	movwf	band	;lcd LSB ou USB et aussi si l'osc est en supra ou en infradyne
 
	;	movlw	0x00			;si =0 alors supradyne si = 1 alors infradyne
;	movwf	add_sous_affi	;variable pour OSC 
	MOVLW 	0X04   
	XORWF 	numero_touche,w
	PCLAX	cinquante_mega
	btfss	STATUS,B_Z
	GOTSX	cinquante_mega	
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x3d
	movwf	freq_02
		movlw 	0X09
	movwf	freq_01
		movlw 	0X00
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
	movlw   0X80              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   '4'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
		;incf 	nombre_appui,f
	;GOTOX	int_retour	
		
cinquante_mega	

	;MOVLW 	0X05   
	;XORWF 	numero_touche,w
	;PCLAX	fin_dix
	;btfss	STATUS,B_Z
;	GOTSX	fin_dix	
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x4c
	movwf	freq_02
		movlw 	0X4b
	movwf	freq_01
		movlw 	0X40
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
	movlw   0X80              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   ' '               ; 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   '5'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
	;incf 	nombre_appui,f
;fin_dix		
	;GOTOX	int_retour			
	
;**************************************************************************************
un_mega_0
	
	MOVLW 	0X00    	
	XORWF 	numero_touche,w		;touche 0
	PCLAX	un_mega
	btfss	STATUS,B_Z
	GOTSX	un_mega
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '0'               ;ECRIRE 
	GOTOX	affi_num_touche_lcd	
;	movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f
		;GOTOX	int_retour
			
un_mega   	
	MOVLW 	0X01    	
	XORWF 	numero_touche,w		;touche 1
	PCLAX	deux_mega
	btfss	STATUS,B_Z
	GOTSX	deux_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x01
	movwf	freq_02
		movlw 	0X86
	movwf	freq_01
		movlw 	0Xa0
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
	
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   '1'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD	
;	incf 	nombre_appui,f		
;	GOTOX	int_retour

	
deux_mega   	
	MOVLW 	0X02    	
	XORWF 	numero_touche,w		;touche 2
	PCLAX	trois_mega
	btfss	STATUS,B_Z
	GOTSX	trois_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x03
	movwf	freq_02
		movlw 	0X0d
	movwf	freq_01
		movlw 	0X40

	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '2'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f	
;	GOTOX	int_retour
	
trois_mega   	
	MOVLW 	0X03    	
	XORWF 	numero_touche,w		;touche 3
	PCLAX	quatre_mega
	btfss	STATUS,B_Z
	GOTSX	quatre_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x04
	movwf	freq_02
		movlw 	0X93
	movwf	freq_01
		movlw 	0Xe0
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
	
	
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '3'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f	
	;GOTOX	int_retour
	
quatre_mega   	
	MOVLW 	0X04    	
	XORWF 	numero_touche,w		;touche 4
	PCLAX	cinq_mega
	btfss	STATUS,B_Z
	GOTSX	cinq_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x06
	movwf	freq_02
		movlw 	0X1a
	movwf	freq_01
		movlw 	0X80
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '4'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
cinq_mega   	
	MOVLW 	0X05   	
	XORWF 	numero_touche,w		;touche 5
	PCLAX	six_mega
	btfss	STATUS,B_Z
	GOTSX	six_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x07
	movwf	freq_02
		movlw 	0Xa1
	movwf	freq_01
		movlw 	0X20
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '5'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
	
six_mega   	
	MOVLW 	0X06    	
	XORWF 	numero_touche,w		;touche 6
	PCLAX	sept_mega
	btfss	STATUS,B_Z
	GOTSX	sept_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x09
	movwf	freq_02
		movlw 	0X27
	movwf	freq_01
		movlw 	0Xc0
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '6'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
sept_mega   	
	MOVLW 	0X07   	
	XORWF 	numero_touche,w		;touche 7
	PCLAX	huit_mega
	btfss	STATUS,B_Z
	GOTSX	huit_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x0a
	movwf	freq_02
	movlw 	0Xae
	movwf	freq_01
		movlw 	0X60
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '7'               ;ECRIRE
		GOTOX	affi_num_touche_lcd 
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f
		
	GOTOX	int_retour				
	
huit_mega   	
	MOVLW 	0X08    	
	XORWF 	numero_touche,w		;touche 8
	PCLAX	neuf_mega
	btfss	STATUS,B_Z
	GOTSX	neuf_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x0c
	movwf	freq_02
		movlw 	0X35
	movwf	freq_01
		movlw 	0X00
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '8'               ;ECRIRE 
	GOTOX	affi_num_touche_lcd	
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
	
neuf_mega   	
	;MOVLW 	0X09    	
	;XORWF 	numero_touche,w		;touche 9
	;PCLAX	dix_mega
	;btfsc	STATUS,B_Z
	;GOTOX	dix_mega
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x0d
	movwf	freq_02
		movlw 	0Xbb
	movwf	freq_01
		movlw 	0Xa0
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X82              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '9'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour	
	
;********************************************************************************************	
	
cent_kilo_0	

	MOVLW 	0X00    	
	XORWF 	numero_touche,w		;touche 0
	PCLAX	un_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	un_cent_kilo
	
;	CALLX	addition_valeur_des touches 
			
		movlw   0X84             ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '0'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f
	;GOTOX	int_retour	
			
un_cent_kilo 
  	
	MOVLW 	0X01    	
	XORWF 	numero_touche,w		;touche 1
	PCLAX	deux_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	deux_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X27
	movwf	freq_01
		movlw 	0X10
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
			movlw   0X84             ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '1'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD	
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
deux_cent_kilo  	
	MOVLW 	0X02   	
	XORWF 	numero_touche,w		;touche 2
	PCLAX	trois_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	trois_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X4e
	movwf	freq_01
		movlw 	0X20
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '2'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
trois_cent_kilo   	
	MOVLW 	0X03    	
	XORWF 	numero_touche,w		;touche 3
	PCLAX	quatre_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	quatre_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X75
	movwf	freq_01
		movlw 	0X30
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84             ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '3'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
	;CALLX    data2LCD
;	incf 	nombre_appui,f
;	GOTOX	int_retour
	
quatre_cent_kilo   	
	MOVLW 	0X04    	
	XORWF 	numero_touche,w		;touche 4
	PCLAX	cinq_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	cinq_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X9c
	movwf	freq_01
		movlw 	0X40
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '4'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
	
cinq_cent_kilo  	
	MOVLW 	0X05   	
	XORWF 	numero_touche,w		;touche 5
	PCLAX	six_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	six_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0Xc3
	movwf	freq_01
		movlw 	0X50
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '5'               ;ECRIRE 
	GOTOX	affi_num_touche_lcd	
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
six_cent_kilo  	
	MOVLW 	0X06    	
	XORWF 	numero_touche,w		;touche 6
	PCLAX	sept_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	sept_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0Xea
	movwf	freq_01
		movlw 	0X60
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '6'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
	
sept_cent_kilo   	
	MOVLW 	0X07   	
	XORWF 	numero_touche,w		;touche 7
	PCLAX	huit_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	huit_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x01
	movwf	freq_02
		movlw 	0X11
	movwf	freq_01
		movlw 	0X70
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84             ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '7'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour				
	
huit_cent_kilo   	
	MOVLW 	0X08    	
	XORWF 	numero_touche,w		;touche 8
	PCLAX	neuf_cent_kilo
	btfss	STATUS,B_Z
	GOTSX	neuf_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x01
	movwf	freq_02
		movlw 	0X38
	movwf	freq_01
		movlw 	0X80
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '8'               ;ECRIRE 
	GOTOX	affi_num_touche_lcd	
;	movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
neuf_cent_kilo   	
	;MOVLW 	0X09    	
	;XORWF 	numero_touche,w		;touche 1
	;PCLAX	dix_cent_kilo
;	btfsc	STATUS,B_Z
;	GOTOX	dix_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x01
	movwf	freq_02
		movlw 	0X5f
	movwf	freq_01
		movlw 	0X90
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X84              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '9'               ;ECRIRE 
	GOTOX	affi_num_touche_lcd	
;	movwf   LCD_char          ; 
	;CALLX    data2LCD
;	incf 	nombre_appui,f
		
;	GOTOX	int_retour
		
;************************************************************************************

dix_kilo_0	
	
	MOVLW 	0X00    	
	XORWF 	numero_touche,w		;touche 0
	PCLAX	dix_kilo
	btfss	STATUS,B_Z	;si z =1 sauter et voir pour 0x01
	GOTSX	dix_kilo
	;CALLX	addition_valeur_des touches 
		
		movlw   0X85              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	
	movlw   '0'               ;ECRIRE 
	GOTOX	affi_num_touche_lcd	
	;movwf   LCD_char          ; 
;	CALLX    data2LCD		
	;	incf 	nombre_appui,f
	;	GOTOX	int_retour	
		
dix_kilo 
  	
	MOVLW 	0X01    	
	XORWF 	numero_touche,w		;touche 1
	PCLAX	vingt_kilo
	btfss	STATUS,B_Z
	GOTSX	vingt_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00;10
	movwf	freq_02
		movlw 	0X03;e2
	movwf	freq_01
		movlw 	0Xe8;a9
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X85              ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '1'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
vingt_kilo  	
	MOVLW 	0X02   	
	XORWF 	numero_touche,w		;touche 2
	PCLAX	trente_kilo
	btfss	STATUS,B_Z
	GOTSX	trente_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0x07
	movwf	freq_01
		movlw 	0Xd0
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
		
		movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '2'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f		
	;GOTOX	int_retour
	
trente_kilo   	
	MOVLW 	0X03    	
	XORWF 	numero_touche,w		;touche 3
	PCLAX	quarante_kilo
	btfss	STATUS,B_Z
	GOTSX	quarante_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X0b
	movwf	freq_01
		movlw 	0Xb8
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
			movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	
	movlw   '3'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
	;CALLX    data2LCD
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
	
quarante_kilo   	
	MOVLW 	0X04    	
	XORWF 	numero_touche,w		;touche 4
	PCLAX	cinquante_kilo
	btfss	STATUS,B_Z
	GOTSX	cinquante_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X0f
	movwf	freq_01
		movlw 	0Xa0
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
			movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '4'               ;ECRIRE
	GOTOX	affi_num_touche_lcd	 
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
	
cinquante_kilo  	
	MOVLW 	0X05   	
	XORWF 	numero_touche,w		;touche 5
	PCLAX	soixante_kilo
	btfss	STATUS,B_Z
	GOTSX	soixante_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X13
	movwf	freq_01
		movlw 	0X88
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
			movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '5'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
	
soixante_kilo  	
	MOVLW 	0X06    	
	XORWF 	numero_touche,w		;touche 6
	PCLAX	soixantedix_kilo
	btfss	STATUS,B_Z
	GOTSX	soixantedix_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X17
	movwf	freq_01
		movlw 	0X70
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
			movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '6'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
;	incf 	nombre_appui,f		
	;GOTOX	int_retour
	
soixantedix_kilo   	
	MOVLW 	0X07   	
	XORWF 	numero_touche,w		;touche 7
	PCLAX	quatrevingt_kilo
	btfss	STATUS,B_Z
	GOTSX	quatrevingt_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X1b
	movwf	freq_01
		movlw 	0X58
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
			movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '7'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD
	;incf 	nombre_appui,f		
;	GOTOX	int_retour				
	
quatrevingt_kilo   	
	MOVLW 	0X08    	
	XORWF 	numero_touche,w		;touche 8
	PCLAX	quatrevingtdix_kilo
	btfss	STATUS,B_Z
	GOTSX	quatrevingtdix_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X1f
	movwf	freq_01
		movlw 	0X40
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches
			movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '8'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
	;movwf   LCD_char          ; 
	;CALLX    data2LCD
	;incf 	nombre_appui,f		
	;GOTOX	int_retour
	
quatrevingtdix_kilo   	
	;MOVLW 	0X09    	
	;XORWF 	numero_touche,w		;touche 1

			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X23
	movwf	freq_01
		movlw 	0X28
	movwf	freq_00
	
	CALLX	addition_valeurs_des_touches 
			movlw   0X85            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '9'               ;ECRIRE 
		GOTOX	affi_num_touche_lcd
;	movwf   LCD_char          ; 
;	CALLX    data2LCD	
;	incf 	nombre_appui,f		
;	GOTOX	int_retour
		
;************************************************************************************		
un_kilo_0	
		
	MOVLW 	0X00    	
	XORWF 	numero_touche,w		;touche 0
	PCLAX	un_kilo
	btfss	STATUS,B_Z
	GOTSX	un_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X00
	movwf	freq_01
		movlw 	0X00
	movwf	freq_00	
	
		CALLX	fin_touches
			
		movlw   0X86            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '0'               ;ECRIRE 
		GOTOX	affi_num_touche
		
	;movwf   LCD_char          ; 
	;CALLX    data2LCD	

	;GOTOX	int_retour
			
un_kilo 
  	
	MOVLW 	0X01    	
	XORWF 	numero_touche,w		;touche 1
	PCLAX	deux_kilo
	btfss	STATUS,B_Z
	GOTSX	deux_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0x00
	movwf	freq_01
		movlw 	0X64
	movwf	freq_00
	CALLX	fin_touches
	
		movlw   0X86            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	movlw   '1'  ;ECRIRE 
	   	GOTOX	affi_num_touche        
	;movwf   LCD_char          ; 
	;CALLX    data2LCD	
		
	;GOTOX	int_retour
	
deux_kilo  	
	MOVLW 	0X02   	
	XORWF 	numero_touche,w		;touche 2
	PCLAX	trois_kilo
	btfss	STATUS,B_Z
	GOTSX	trois_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0x00
	movwf	freq_01
		movlw 	0Xc8
	movwf	freq_00
	CALLX	fin_touches
	
			movlw   0X86            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 				
	movlw   '2'               ;ECRIRE 
		GOTOX	affi_num_touche
;	movwf   LCD_char          ; 
;	CALLX    data2LCD	

;	GOTOX	int_retour
	
trois_kilo   	
	MOVLW 	0X03    	
	XORWF 	numero_touche,w		;touche 3
	PCLAX	quarte_kilo
	btfss	STATUS,B_Z
	GOTSX	quarte_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X01
	movwf	freq_01
		movlw 	0X2c
	movwf	freq_00
		CALLX	fin_touches
	
		movlw   0X86           ;POSITION OU COMMENCER A ECRIRE sur le lcd 				
	CALLX    cmnd2LCD 		
	movlw   '3'               ;ECRIRE 
	GOTOX	affi_num_touche
	;movwf   LCD_char          ; 
	;CALLX    data2LCD	
	
	;GOTOX	int_retour
	
quarte_kilo   	
	MOVLW 	0X04    	
	XORWF 	numero_touche,w		;touche 4
	PCLAX	cinq_kilo
	btfss	STATUS,B_Z
	GOTSX	cinq_kilo
	
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X01
	movwf	freq_01
		movlw 	0X90
	movwf	freq_00
	
		CALLX	fin_touches
		
			movlw   0X86           ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '4'  ;ECRIRE 
	   	GOTOX	affi_num_touche
	   	
	;movwf   LCD_char          ; 
	;CALLX    data2LCD		
	;GOTOX	int_retour
	
cinq_kilo  	
	MOVLW 	0X05   	
	XORWF 	numero_touche,w		;touche 5
	PCLAX	six_kilo
	btfss	STATUS,B_Z
	GOTSX	six_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X01
	movwf	freq_01
		movlw 	0Xf4
	movwf	freq_00
		
				
		CALLX	fin_touches
				movlw   0X86           ;POSITION OU COMMENCER A ECRIRE sur le lcd 		
	CALLX    cmnd2LCD 	

	movlw   '5'               ;ECRIRE 
		GOTOX	affi_num_touche

	
six_kilo  	
	MOVLW 	0X06    	
	XORWF 	numero_touche,w		;touche 6
	PCLAX	sept_kilo
	btfss	STATUS,B_Z
	GOTSX	sept_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X02
	movwf	freq_01
		movlw 	0X58
	movwf	freq_00
	
		CALLX	fin_touches
				movlw   0X86            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 
		movlw   '6' 	 ;ECRIRE 
	GOTOX	affi_num_touche
             

	
sept_kilo   	
	MOVLW 	0X07   	
	XORWF 	numero_touche,w		;touche 7
	PCLAX	huit_kilo
	btfss	STATUS,B_Z
	GOTSX	huit_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X02
	movwf	freq_01
		movlw 	0Xbc
	movwf	freq_00
	
	CALLX	fin_touches
				movlw   0X86            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '7'               ;ECRIRE 
	GOTOX	affi_num_touche
 		
	
huit_kilo   	
	MOVLW 	0X08    	
	XORWF 	numero_touche,w		;touche 8
	PCLAX	neuf_kilo
	btfss	STATUS,B_Z
	GOTSX	neuf_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X03
	movwf	freq_01
		movlw 	0X20
	movwf	freq_00
		CALLX	fin_touches
				movlw   0X86            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 	
	
	movlw   '8' ;ECRIRE 
	 GOTOX	affi_num_touche            

neuf_kilo   	
	;MOVLW 	0X09    	
	;XORWF 	numero_touche,w		;touche 9
	;;PCLAX	dix_cent_kilo
;;	btfsc	STATUS,B_Z
;;	GOTOX	dix_cent_kilo
	
			movlw 	0X00
	movwf	freq_03
		movlw 	0x00
	movwf	freq_02
		movlw 	0X03
	movwf	freq_01
		movlw 	0X84
	movwf	freq_00
		CALLX	fin_touches
	
	movlw   0X86            ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 		
	movlw   '9'               ;ECRIRE 
	GOTOX	affi_num_touche

;*****************************************************************************
fin_touches	
	movlw	0x01			;indique que la touche "NUM"rique N'EST PLUS ACTIVE
	movwf	tester_entre	; les 5 chiffres ont ete envoyer
	
	CALLX	addition_valeurs_des_touches 
	 
	 	clrf	nombre_appui	 	
	 	CALLX    MEGAH	
	 	CALLX	lsb_usb
		CALLX	write3	
	return
;*******************************************************************************	
addition_valeurs_des_touches   ;addition de la valeur donnees par les
							 ;touches resultat dans freq_0,freq_1,freq_2,freq_3 
		 movf  freq_00,w           ; Get low byte of the increment
        addwf   freq_0,f       ; Add it to the low byte of freq
        
        PCLAX	IFadd01
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFadd01             ; No, add next byte
        
       PCLAX    IFadd01 
        incfsz  freq_01,f       ; Ripple carry up to the next byte
        GOTSX    IFadd01             ; No new carry, add next byte
        
        PCLAX	IFadd01 
        incfsz  freq_02,f       ; Ripple carry up to the next byte
        GOTSX    IFadd01             ; No new carry, add next byte
        
        incf    freq_03,f       ; Ripple carry up to the highest byte
IFadd01
        movf   freq_01,w           ; Get the next increment byte
        addwf   freq_1,f       ; Add it to the next higher byte
        
        PCLAX   IFadd02         
        btfss   STATUS,B_C         ; Any carry?
        GOTSX   IFadd02             ; No, add next byte
        
        PCLAX	IFadd02
        incfsz  freq_02,f       ; Ripple carry up to the next byte
        GOTSX   IFadd02             ; No new carry, add next byte
        
        incf    freq_03,f       ; Ripple carry up to the highest byte
IFadd02
        movf   freq_02,w           ; Get the next to most significant increment
        addwf   freq_2,f       ; Add it to the freq byte
        PCLAX	IFadd03  
        btfss   STATUS,B_C         ; Any carry?
        GOTSX   IFadd03             ; No, add last byte
        
        incf    freq_03,f       ; Ripple carry up to the highest byte
IFadd03
        movf   freq_03,w           ; Get the most significant increment byte
        addwf   freq_3,f       ; Add it to the most significant freq
                 ; Finished
         return							 
						          

;***************************************************	
affi_num_touche_lcd
	incf 	nombre_appui,f
affi_num_touche	
	movwf   LCD_char          ; 
	CALLX    data2LCD

	GOTOX	int_retour		
;********************************************************************************
CHUT	;S/P faire le silence pour eviter les bruits des commutations transmis par l'ampli bf 
	return

	movlw 	0x01 				;inverser le 1er bit 
	xorwf	chut,f 	;si = 0 = chut hors service
	
	;movlw	0x70;71
	;movwf	adresse
	
	movlw	0X00		 
	SUBWF	chut,w	
	PCLAX	aller_80
	btfss	STATUS,B_Z	;si  0X00 - chut = 1 alors
	GOTSX	aller_80	
	movlw	0X80		;mettre le bit 7 du pcf8574 a 1 (p8)
	CALLX	SOUS_U17					;saut aller_80_
	return
	
aller_80
	movlw	0x80
	CALLX	ADD_U17
	return

;*******************************************************************************

;=============================================================================
;
;	S-Meter Routine
;
;	The receiver AGC line and BIAS (Threhold voltage) is sampled by the A to D 
;	and the difference is ranged by comparison with the "S" constants. These 
; 	are user definable to allow the S-Meter to be calibrated to the AGC law, 
;	which could be non linear.
;
;	The S point value is stored in S_point. This is used to count the number
;	of blocks written to display line 2.
;
;=============================================================================

S_Meter
	MOVLW   0X7f ;ff
	MOVWF   count1	
	MOVLW   0Xff
	MOVWF   count2	
	

	Bank0
;; Measure AGC System Bias Voltage
;	movlw	b'10001001'	; Channel 1,xtal=20mgh,go=0,adon=1
;	movwf	ADCON0
;	bsf	ADCON0,GO	; Start A to D
;	CALLX	wait_16ms	; Pause

Wait_BIAS
	;PCLAX	Wait_BIAS
	;btfsc	ADCON0,GO 	;	
;	GOTSX	Wait_BIAS
;	movf	ADRESH,w	; Save A to D result
;	movwf	AGC_BIAS

		
; Measure AGC Voltage
	movlw	b'10000001'	; Channel 0;RA0/NA0
	movwf	ADCON0
	bsf	ADCON0,GO	; Start A to D
	CALLX	wait_16ms	; Pause
	
Wait_AGC
	PCLAX	Wait_AGC
	btfsc	ADCON0,GO 	;	
	GOTSX	Wait_AGC
	movf	ADRESH,w	; Save A to D result

;;	subwf	AGC_BIAS,w	; Get difference between AGC voltage and BIAS
		             	; AGC Threshold voltage
				
	movwf	AGC_level
	;CALLX 	toggle_octet
	clrf	S_point		; Initialise table record counter
	movlw	0xD0;90;C0		; Point to start of bar graph on line 2
	CALLX	cmnd2LCD	
	movlw	0x0D		; Load count with 13 dec (S1..S9+40)
	movwf	count
	bcf	Flag,bar_flag
		
Scale
	movf 	S_point,w	; Point to value in S_Table
;	BSF PCLATH,1
;	bsf	PCLATH,2
		
	CALLX	S_Table		; Retrieve constant
	
;	BcF PCLATH,1
;	BcF PCLATH,2	
	subwf	AGC_level,w	; Compare to A to D sample
	
	PCLAX	blank_bar
	btfss	STATUS,B_C	; Is the result negative?
	GOTSX	blank_bar	; Yes, goto write a blank
	movlw	0xFF		; No, write a block
	CALLX	data2LCD	; Send to display
	incf	S_point,f	; Increment table address
	GOTOX	progress_check	; 
blank_bar
	PCLAX	blank		
	btfsc	Flag,bar_flag	; Check flag to see if is first blank (filled with S number)
	GOTSX	blank		; Yes, digit displayed continue with blanks
	movf	S_point,w	; No, load up current S value
	addlw   0x30		; Add ASCII offset
	bsf		Flag,bar_flag	; Set flag to mark this operation has taken place
	CALLX	data2LCD	; Send S digit to display
	movlw	0x0A		; Are we over S9?
	subwf	S_point,w	; If < 9 answer will be negative
	PCLAX	S9_over
	btfsc	STATUS,B_C
	GOTSX	S9_over		; Yes, Special write required
	GOTOX	Scale		; No, go round again
blank	
	movlw	0x5F		; Move blank symbol to w
	CALLX	data2LCD	; Send to display
	incf	S_point,f	; Increment table address

progress_check
	PCLAX	Scale
	decfsz	count,f		; Count each write
	GOTSX	Scale		; Go round again
	return
S9_over				; Special write case e.g. S9+40
	movlw	0xd9;99;C9		; Point to S9 position
	CALLX	cmnd2LCD	; Send address
	movlw	'9'		; Send a"9"
	CALLX	data2LCD	
	movlw	'+'		; Send a "+"
	CALLX	data2LCD
	movlw	0x09
	subwf	S_point,w	; Get 10dB value
	addlw	0x30		; Add ASCII offset
	CALLX	data2LCD	; Send it
	movlw	'0'		; Send a zero
	CALLX	data2LCD	; 
	return			; Finished, return to caller
	;****************************************************************

S_Table	

	;BSF    PCLATH,1
	BSF    PCLATH,0
	BSF    PCLATH,1
	addwf	PCL,f; Add S_point offset
	retlw	0x06	;S1
	retlw	0x0C	;S2
	retlw	0x12	;S3
	retlw	0x18	;S4
	retlw	0x1E	;S5
	retlw	0x24	;S6
	retlw	0x2A	;S7
	retlw	0x30	;S8
	retlw	0x36	;S9
	retlw	0x40	;S910
	retlw	0x4A	;S920
	retlw	0x54	;S930 
	retlw	0x5E	;S940 Limit
	return		
	
;************************************************************************
toggle_octet		; 			ROUTINES GESTION BUS I2C                   
	;************************************************
	;movlw 	0x8f
;	movwf	AGC_level
	movwf	temp						
						
I2c_Slave_In_Byte
;--------------
		;bcf		STATUS , RP0			; slectionner banque0
		;clrf 	S_point;s_i_byte         						; on vide le tampon d'entre s_i_byte
	
		movlw 	0x08      						; on initialise l'index  8 bits
		movwf	 timer2;	cmpt_bit_i2c					; on charge dans "cmpt_bit_i2c"

;--------------	
I2c_Slave_In_Bit  
;--------------
					;se positionne sur milieu de la data  lire	 
  	    PCLAX	I2c_Slave_In_Bit_0	
		btfss 	temp,7					; on lit la donne prsente sur _SDA  en testant son tat
		GOTSX		I2c_Slave_In_Bit_0	
		GOTOX		I2c_Slave_In_Bit_1
	
I2c_Slave_In_Bit_0
	;--------------
		bcf 		STATUS,B_C      			; on force  un 0
		rrf 	temp1,f;S_point,f;temp,f;	s_i_byte,f       					; on dcale  gauche s_i_byte (le bit est  0) 
		rlf		temp,f			     						; _SDA  est  0, on met le bit de poid faible  0
		GOTOX		I2c_Slave_In_Bit_Next

I2c_Slave_In_Bit_1
	;--------------
		bsf 		STATUS,B_C        			; on force  un 1
		rrf 		temp1,f;S_point,f;temp,f;s_i_byte,f       					; on dcale  gauche s_i_byte (le bit est  1) 
		rlf		temp,f		     						; _SDA  est  1, on met le bit de poid faible  1

I2c_Slave_In_Bit_Next
	;--------------
	  PCLAX		I2c_Slave_In_Bit  
		decfsz 	timer2,f;cmpt_bit_i2c,f          			; on dcremente l'index
		GOTSX      	I2c_Slave_In_Bit    ; on lit le bit suivant tant que l'index n'est pas 0
		movf	temp1,w;S_point,w
		movwf	AGC_level
		return  			     


;****************************************************************************
ATTENU_m
		movlw   0X94           ;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	
;	movlw   '+'               ; 
 ; 	movwf   LCD_char      
;	CALLX    data2LCD
	
	movlw   'a'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'T'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'T'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	return
;*********************************************************************************
ATTENU_p
	movlw   0X94           ;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	
	movlw   'A'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
	return
;************************************************************************************
PRE_m
	movlw   0X90;D7            ;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	
;	movlw   '-'               ; 
  ;	movwf   LCD_char      
;	CALLX    data2LCD
	
	movlw   'p'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'R'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
	movlw   'E'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD 	
	return
;************************************************************************	
PRE_p	

	movlw   0X90;D7            ;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	
	movlw   'P'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
	return
;********************************************************************************
TONE_m 
		movlw   0X98	;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	
		;movlw   '-'               ; 
  	;movwf   LCD_char      
	;CALLX    data2LCD
	
		movlw   't'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
		movlw   'O'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
		movlw   'N'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
	return	
;*******************************************************************************
CAG_m 
		movlw   0X9C	;POSITION OU COMMENCER A ECRIRE sur le lcd 	     
	CALLX    cmnd2LCD 	
	

		movlw   'c'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
		movlw   'A'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
		movlw   'G'               ; 
  	movwf   LCD_char      
	CALLX    data2LCD
	return		
;********************************************************************************
entr_fi
	movlw	0x00
	movwf	entre_fi_qtz	;  si = a  1 on add pas la valeur de la fi sur le lcd
							;et aussi mettre la frequence du filtre a quartz en memoire
	
	movlw   0X80  ;POSITION OU COMMENCER A ECRIRE sur le lcd 
	CALLX    cmnd2LCD 
	movlw   	'D'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
		movlw   'U'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   ' '               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
		movlw   'F'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'L'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'I'                   ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	
	 	movlw   'T'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	   
		movlw   'R'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	   	movlw   'E'            ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
	 	movlw   ' '               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD 
	 	movlw   'Q'               ;ECRIRE 
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'U'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'A'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
		movlw   'R'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD
    	movlw   'T'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD  
    	movlw   'Z'               ;ECRIRE F
	movwf   LCD_char          ; 
	CALLX    data2LCD  
	
	clrf	freq_0	; dans cette config on add pas la valeur de la fi 
	clrf	freq_1  ; avec la valeur donne par les touches 
	clrf	freq_2
	clrf	freq_3 
	
	GOTOX	int_retour		
;********************************************************************************
; *****************************************************************************
; *                                                                           *
; * Purpose:  This routine adds the 32 bit value of fstep to the 32 bit       *
; *           value in freq.  When incrementing, the fstep value is a         *
; *           positive integer.  When decrementing, fstep is the complement   *
; *           of the value being subtracted.                                  *
; *                                                                           *
; *   Input:  The 32 bit values in fstep and freq                             *
; *                                                                           *
; *  Output:  The sum of fstep and freq is stored in DDS_freq.                *
; *                                                                           *
; *****************************************************************************
newdan


	
	movf 	freq_0,w		; Copy current "freq" into "DDS_freq" variables
	movwf	DDS_freq_0		; to allow offset to be added without
	movf 	freq_1,w		; affecting display value which is based
	movwf	DDS_freq_1		; upon content of "freq"
	movf 	freq_2,w		
	movwf	DDS_freq_2		
	movf 	freq_3,w
	movwf	DDS_freq_3
	
		movlw	0X01	
	SUBWF	add_sous_affi,w	
	PCLAX	add_IF
	btfss	STATUS,B_Z	;si 0x01 add_IF ;si= 0 ALORS sub_IF
	GOTSX	add_IF		;osc local en supradyne
	GOTSX	sub_IF		;osc local en infradyne

;**************************************


add_IF
;maintenent il faut savoir si on est en lsb ou en usb pour cela on regarde la variable bli_bls
	;si elle est a 1 on est en usb si elle set a zero on est en lsb	
	movlw	0X01	
		
	SUBWF	bli_bls,w
	PCLAX	pour_mlsb	
	btfss	STATUS,B_Z	
	GOTSX	pour_mlsb	;si  = 1 alors goto pour_lsb
	GOTSX	pour_musb   ;si  = 0 alors goto pour_usb
	
;****************************************************************************************	
pour_mlsb
	Bank1
      movf   IF_Add_0,w          ; Get low byte of the increment
      	Bank0
        addwf   DDS_freq_0,f       ; Add it to the low byte of freq
        
        PCLAX	IFadd1
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFadd1             ; No, add next byte
        
       PCLAX    IFadd1 
        incfsz  DDS_freq_1,f       ; Ripple carry up to the next byte
        GOTSX    IFadd1             ; No new carry, add next byte
        
        PCLAX	IFadd1 
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX    IFadd1             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFadd1
	Bank1
      movf   IF_Add_1,w           ; Get the next increment byte
      	Bank0
        addwf   DDS_freq_1,f       ; Add it to the next higher byte
        
        PCLAX   IFadd2         
        btfss   STATUS,B_C         ; Any carry?
        GOTSX   IFadd2             ; No, add next byte
        
        PCLAX	IFadd2
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX   IFadd2             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFadd2
	Bank1
      movf   IF_Add_2,w          ; Get the next to most significant increment
      	Bank0
        addwf   DDS_freq_2,f       ; Add it to the freq byte
        PCLAX	IFadd3  
        btfss   STATUS,B_C         ; Any carry?
        GOTSX   IFadd3             ; No, add last byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFadd3
	Bank1
      movf   IF_Add_3,w          ; Get the most significant increment byte
      	Bank0
        addwf   DDS_freq_3,f       ; Add it to the most significant freq
                 ; Finished
         return	
 ;*******************************************************************************        
    ;on est en supradyne et en usb 
;*****************************************************************************    
    
pour_musb 

  	Bank1
 	movf   IF_Sub_0,w          ; Get low byte of the increment
 		Bank0
        addwf   DDS_freq_0,f       ; Add it to the low byte of freq
        
        PCLAX	IFadd12
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFadd12             ; No, add next byte
        
       PCLAX    IFadd12 
        incfsz  DDS_freq_1,f       ; Ripple carry up to the next byte
        GOTSX    IFadd12             ; No new carry, add next byte
        
        PCLAX	IFadd12 
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX    IFadd12             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFadd12
	Bank1
      movf  IF_Sub_1,w           ; Get the next increment byte
      	Bank0
        addwf   DDS_freq_1,f       ; Add it to the next higher byte
        
        PCLAX   IFadd22         
        btfss   STATUS,B_C         ; Any carry?
        GOTSX   IFadd22             ; No, add next byte
        
        PCLAX	IFadd22
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX   IFadd22             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFadd22
	Bank1
      movf  IF_Sub_2,w          ; Get the next to most significant increment
      	Bank0
        addwf   DDS_freq_2,f       ; Add it to the freq byte
        PCLAX	IFadd32  
        btfss   STATUS,B_C         ; Any carry?
        GOTSX   IFadd32             ; No, add last byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFadd32
	Bank1
      movf   IF_Sub_3,w          ; Get the most significant increment byte
      	Bank0
        addwf   DDS_freq_3,f       ; Add it to the most significant freq
                 ; Finished
         return	

;==============================================================================   
;
; *****************************************************************************
; *                                                                           *
; * Purpose:  Subtract the IF from freq, checking that it does                *
; *           not go below zero.                                              *
; *                                                                           *
; *   Input:  The values in IF and DDS_freq.                                  *
; *                                                                           *
; *  Output:  The updated value in DDS_freq.                                  *
; *                                                                           *
; *****************************************************************************

sub_IF
	
;maintenent il faut savoir si on est en lsb ou en usb pour cela on regarde la variable bli_bls
	;si elle est a 1 on est en usb si elle set a zero on est en lsb	
	
	movlw	0X01			
	SUBWF	bli_bls,w
	PCLAX	pour_50lsb	
	btfss	STATUS,B_Z	
	GOTSX	pour_50lsb	;si  = 1 alors goto pour_lsb
	GOTSX	pour_50usb   ;si  = 0 alors goto pour_usb

pour_50lsb
        comf    DDS_freq_0,f         ; Subtraction of fstep from
        comf    DDS_freq_1,f         ; freq is done by adding the
        comf    DDS_freq_2,f         ; twos compliment of freq
        comf    DDS_freq_3,f         ; to IF
        PCLAX	IF_comp_done
        incfsz  DDS_freq_0,f         ; Increment last byte
        GOTSX    IF_comp_done         ; Non-zero, continue
        PCLAX   IF_comp_done
        incfsz  DDS_freq_1,f         ; Increment next byte
        GOTSX    IF_comp_done         ; Non-zero, continue
        PCLAX   IF_comp_done
        incfsz  DDS_freq_2,f         ; Increment next byte
        GOTSX    IF_comp_done         ; Non-zero, continue
        incf    DDS_freq_3,f         ; Increment the high byte
IF_comp_done
;
;	Now add the compliment of DDS_freq and IF
;
	Bank1
 
 	movf  IF_Add_0,w           ; Get low byte of the increment
    	Bank0
        addwf   DDS_freq_0,f       ; Add it to the low byte of freq
        PCLAX   IFSadd1 
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFSadd1            ; No, add next byte
        
        PCLAX   IFSadd1 
        incfsz  DDS_freq_1,f       ; Ripple carry up to the next byte
        GOTSX   IFSadd1             ; No new carry, add next byte
        
        PCLAX   IFSadd1 
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX    IFSadd1             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFSadd1
	Bank1
      movf   IF_Add_1,w           ; Get the next increment byte
        	Bank0
        addwf   DDS_freq_1,f       ; Add it to the next higher byte
        
        PCLAX   IFSadd2
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFSadd2            ; No, add next byte
        
        PCLAX   IFSadd2
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX    IFSadd2             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFSadd2
	Bank1
      movf   IF_Add_2,w           ; Get the next to most significant increment
       	Bank0 
        addwf   DDS_freq_2,f       ; Add it to the freq byte
        
        PCLAX   IFSadd3 
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFSadd3            ; No, add last byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFSadd3
	Bank1
      movf   IF_Add_3,w           ; Get the most significant increment byte
      	Bank0  
        addwf   DDS_freq_3,f       ; Add it to the most significant freq
;
;       If the frequency has gone negative, two's compliment it.
;
        btfss   DDS_freq_3,7      ; Is high order frequency byte "negative"?   
        return;goto    IF_exit1          ; No, keep going
IF_comp
        comf    DDS_freq_0,f          ; Yes, two's compliment result
        comf    DDS_freq_1,f          ; to make positive difference
        comf    DDS_freq_2,f          ; 
        comf    DDS_freq_3,f          ; 
        
        PCLAX    IF_exit2    
	incfsz  DDS_freq_0,f          ; Increment last byte
        GOTSX    IF_exit2              ; Non-zero, continue
        
        PCLAX   IF_exit2     
        incfsz  DDS_freq_1,f          ; Increment next byte
        GOTSX   IF_exit2              ; Non-zero, continue
        
        PCLAX   IF_exit2    
        incfsz  DDS_freq_2,f          ; Increment next byte
        GOTSX   IF_exit2              ; Non-zero, continue
        
        incf    DDS_freq_3,f          ; Increment the high byte
IF_exit2
        return                ; Subtract finished
;**************************************************************************** 

pour_50usb 

        comf    DDS_freq_0,f         ; Subtraction of fstep from
        comf    DDS_freq_1,f         ; freq is done by adding the
        comf    DDS_freq_2,f         ; twos compliment of freq
        comf    DDS_freq_3,f         ; to IF
        PCLAX	IF_comp_done5
        incfsz  DDS_freq_0,f         ; Increment last byte
        GOTSX    IF_comp_done5         ; Non-zero, continue
        PCLAX   IF_comp_done5
        incfsz  DDS_freq_1,f         ; Increment next byte
        GOTSX    IF_comp_done5         ; Non-zero, continue
        PCLAX   IF_comp_done5
        incfsz  DDS_freq_2,f         ; Increment next byte
        GOTSX    IF_comp_done5         ; Non-zero, continue
        incf    DDS_freq_3,f         ; Increment the high byte
IF_comp_done5
;
;	Now add the compliment of DDS_freq and IF
;
		Bank1
  
	 movf  IF_Sub_0,w           ; Get low byte of the increment
	Bank0
        addwf   DDS_freq_0,f       ; Add it to the low byte of freq
        PCLAX   IFSadd15 
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFSadd15            ; No, add next byte
        
        PCLAX   IFSadd15 
        incfsz  DDS_freq_1,f       ; Ripple carry up to the next byte
        GOTSX   IFSadd15             ; No new carry, add next byte
        
        PCLAX   IFSadd15 
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX    IFSadd15             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFSadd15

		Bank1
        movf   IF_Sub_1,w           ; Get the next increment byte
        	Bank0
        addwf   DDS_freq_1,f       ; Add it to the next higher byte
        
        PCLAX   IFSadd25
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFSadd25            ; No, add next byte
        
        PCLAX   IFSadd25
        incfsz  DDS_freq_2,f       ; Ripple carry up to the next byte
        GOTSX    IFSadd25             ; No new carry, add next byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFSadd25
    	Bank1
        movf   IF_Sub_2,w           ; Get the next to most significant increment
       	Bank0 
        addwf   DDS_freq_2,f       ; Add it to the freq byte
        
        PCLAX   IFSadd35 
        btfss   STATUS,B_C         ; Any carry?
        GOTSX    IFSadd35            ; No, add last byte
        
        incf    DDS_freq_3,f       ; Ripple carry up to the highest byte
IFSadd35
	    Bank1
        movf   IF_Sub_3,w           ; Get the most significant increment byte
        Bank0
        addwf   DDS_freq_3,f       ; Add it to the most significant freq
;
;       If the frequency has gone negative, two's compliment it.
;
        btfss   DDS_freq_3,7      ; Is high order frequency byte "negative"?   
        return;goto    IF_exit1          ; No, keep going
;;IF_comp
        comf    DDS_freq_0,f          ; Yes, two's compliment result
        comf    DDS_freq_1,f          ; to make positive difference
        comf    DDS_freq_2,f          ; 
        comf    DDS_freq_3,f          ; 
        
        PCLAX    IF_exit25    
	incfsz  DDS_freq_0,f          ; Increment last byte
        GOTSX    IF_exit25              ; Non-zero, continue
        
        PCLAX   IF_exit25     
        incfsz  DDS_freq_1,f          ; Increment next byte
        GOTSX   IF_exit25              ; Non-zero, continue
        
        PCLAX   IF_exit25    
        incfsz  DDS_freq_2,f          ; Increment next byte
        GOTSX   IF_exit25              ; Non-zero, continue
        
        incf    DDS_freq_3,f          ; Increment the high byte
IF_exit25
        return                ; Subtract finished
  
;==========================================================================
;***************************************************************************
; s/p de commutation des pins du pcf8474a et des relais du pass bas	
;***************************************************************************

COM_PASS_PAS
	;return
;bande 3.5
	movlw	0x70
	movwf	adresse
	
	movlw	0X00		
	SUBWF	band,w
	PCLAX	pour_04_pas_bas		
	btfss	STATUS,B_Z	;si  0X00 - band = 0 alors
	GOTSX	pour_04_pas_bas		;sinon saut a pour_04
	movlw	0X08		;mettre le bit 3 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX	i2c_continue    ;il n'y a pas d'autres data	
pour_04_pas_bas
	
;bande 7

	movlw	0X04
	SUBWF	band,w
	PCLAX	pour_08_pas_bas		
	btfss	STATUS,B_Z
	GOTSX	pour_08_pas_bas	
	movlw	0x80		;mettre le bit 7 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX i2c_continue
pour_08_pas_bas	

;bande 10.1
	movlw	0X08
	SUBWF	band,w
	PCLAX	pour_0c_pas_bas		
	btfss	STATUS,B_Z
	GOTSX	pour_0c_pas_bas	
	movlw	0X04		;mettre le bit 2 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX i2c_continue	
pour_0c_pas_bas	

;bande 14
	movlw	0X0c
	SUBWF	band,w
	PCLAX	pour_10_pas_bas	
	btfss	STATUS,B_Z
	GOTSX	pour_10_pas_bas	
	movlw	0X04       ;mettre le bit 2 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX i2c_continue
pour_10_pas_bas
	
;bande 18.068
	movlw	0X10
	SUBWF	band,w
	PCLAX	pour_14_pas_bas	
	btfss	STATUS,B_Z
	GOTSX	pour_14_pas_bas	
	movlw	0X01		;mettre le bit 0 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX i2c_continue
pour_14_pas_bas	

;bande 21
	movlw	0X14
	SUBWF	band,w
	PCLAX	pour_18_pas_bas	
	btfss	STATUS,B_Z
	GOTSX	pour_18_pas_bas	
	movlw	0X01			;mettre le bit 0 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX i2c_continue
pour_18_pas_bas	

;bande 24.890
	movlw	0X18
	SUBWF	band,w
	PCLAX	pour_1c_pas_bas	
	btfss	STATUS,B_Z
	GOTSX	pour_1c_pas_bas
	movlw	0X02     ;mettre le bit 1 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX	i2c_continue
pour_1c_pas_bas
	
;bande 28
	movlw	0X1c
	SUBWF	band,w
	PCLAX	pour_20_pas_bas		
	btfss	STATUS,B_Z
	GOTSX	pour_20_pas_bas	
	movlw	0X02		;mettre le bit 1 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX	i2c_continue
pour_20_pas_bas	

;bande 50
	movlw	0X20
	SUBWF	band,w
	PCLAX	pour_24_pas_bas	
	btfss	STATUS,B_Z
	GOTSX	pour_24_pas_bas	
	movlw	0X40        ;mettre le bit 6 du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX	i2c_continue
pour_24_pas_bas
	
;bande 1.8
	movlw	0X24
	SUBWF	band,w
	PCLAX	pour_nop_pas_bas	
	btfss	STATUS,B_Z
	GOTSX	pour_nop_pas_bas	
		nop 			
	movlw	0X08		;mettre le bit  du pcf8574 a 0 
	movwf	temp1
	CALLX	envoie_i2c
	CALLX	i2c_continue
pour_nop_pas_bas				
	return

;**********************************************************************************
	org	   0x1F00	 
  		DA	0xC4              ; Least significant byteref _osc_0
  		DA	 0x1D              ; Next byteref _osc_1
  		DA	0xF3                ; Next byteref _osc_2 
  	 	DA	0x2A        ; Most significant osc byte_osc_3
  		
	END
	
	
	


	

     
